#include "priv.h"
#include <commctrl.h>

#include "gen.h"			//general purpose plugin Winamp
#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"
#include "ID3V1.h"
#include "Language.h"


// external declaration
extern winampGeneralPurposePlugin plugin;
extern STATGENRESTRUCT gStatGenre[ID3TAG_MAX_GENRE+2];
extern INITSTRUCT gParam;
extern CLanguage    *lng;





#define BMWIDTH				16
#define BMHEIGHT			16
#define RGBREPLACE			0x00FF00FF // solid magenta


/* ************************************************************************ */
CDlgDriveVolume::CDlgDriveVolume(CDlgPlaylist* pThis): CDlg()
{
	m_pThisPlaylist = pThis;
	
	m_nOldSortColumn = 0; 
	m_bSortOrderAscending  = TRUE; 

	m_hIml = NULL;
}




/* ************************************************************** */
BOOL CDlgDriveVolume::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg) 
    {
    case WM_INITDIALOG:
		Init();
		LoadList();
        return FALSE;

	case WM_NOTIFY:
		NotifyHandler(uMsg, wParam, lParam, plugin.hDllInstance);
		return TRUE;

	case WM_DESTROY:
		if(m_hIml)
		{
			ImageList_Destroy(m_hIml);
			m_hIml = NULL;
		}
		return TRUE;
	
	case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDOK:
				case IDCANCEL:
					EndDialog(m_hDlg, (int)TRUE);
					return TRUE;
				
			
			}

    }
    return FALSE;
}

/* ************************************************************** */
void CDlgDriveVolume::Init()
{
	LV_COLUMN lvc;				// list view column structure
	TCHAR szText[80];
	HWND hlvw;

	// ///////////////////////////
	// ///////////////////	
	// Initialize Listview
	hlvw = GetDlgItem(m_hDlg, IDC_LVW_DRIVEVOLUME);
	ListView_SetExtendedListViewStyle(hlvw, LVS_EX_FULLROWSELECT  );

	// Delete columns
	ListView_DeleteColumn(hlvw,0);
	ListView_DeleteColumn(hlvw,1);
	ListView_DeleteColumn(hlvw,2);

	lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvc.pszText = szText;
	
	lvc.iSubItem = 0;							// Volume name
	lvc.fmt = LVCFMT_LEFT;	
	lvc.cx = 130;			
	_tcscpy(szText, GS(LNG_HEADER_VOLUMENAME) );
	ListView_InsertColumn (hlvw, 0, &lvc);

	lvc.iSubItem = 1;							// Type media
	lvc.fmt = LVCFMT_LEFT;	
	lvc.cx = 120;			
	_tcscpy(szText, GS(LNG_HEADER_TYPE) );
	ListView_InsertColumn (hlvw, 1, &lvc);

	lvc.iSubItem = 2;							// Total file
	lvc.fmt = LVCFMT_RIGHT;	
	lvc.cx = 80;			
	_tcscpy(szText, GS(LNG_HEADER_TOTALFILE) );
	ListView_InsertColumn (hlvw, 2, &lvc);


	// ///////////////////	
	// Initialize listview
	m_hIml = ImageList_LoadBitmap(plugin.hDllInstance, MAKEINTRESOURCE(IDB_VOLUMETYPE), BMWIDTH, 0, RGBREPLACE);

	if (m_hIml != NULL)
		ListView_SetImageList(hlvw, m_hIml, LVSIL_SMALL);
	
}


/* ************************************************************** */
void CDlgDriveVolume::LoadList()
{
	LV_ITEM lvi;				// list view item structure
	int i, iItem;
	HWND hlvw;

	hlvw = GetDlgItem(m_hDlg, IDC_LVW_DRIVEVOLUME);

	ListView_DeleteAllItems(hlvw); 

	 // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_PARAM | LVIF_STATE | LVIF_IMAGE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

	iItem = 0;

// ///////////////////////////
// VOLUME INFORMATION
// ///////////////////////////

	int nCountDrive = m_pThisPlaylist->m_minfo.GetCountDriveVolume();

	// Scan all playlist  from memory 
	for (i=0; i < nCountDrive; i++)
	{
		DRIVEVOLUMESTRUCT *dvs=NULL;

		dvs = m_pThisPlaylist->m_minfo.GetInfoDriveVolume(i);

		switch(dvs->nTypeMedia)
		{
			case DRIVE_UNKNOWN:
			case DRIVE_FIXED:
				lvi.iImage = 0;
				break;
			case DRIVE_CDROM:
				lvi.iImage = 1;
				break;
			case DRIVE_REMOVABLE:
				lvi.iImage = 2;
				break;
			case DRIVE_REMOTE:
				lvi.iImage = 3;
				break;
			case DRIVE_RAMDISK:
				lvi.iImage = 4;
				break;
			default: //DRIVE_NO_ROOT_DIR The root directory does not exist. 
				lvi.iImage = 0;
				break;
		}
		
		lvi.iItem    = iItem; 
		lvi.iSubItem = 0;         
		lvi.lParam   = (LPARAM) dvs;    // item data  
		ListView_InsertItem(hlvw, &lvi);  

		iItem++;
	}



}





/****************************************************************************
* 
*    FUNCTION: NotifyHandler(HWND, UINT, UINT, LONG)
*
*    PURPOSE: This function is the handler for the WM_NOTIFY that is 
*    sent to the parent of the list view window.
*
****************************************************************************/
LRESULT CDlgDriveVolume::NotifyHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, HINSTANCE hInst)
{
	LV_DISPINFO FAR * pnmv = (LV_DISPINFO FAR *) lParam;
	NM_LISTVIEW *pNm = (NM_LISTVIEW *)lParam;
	
	
	switch(((LPNMHDR) lParam)->code)
	{
		case LVN_GETDISPINFO:
			// Provide the item or subitem's text, if requested. 
		    if (pnmv->item.mask & LVIF_TEXT) 
			{ 
				TCHAR szCol[255];
				DRIVEVOLUMESTRUCT *dvs = (DRIVEVOLUMESTRUCT *) (pnmv->item.lParam); 


				switch(pnmv->item.iSubItem)
				{
				// Volume name	
				case 0: 
					_stprintf(szCol , _T("%s (%s)"), dvs->szName, dvs->szRootDrive);
					break;
				
				// Type media
				case 1: 
					GetTypeMediaString(dvs->nTypeMedia, szCol);
					break;

				// Total Files	
				case 2: 
					_stprintf(szCol, _T("%d"), dvs->nCountFile); 
					break;
				}

				_tcscpy(pnmv->item.pszText, szCol);
				
			}
			break;

		case LVN_COLUMNCLICK:
			if (m_nOldSortColumn == pNm->iSubItem)
				m_bSortOrderAscending = !m_bSortOrderAscending;

			m_nOldSortColumn = pNm->iSubItem;
			ListView_SortItems(pNm->hdr.hwndFrom, ListViewCompareProc,	this);
			break;

		default:
			break;
	}
	return 0L;
}


/**************************************************************************** */
int CALLBACK CDlgDriveVolume::ListViewCompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort )
{     
	DRIVEVOLUMESTRUCT *pItem1 = (DRIVEVOLUMESTRUCT *) lParam1; 
    DRIVEVOLUMESTRUCT *pItem2 = (DRIVEVOLUMESTRUCT *) lParam2;      
	CDlgDriveVolume *pThis = (CDlgDriveVolume *)lParamSort;
	int iCmp; 

	// Compare the specified column. 
	switch(pThis->m_nOldSortColumn)
	{
		case 0:
			iCmp = lstrcmpi(pItem1->szName, pItem2->szName);
			break;
		
		case 1:
			{
				TCHAR szMedia1[255];
				TCHAR szMedia2[255];

				GetTypeMediaString(pItem1->nTypeMedia, szMedia1);
				GetTypeMediaString(pItem2->nTypeMedia, szMedia2);
				
				iCmp = lstrcmpi(szMedia1, szMedia2);
			}
			break;

		case 2:
			if (pItem1->nCountFile == pItem2->nCountFile)
				iCmp = 0;
			else if (pItem1->nCountFile > pItem2->nCountFile)
				iCmp = 1;
			else
				iCmp = -1;
			break;
		

	}

	if (pThis->m_bSortOrderAscending == FALSE)
		iCmp *= -1;
	
	return (iCmp); 
}


/* ************************************************************** */
void CDlgDriveVolume::GetTypeMediaString(int nTypeMedia, TCHAR *szResult)
{
	switch(nTypeMedia)
	{
		case DRIVE_UNKNOWN:
			_tcscpy(szResult, GS(LNG_DRIVE_UNKNOWN) );
			break;
		case DRIVE_REMOVABLE:
			_tcscpy(szResult, GS(LNG_DRIVE_REMOVABLE) );
			break;
		case DRIVE_FIXED:
			_tcscpy(szResult, GS(LNG_DRIVE_FIXED) );
			break;
		case DRIVE_CDROM:
			_tcscpy(szResult, GS(LNG_DRIVE_CDROM) ) ;
			break;
		case DRIVE_RAMDISK:
			_tcscpy(szResult, GS(LNG_DRIVE_RAMDRIVE) ) ;
			break;
		case DRIVE_REMOTE:
			_tcscpy(szResult, GS(LNG_DRIVE_NETWORK) ) ;
			break;
		default:
			//DRIVE_NO_ROOT_DIR The root directory does not exist. 
			_tcscpy(szResult, GS(LNG_DRIVE_OTHER) );
			break;
	}

}