#include "priv.h"

#include "resource.h"
#include "definition.h"
#include "Dlg.h"
#include "Util.h"
#include "Language.h"


// external declaration
extern INITSTRUCT gParam;
extern CLanguage    *lng;

#define ID_UPDOWNTIME     2000
#define ID_UPDOWNFILESIZE 2001

/* ************************************************************************ */
BOOL CDlgDuplicate::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		InitLanguage();
		LoadParam();
		return FALSE;

	case WM_COMMAND:

		switch (LOWORD(wParam))
		{
		case IDC_DUPLICATE_TIME:
			if (HIWORD(wParam) == BN_CLICKED)
				EnabledUpDownCtrl(IDC_DUPLICATE_TIME, IDC_DUPLICATE_TIME_EDIT, m_hWndUpDownTime);
			return TRUE;

		case IDC_DUPLICATE_FILESIZE:
			if (HIWORD(wParam) == BN_CLICKED)
				EnabledUpDownCtrl(IDC_DUPLICATE_FILESIZE, IDC_DUPLICATE_FILESIZE_EDIT, m_hWndUpDownFilesize);
			return TRUE;
		
		case IDOK:
			if (SaveParam() == TRUE)
				EndDialog(m_hDlg, (int)TRUE);
			return TRUE;

		case IDCANCEL:
			EndDialog(m_hDlg, 0);
			return TRUE;
		}
		
	}
	return FALSE;
}

// **********************************************
void CDlgDuplicate::InitLanguage()
{
	TCHAR szTmp[MAX_PATH];
	
	LNGRESSTRUCT lrs[5] = 
	{
		{ R_DUPLI_FRAME, LNG_DUPLI_FRAME },
		{ IDC_DUPLICATE_FILENAME, LNG_FILENAME },
		{ IDC_DUPLICATE_GENRE, LNG_GENRE_SHORTCUT },
		{ IDC_DUPLICATE_TIME, LNG_DUPLI_DIFF_LENGTH },
		{ IDC_DUPLICATE_FILESIZE, LNG_DUPLI_DIFF_SIZE }
	};
	
	for (int i = 0 ; i<NUMELEMENTS(lrs); i++)
		DlgSetText(m_hDlg, lrs[i].idRes, GS(lrs[i].idLNG) );

	SetWindowText(m_hDlg, GS(LNG_DUPLI_TITLE) );

	_stprintf(szTmp, _T("&%s && %s"), GS(LNG_ARTIST), GS(LNG_TITLE) );
	DlgSetText(m_hDlg, IDC_DUPLICATE_ARTIST_TITLE, szTmp);

}




/* ************************************************************************ */
void CDlgDuplicate::LoadParam()
{
	HWND hWndBuddy;

	for (int i = 0; i < 5; i++)
		CheckDlgButton(m_hDlg, IDC_DUPLICATE_ARTIST_TITLE+i,(gParam.nTypeDuplicate & (1<<i))?BST_CHECKED:BST_UNCHECKED);

	
	// get a handle to the edit "buddy" control
	hWndBuddy = GetDlgItem(m_hDlg, IDC_DUPLICATE_TIME_EDIT);

	// create up-down control
	m_hWndUpDownTime = CreateWindowEx( 
		0L,
		UPDOWN_CLASS,
		_T(""),
		WS_CHILD | WS_BORDER | WS_VISIBLE | UDS_WRAP | UDS_ARROWKEYS |
		UDS_ALIGNRIGHT | UDS_SETBUDDYINT,
		0, 0, 8, 8,
		m_hDlg,
		(HMENU)ID_UPDOWNTIME,
		GetModuleHandle(NULL),		// hinst
		NULL );

	SendMessage( m_hWndUpDownTime, UDM_SETBUDDY, (LONG)hWndBuddy, 0L );
	SendMessage( m_hWndUpDownTime, UDM_SETRANGE, 0L, MAKELONG (60, 0));
	SetDlgItemInt( m_hDlg, IDC_DUPLICATE_TIME_EDIT, gParam.nDuplicateTime, FALSE);

	
	// get a handle to the edit "buddy" control
	hWndBuddy = GetDlgItem(m_hDlg, IDC_DUPLICATE_FILESIZE_EDIT);

	// create up-down control
	m_hWndUpDownFilesize = CreateWindowEx( 
		0L,
		UPDOWN_CLASS,
		_T(""),
		WS_CHILD | WS_BORDER | WS_VISIBLE | UDS_WRAP | UDS_ARROWKEYS |
		UDS_ALIGNRIGHT | UDS_SETBUDDYINT,
		0, 0, 8, 8,
		m_hDlg,
		(HMENU)ID_UPDOWNFILESIZE,
		GetModuleHandle(NULL),		// hinst
		NULL );

	SendMessage( m_hWndUpDownFilesize, UDM_SETBUDDY, (LONG)hWndBuddy, 0L );
	SendMessage( m_hWndUpDownFilesize, UDM_SETRANGE, 0L, MAKELONG (100, 95));
	SetDlgItemInt( m_hDlg, IDC_DUPLICATE_FILESIZE_EDIT, gParam.nDuplicateFilesize, FALSE);

	
	EnabledUpDownCtrl(IDC_DUPLICATE_TIME, IDC_DUPLICATE_TIME_EDIT, m_hWndUpDownTime);
	EnabledUpDownCtrl(IDC_DUPLICATE_FILESIZE, IDC_DUPLICATE_FILESIZE_EDIT, m_hWndUpDownFilesize);
}


/* ************************************************************************ */
BOOL CDlgDuplicate::SaveParam()
{
	BOOL bNoErr;
	int nRet;
	DWORD dwRet;
	int nLow, nHigh;
	TCHAR szMsgErr[256];

	// Save options
	gParam.nTypeDuplicate=0;

	for (int i = 0; i < 5; i++)
		if (IsDlgButtonChecked(m_hDlg,IDC_DUPLICATE_ARTIST_TITLE+i))
			gParam.nTypeDuplicate |= 1<<i;

	if (gParam.nTypeDuplicate ==0)
	{	MessageBox(m_hDlg, GS(LNG_MSG_SELECT_FIELD), GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);
		return FALSE;
	}

	// Test value Time
	if (IsDlgButtonChecked(m_hDlg,IDC_DUPLICATE_TIME))
	{
		nRet = (int)GetDlgItemInt(m_hDlg, IDC_DUPLICATE_TIME_EDIT, &bNoErr, FALSE );

		if(bNoErr==FALSE)
		{	MessageBox(m_hDlg, GS(LNG_MSG_ERR_BAD_NUMBER), GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);
			return FALSE;	
		}
		
		dwRet = SendMessage( m_hWndUpDownTime, UDM_GETRANGE, 0L, 0L);
		nLow = HIWORD(dwRet);
		nHigh= LOWORD(dwRet);

		if (nRet < nLow || nRet > nHigh)
		{
			_stprintf(szMsgErr, GS(LNG_MSG_ERR_BAD_RANGE_TIME), nLow,nHigh);
			MessageBox(m_hDlg, szMsgErr, GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);
			return FALSE;	
		}
		
		gParam.nDuplicateTime = nRet;
	}

	// Test value File size
	if (IsDlgButtonChecked(m_hDlg,IDC_DUPLICATE_FILESIZE))
	{
		nRet = (int)GetDlgItemInt(m_hDlg, IDC_DUPLICATE_FILESIZE_EDIT, &bNoErr, FALSE );

		if(bNoErr==FALSE)
		{	MessageBox(m_hDlg, GS(LNG_MSG_ERR_BAD_NUMBER), GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);
			return FALSE;	
		}
		
		dwRet = SendMessage( m_hWndUpDownFilesize, UDM_GETRANGE, 0L, 0L);
		nLow = HIWORD(dwRet);
		nHigh= LOWORD(dwRet);

		if (nRet < nLow || nRet > nHigh)
		{
			_stprintf(szMsgErr, GS(LNG_MSG_ERR_BAD_RANGE_FILESIZE), nLow,nHigh);
			MessageBox(m_hDlg, szMsgErr, GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);
			return FALSE;	
		}
		
		gParam.nDuplicateFilesize = nRet;
	}

	return TRUE;
}


/* ************************************************************************ */
void CDlgDuplicate::EnabledUpDownCtrl(int nChkID, int nEditID, HWND hwndUpDown)
{
	BOOL bEnabled;
	
	bEnabled = (IsDlgButtonChecked(m_hDlg, nChkID) == BST_CHECKED);	
	EnableWindow(GetDlgItem(m_hDlg, nEditID), bEnabled);
	EnableWindow(hwndUpDown, bEnabled);
}



// //////////////////////////////////
// End part for CDlgDuplicate Dialog
// //////////////////////////////////