#include "priv.h"
#include <commctrl.h>
#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"
#include "Language.h"

// external declaration
extern INITSTRUCT gParam;
extern CLanguage    *lng;

extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);


/* ************************************************************************ */
BOOL CDlgID3ConversionTag::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		CheckDlgButton(m_hDlg, IDC_TAG_CHK_ONLYSELECTED, BST_CHECKED); 
		InitLanguage();
		InitFileList();
		return FALSE;
	
	case WM_CLOSE:
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDOK:
		case IDCANCEL:
			EndDialog(m_hDlg,0);
			return TRUE;
	
		case IDC_TAG_LIST_FILE: 	
			if (HIWORD(wParam) == LBN_SELCHANGE)
				SaveSelectedItem();
			return TRUE;

		case IDC_TAG_BTN_ID3V1TOID3V2:
		case IDC_TAG_BTN_ID3V2TOID3V1:
		case IDC_TAG_BTN_REMOVEID3V1:
		case IDC_TAG_BTN_REMOVEID3V2:
		case IDC_TAG_BTN_REMOVEID3V1_ID3V2:
			{
				int nAction = (LOWORD(wParam) - IDC_TAG_BTN_ID3V1TOID3V2);
				BtnGO(nAction);
				return TRUE;
			}

		case IDC_TAG_CHK_LOG:
			LOGEnable();
			return TRUE;
		case IDC_TAG_BTN_LOG:
			LOGChooseFile();
			return TRUE;
		
		}
	}
	return FALSE;
}


// **********************************************
void CDlgID3ConversionTag::InitLanguage()
{
	LNGRESSTRUCT lrs[7] = 
	{
		{ IDC_TAG_BTN_REMOVEID3V1,			LNG_DLGID3TAG_BTN_REMOVEID3V1    } ,
		{ IDC_TAG_BTN_REMOVEID3V2, 			LNG_DLGID3TAG_BTN_REMOVEID3V2    } ,
		{ IDC_TAG_BTN_REMOVEID3V1_ID3V2, 	LNG_DLGID3TAG_BTN_REMOVEID3V1_2  } ,
		{ IDC_TAG_FRAME_CONVERSION, 		LNG_PAGE_CONVERSIONREMOVE_TITLE  } ,
		{ IDC_TAG_BTN_GO, 					LNG_BTN_GO                       } ,
		{ IDC_TAG_CHK_LOG, 					LNG_DLGID3TAG_TRACE_LOG          } ,
		{ IDC_TAG_CHK_ONLYSELECTED, 		LNG_DLGID3TAG_ONLY_SELECTED_ITEM } ,
	};
	
	for (int i = 0 ; i<NUMELEMENTS(lrs); i++)
		DlgSetText(m_hDlg, lrs[i].idRes, GS(lrs[i].idLNG) );
	
}



/* ************************************************************************ */
void CDlgID3ConversionTag::BtnGO(int nAction)
{
	HCURSOR hcurOld;
	int i, nCount;
	int *pArray = NULL;
	FILE *filelog = NULL;
	BOOL bOK, bSelected, bLog;
	const TCHAR *Msg_label[] = {GS(LNG_FAILED), GS(LNG_SUCCESS)};
	TCHAR szMsg[1024];
	int nIDLog;

	bOK = PrepareItemList(bSelected, nCount, (int**)&pArray, 3+nAction);

	if(!bOK)
		return;

	bOK = PrepareLogFile(bLog, filelog);
	if(!bOK)
		return;
	
	if (bLog)
	{
		nIDLog = LNG_LOG_COPYID3V1_ID3V2;

		switch(nAction)
		{
			case 0: nIDLog = LNG_LOG_COPYID3V1_ID3V2;	break;
			case 1: nIDLog = LNG_LOG_COPYID3V2_ID3V1;	break;
			case 2: nIDLog = LNG_LOG_REMOVEID3V1;		break;
			case 3: nIDLog = LNG_LOG_REMOVEID3V2;		break;
			case 4: nIDLog = LNG_LOG_REMOVEID3V1_ID3V2; break;
		}

		_ftprintf(filelog, GS(nIDLog), nCount);
		_ftprintf(filelog, _T("\n\n"));

	}

	hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	ActiveButton(FALSE);
	

	// Init Progress bar
	HWND hProgress = GetDlgItem(m_hDlg, IDC_PROGRESS1);
	HWND hStatus = GetDlgItem(m_hDlg, IDC_TAG_CONVERSION_STATUS);
	ShowWindow(hStatus, SW_SHOW);
	
	InitProgressBar(hProgress, nCount);

	// Scan file item	
	for (i = 0; i<nCount; i++)
	{
		int iSel = (bSelected ? pArray[i]:i); 
		
		CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETITEMDATA , (WPARAM)iSel, 0);;

		switch(nAction)
		{
			case 0: 
				bOK = CopyID3v1_ID3v2(mp3i); 
				break;
			case 1: 
				bOK = CopyID3v2_ID3v1(mp3i); 
				break;
			case 2: 
				bOK = RemoveID3Tag(mp3i, VERSION_ID3V1); 	
				break;
			case 3: 
				bOK = RemoveID3Tag(mp3i, VERSION_ID3V2); 	
				break;
			case 4: 
				bOK = RemoveID3Tag(mp3i, VERSION_ID3V1+VERSION_ID3V2);
				break;
		}
		
		if (bLog)
		{	_ftprintf(filelog, GS(LNG_LOG_FILE), i+1, Msg_label[bOK], mp3i->GetCFilename());	
			_ftprintf(filelog, _T("\n") );
		}

		SendMessage(hProgress, PBM_STEPIT, 0, 0);
		_stprintf(szMsg, GS(LNG_STATUS_WORKING), i+1, nCount, mp3i->GetCFilename());
		SetWindowText(hStatus, szMsg);
		
	}
	
	ActiveButton(TRUE);
	SetCursor(hcurOld);

	if (bSelected)
		free(pArray);

	if (bLog)
		fclose(filelog);

	// Show resume action in MsgBox
	nIDLog = LNG_RESULT_COPYID3V1_ID3V2;
	switch(nAction)
	{
		case 0: nIDLog = LNG_RESULT_COPYID3V1_ID3V2;	break;
		case 1: nIDLog = LNG_RESULT_COPYID3V2_ID3V1;	break;
		case 2: nIDLog = LNG_RESULT_REMOVEID3V1;		break;
		case 3: nIDLog = LNG_RESULT_REMOVEID3V2;		break;
		case 4: nIDLog = LNG_RESULT_REMOVEID3V1_ID3V2;	break;
	}

	_stprintf(szMsg, GS(nIDLog), nCount);
	MessageBox(m_hDlg, szMsg, GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);

	ShowWindow(hProgress, SW_HIDE);
	ShowWindow(hStatus, SW_HIDE);
}



/* ************************************************************************ */
BOOL CDlgID3ConversionTag::RemoveID3Tag(CMP3Info *mp3i, int nID3Version)
{
	if (!mp3i)	
		return FALSE;

	// ////////////////////
	// Remove TAG ID3v2
	// ////////////////////
	mp3i->DeleteTag(nID3Version);
	if (mp3i->SaveTag() != ID3V2_ERR_OK)
		return FALSE;

	mp3i->ParseFilename();
	return TRUE;
}


/* ************************************************************************ */
BOOL CDlgID3ConversionTag::CopyID3v1_ID3v2(CMP3Info *mp3i)
{
	if (!mp3i)	
		return FALSE;

	if (mp3i->HaveID3v1() == FALSE)
		return FALSE;

	struct ID3V1MP3Info *id3i = mp3i->GetID3v1();

	if (!id3i)
		return FALSE;

	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TPE1); // Artist
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TIT2); // Title, song name
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TALB); // Album
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TYER); // Year
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TCON); // Genre
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_COMM); // Comment
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TRCK); // Track
	
	mp3i->SetTagVersion(VERSION_ID3V1 + VERSION_ID3V2);

	if (mp3i->SaveTag() != ID3V2_ERR_OK)
		return FALSE;
	
	return TRUE;
}
/* ************************************************************************ */
BOOL CDlgID3ConversionTag::CopyID3v2_ID3v1(CMP3Info *mp3i)
{
	if (!mp3i)	
		return FALSE;

	if (mp3i->HaveID3v2() == FALSE)
		return FALSE;

	struct ID3V1MP3Info id3iNew;
	memset(&id3iNew, 0, sizeof(struct ID3V1MP3Info));


	// ////////////////////////////////////
	// Copy information from ID3v2 to ID3v1
	// ////////////////////////////////////
	CMP3StringFrame *sf;

	// Artist
	if (sf = mp3i->GetStringFrame(ID3TAG_TPE1)) {
		t2ancpy((LPSTR)&id3iNew.Artist, sf->GetString(), sizeof(id3iNew.Artist));
		id3iNew.Artist[30]=0;
	}

	// Title,  SongName
	if (sf = mp3i->GetStringFrame(ID3TAG_TIT2)) {
		t2ancpy((LPSTR)&id3iNew.SongName, sf->GetString(), sizeof(id3iNew.SongName));
		id3iNew.SongName[30]=0;
	}

	// Album
	if (sf = mp3i->GetStringFrame(ID3TAG_TALB)) {
		t2ancpy((LPSTR)&id3iNew.Album, sf->GetString(), sizeof(id3iNew.Album));
		id3iNew.Album[30]=0;
	}

	// Year
	if (sf = mp3i->GetStringFrame(ID3TAG_TYER)) {
		t2ancpy((LPSTR)&id3iNew.Year, sf->GetString(), sizeof(id3iNew.Year));
		id3iNew.Year[4]=0;
	}

	// genre
	if (sf = mp3i->GetStringFrame(ID3TAG_TCON)) {
		id3iNew.Genre = GenreStr2Int(sf->GetString().c_str());
	}

	// Comment
	if (sf = mp3i->GetStringFrame(ID3TAG_COMM)) {
		t2ancpy((LPSTR)&id3iNew.Comment, sf->GetString(), sizeof(id3iNew.Comment));
		id3iNew.Comment[30]=0;
	}
	
	// Track
	if (sf = mp3i->GetStringFrame(ID3TAG_TRCK)) {
		id3iNew.Track = _ttoi(sf->GetString().c_str());
	}

	
	mp3i->SetTagVersion(VERSION_ID3V1 + VERSION_ID3V2);
	mp3i->SetID3v1(&id3iNew);

	if (mp3i->SaveTag() != ID3V2_ERR_OK)
		return FALSE;
	
	return TRUE;
}

/* ************************************************************************ */
void CDlgID3ConversionTag::ActiveButton(BOOL bEnabled)
{
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_ID3V1TOID3V2), bEnabled);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_ID3V2TOID3V1), bEnabled);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_REMOVEID3V1), bEnabled);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_REMOVEID3V2), bEnabled);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_REMOVEID3V1_ID3V2), bEnabled);
}


/* ************************************************************************ */
void CDlgID3ConversionTag::CopyInfoID3v1_ID3v2(CMP3Info *mp3i, struct ID3V1MP3Info *id3i, int fid)
{
	CMP3Frame *mp3f = NULL;
	tstring str = _T("");

	// ////////////////////////////////////
	// Copy information from ID3v1 to ID3v2
	// ////////////////////////////////////
	switch(fid)
	{
		case ID3TAG_COMM: // read the Comment
			if (!IsStringEmpty((LPTSTR)id3i->Comment)) 
				a2tcpy(str, (LPTSTR)id3i->Comment);
			break;

		case ID3TAG_TYER: // read the Year
			if (!IsStringEmpty((LPTSTR)id3i->Year)) 
				a2tcpy(str, (LPTSTR)id3i->Year);
			break;

		case ID3TAG_TALB: // read the Album
			if (!IsStringEmpty((LPTSTR)id3i->Album)) 
				a2tcpy(str, (LPTSTR)id3i->Album);
			break;

		case ID3TAG_TPE1: // read the Artist
			if (!IsStringEmpty((LPTSTR)id3i->Artist)) 
				a2tcpy(str, (LPTSTR)id3i->Artist);
			break;

		case ID3TAG_TIT2:  // read the Title:Song name
			if (!IsStringEmpty((LPTSTR)id3i->SongName)) 
				a2tcpy(str, (LPTSTR)id3i->SongName);
			break;

		case ID3TAG_TCON: // read the genre
			a2tcpy(str, (LPTSTR)GenreInt2Str(id3i->Genre));
			break;

		case ID3TAG_TRCK: // read the Track
			if (id3i->Track != 0) 
			{
				TCHAR szTrack[4];
				_stprintf(szTrack, _T("%d"), id3i->Track);
				a2tcpy(str, (LPTSTR)szTrack);
			}

	}


	mp3f = new CMP3StringFrame(fid, str);

	if (mp3f)
		mp3i->ReplaceFrame(mp3f);

	return;
}


