#include "priv.h"
#include <commctrl.h>
#include <winbase.h>		// MoveFile: Rename file

#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"
#include "Language.h"

// external declaration
extern INITSTRUCT gParam;
extern CLanguage    *lng;


/* ************************************************************************ */
BOOL CDlgID3RenameFileFromTag::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		InitLanguage();
		LoadParam();
		InitFileList();
		return FALSE;

	case WM_CLOSE:
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_DESTROY:
		SaveParam();
		return 0;

	case WM_DRAWITEM:
		OwnerDrawBtn((LPDRAWITEMSTRUCT)lParam);		
		return TRUE;

	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDOK:
		case IDCANCEL:
			EndDialog(m_hDlg,0);
			return TRUE;

		case IDC_TAG_OPTIONCAPITALIZE:
			MenuButton(IDC_TAG_OPTIONCAPITALIZE, IDR_MENU1, 5); // 5:Capitalize SubMenu 				
			return FALSE;

		case ID_CAPITALIZE_UNCHANGED:
		case ID_CAPITALIZE_ALLLOWERCASE:
		case ID_CAPITALIZE_ALLUPPERCASE:
		case ID_CAPITALIZE_FIRSTLETTEROFSECTIONUPPERCASE:
		case ID_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE:
			gParam.nOptionCapitalize  = LOWORD(wParam) - ID_CAPITALIZE_UNCHANGED;
			return TRUE;

		case ID_CONVERT_NO:
		case ID_CONVERT_UNDERSCORE_TO_SPACE:
		case ID_CONVERT_SPACE_TO_UNDERSCORE:
			gParam.nOptionConvert  = LOWORD(wParam) - ID_CONVERT_NO;
			return TRUE;

		case IDC_EDITSEPARATOR:	
			if (HIWORD(wParam) == EN_CHANGE) 
				FrmtUpdateSeparator();
			return TRUE;
		

		case IDC_TAG_LIST_FILE:	    
			if (HIWORD(wParam) == LBN_SELCHANGE)
				SaveSelectedItem();
			return TRUE;

		case IDC_TAG_CHCKARTIST_FRMT:
		case IDC_TAG_CHCKTITLE_FRMT:
		case IDC_TAG_CHCKALBUM_FRMT:
		case IDC_TAG_CHCKYEAR_FRMT:
		case IDC_TAG_CHCKCOMMENT_FRMT:
		case IDC_TAG_CHCKTRACK_FRMT:
			if (HIWORD(wParam) == BN_CLICKED)
			{	FrmtCheck2List(LOWORD(wParam));
				FrmtUpdate();
			}
			return TRUE;

		case IDC_TAG_SELECTALL:
		case IDC_TAG_NONE:
			FrmtSelectAll(LOWORD(wParam));
			FrmtUpdate();
			return TRUE;

		case IDC_TAG_MOVEUP:
		case IDC_TAG_MOVEDOWN:
			ListboxMoveItem(GetDlgItem(m_hDlg, IDC_TAG_LIST_FRMT), LOWORD(wParam) == IDC_TAG_MOVEUP);
			FrmtUpdate();
			return TRUE;

		case IDC_TAG_LIST_FRMT:
			if (HIWORD(wParam) == LBN_DBLCLK)
			{	FrmtRemoveCol();
				FrmtUpdate();
			}
			return TRUE;

		case IDC_TAG_CHK_LOG:
			LOGEnable();
			return TRUE;
		case IDC_TAG_BTN_LOG:
			LOGChooseFile();
			return TRUE;
		case IDC_TAG_BTN_GO:
			BtnGO();
			return TRUE;

		}
	}
	return FALSE;
}


// **********************************************
void CDlgID3RenameFileFromTag::InitLanguage()
{
	LNGRESSTRUCT lrs[15] = 
	{
		{ IDC_FILEFORMAT, LNG_DLGID3TAG_FILE_FORMAT } ,
		{ IDC_SEPARATOR, LNG_DLGID3TAG_SEPARATOR } ,
		{ IDC_TAG_CHCKARTIST_FRMT, LNG_ARTIST_SHORTCUT } ,
		{ IDC_TAG_CHCKTITLE_FRMT, LNG_TITLE_SHORTCUT } ,
		{ IDC_TAG_CHCKALBUM_FRMT, LNG_ALBUM_SHORTCUT } ,
		{ IDC_TAG_CHCKYEAR_FRMT, LNG_YEAR_SHORTCUT } ,
		{ IDC_TAG_CHCKCOMMENT_FRMT, LNG_COMMENT_SHORTCUT } ,
		{ IDC_TAG_CHCKTRACK_FRMT, LNG_TRACK_SHORTCUT } ,
		{ IDC_TAG_MOVEUP, LNG_DLGID3TAG_BTN_MOVEUP } ,
		{ IDC_TAG_MOVEDOWN, LNG_DLGID3TAG_BTN_MOVEDOWN } ,
		{ IDC_TAG_SELECTALL, LNG_DLGID3TAG_BTN_SELECTALL } ,
		{ IDC_TAG_NONE, LNG_DLGID3TAG_BTN_NONE } ,
		{ IDC_TAG_BTN_GO, LNG_BTN_GO } ,
		{ IDC_TAG_CHK_LOG, LNG_DLGID3TAG_TRACE_LOG } ,
		{ IDC_TAG_CHK_ONLYSELECTED, LNG_DLGID3TAG_ONLY_SELECTED_ITEM }
	};                                                                        
                                                                              
	for (int i = 0 ; i<NUMELEMENTS(lrs); i++)                                                
		DlgSetText(m_hDlg, lrs[i].idRes, GS(lrs[i].idLNG) ); 
	
}


/* ************************************************************** */
void CDlgID3RenameFileFromTag::LoadParam()
{
	int i;

	CheckDlgButton(m_hDlg, IDC_TAG_CHK_ONLYSELECTED, BST_CHECKED); 
	
	CheckRadioButton(m_hDlg, IDC_TAG_RADIO_ID3V1, IDC_TAG_RADIO_ID3V2, 
		gParam.nRenameFileFromTagWithID3v1 == 1 ? IDC_TAG_RADIO_ID3V1: IDC_TAG_RADIO_ID3V2) ; 
	
	SendDlgItemMessage( m_hDlg, IDC_EDITSEPARATOR, EM_SETLIMITTEXT , 3L, 0L);
	SetWindowText(GetDlgItem(m_hDlg, IDC_EDITSEPARATOR), gParam.szSep);

	// Format Column options 
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_RESETCONTENT, 0, 0);
	
	for (i = 0; i < MAX_COL_FRMT; i++)
	{
		int k = gParam.FrmtCol[i];

		if (k != -1)
		{	
			SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_ADDSTRING, 0, (LPARAM)(LPSTR)m_listFrmt[k]);
			CheckDlgButton(m_hDlg,IDC_TAG_CHCKARTIST_FRMT+k,BST_CHECKED);
		}
	}

	FrmtUpdate();
}


/* ************************************************************** */
void CDlgID3RenameFileFromTag::SaveParam()
{
	int i, k, nCount ;
	TCHAR szTemp[40];
	
	gParam.nRenameFileFromTagWithID3v1 = (IsDlgButtonChecked(m_hDlg, IDC_TAG_RADIO_ID3V1) == BST_CHECKED ? 1 : 0);

	// Save Order format columns
	for (i = 0; i < MAX_COL_FRMT; i++)	
		gParam.FrmtCol[i] = -1;

	nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_GETCOUNT, 0, 0);
	for (i = 0; i < nCount; i++)
	{	
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_GETTEXT, i, (LPARAM) (LPCTSTR)szTemp );
		
		for (k = 0; k < MAX_COL_FRMT; k++)
		{	if (_tcscmp(m_listFrmt[k], szTemp)== 0)
			{	gParam.FrmtCol[i] = k;
				break;
			}				
		}	
	}

}




/* ************************************************************************ */
void CDlgID3RenameFileFromTag::BtnGO()
{
	HCURSOR hcurOld;
	int i, nCount;
	int *pArray = NULL;
	FILE *filelog = NULL;
	BOOL bOK, bSelected, bLog;
	const TCHAR *Msg_label[] = {GS(LNG_FAILED), GS(LNG_SUCCESS)};
	TCHAR szMsg[1024];

	
	SaveParam();

	if (_tcslen(gParam.szSep) == 0)
	{	MessageBox(m_hDlg, GS(LNG_SEPARATORNOTNULL), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
		SetFocus(GetDlgItem(m_hDlg, IDC_EDITSEPARATOR));	
		return;
	}


	bOK = PrepareItemList(bSelected, nCount, (int**)&pArray, 1);
	if(!bOK)
		return;

	bOK = PrepareLogFile(bLog, filelog);
	if(!bOK)
		return;
	
	if (bLog)
	{	_ftprintf(filelog, GS(LNG_MSG_RENAMEFILE_LOG), nCount);
		_ftprintf(filelog, _T("\n\n"));
	}

	hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), FALSE);

	// Init Progress bar
	HWND hProgress = GetDlgItem(m_hDlg, IDC_PROGRESS1);
	HWND hStatus = GetDlgItem(m_hDlg, IDC_TAG_RENAMEFILETAG_STATUS);
	ShowWindow(hStatus, SW_SHOW);
	
	InitProgressBar(hProgress, nCount);

	// Scan file item	
	for (i = 0; i<nCount; i++)
	{
		CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETITEMDATA, (WPARAM)(bSelected ? pArray[i] : i), 0);

		bOK =  RenameFileFromTAG(mp3i);
		
		if (bLog)
		{	_ftprintf(filelog, GS(LNG_LOG_FILE), i+1, Msg_label[bOK], mp3i->GetCFilename());	
			_ftprintf(filelog, _T("\n") );
		}

		SendMessage(hProgress, PBM_STEPIT, 0, 0);
		_stprintf(szMsg, GS(LNG_STATUS_WORKING), i+1, nCount, mp3i->GetCFilename());
		SetWindowText(hStatus, szMsg);

	}
	
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), TRUE);
	SetCursor(hcurOld);

	if (bSelected)
		free(pArray);

	if (bLog)
		fclose(filelog);

	// Show resume action in MsgBox
	_stprintf(szMsg, GS(LNG_MSG_RENAMEFILE_RESULT), nCount);
	MessageBox(m_hDlg, szMsg, GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);

	
	// Refresh list
	InitFileList();
	
	ShowWindow(hProgress, SW_HIDE);
	ShowWindow(hStatus, SW_HIDE);
}


/* ************************************************************************ */
BOOL CDlgID3RenameFileFromTag::RenameFileFromTAG(CMP3Info *mp3i)
{
	int nLen, i, nColFrmt ;
	TCHAR szFormat[1024];
	TCHAR szTmp[1024];
	TCHAR szOutputFile[1024];
	TCHAR *p, *pStart, *pEnd;
	size_t size;

	if (!mp3i) 
		return FALSE;

	// if file did not contain tag exit !!!
	if (mp3i->HaveTag() == FALSE)
		return FALSE;

	szOutputFile[0] = '\0';
	szFormat[0] = '\0';
	nLen = 0;
	
	nColFrmt = SendDlgItemMessage(m_hDlg,IDC_TAG_LIST_FRMT,LB_GETCOUNT,0,0);
	
	if (IsDlgButtonChecked(m_hDlg, IDC_TAG_RADIO_ID3V1) == BST_CHECKED)
	{
		struct ID3V1MP3Info *id3i;
	
		
		id3i = mp3i->GetID3v1();
		
		if(id3i == NULL)
			return FALSE;

		// From ID3v1 Tag
		for (i = 0; i < nColFrmt; i++)
		{
			szTmp[0] = '\0';
			
			switch(gParam.FrmtCol[i])
			{
			case 0:	// Artist
				if (_tcslen((const TCHAR*)&id3i->Artist)==0)
					_tcscpy(szTmp, GS(LNG_UNKOWNARTIST));
				else	
					_tcscpy(szTmp, (const TCHAR*)&id3i->Artist);
				break;
			case 1:	// Title
				if (_tcslen((const TCHAR*)&id3i->SongName)==0)
					_tcscpy(szTmp,  GS(LNG_UNTITLED));
				else	
					_tcscpy(szTmp, (const TCHAR*)&id3i->SongName);
				break;
			case 2:	// Album
				if (_tcslen((const TCHAR*)&id3i->Album)==0)
					_tcscpy(szTmp,  GS(LNG_UNKOWNALBUM));
				else	
					_tcscpy(szTmp,  (const TCHAR*)&id3i->Album);
				break;
			case 3:	// Year
				if (_tcslen((const TCHAR*)&id3i->Year)!=0)
					_tcscpy(szTmp,  (const TCHAR*)&id3i->Year);
				break;
			case 4:	// Comment
				if (_tcslen((const TCHAR*)&id3i->Comment)!=0)
					_tcscpy(szTmp,  (const TCHAR*)&id3i->Comment);
				break;
			case 5:	// Track#
				if (id3i->Track !=0)
					_stprintf(szTmp, _T("%02d"), id3i->Track);

				break;
			}

			
			// Capitalize options
			CapitalizeString(szTmp);
			
			_tcscat(szFormat, szTmp);
			_tcscat(szFormat, gParam.szSep);
		}

	}
	else
	{
		// From ID3v2 Tag
		for (i = 0; i < nColFrmt; i++)
		{
			tstring str = _T("");
			szTmp[0] = '\0';

			switch(gParam.FrmtCol[i])
			{
			case 0:	// Artist
				GetFrameInfo(mp3i, ID3TAG_TPE1, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szTmp, GS(LNG_UNKOWNARTIST));
				else	
					_tcscpy(szTmp, str.c_str());
				break;
			case 1:	// Title
				GetFrameInfo(mp3i, ID3TAG_TIT2, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szTmp, GS(LNG_UNTITLED));
				else	
					_tcscpy(szTmp, str.c_str());
				break;
			case 2:	// Album
				GetFrameInfo(mp3i, ID3TAG_TALB, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szTmp, GS(LNG_UNKOWNALBUM));
				else	
					_tcscpy(szTmp, str.c_str());
				break;
			case 3:	// Year
				GetFrameInfo(mp3i, ID3TAG_TYER, str);
				if (_tcslen(str.c_str())!=0)
					_tcscpy(szTmp, str.c_str());
				break;
			case 4:	// Comment
				GetFrameInfo(mp3i, ID3TAG_COMM, str);
				if (_tcslen(str.c_str())!=0)
					_tcscpy(szTmp, str.c_str());
				break;
			case 5:	// Track#
				GetFrameInfo(mp3i, ID3TAG_TRCK, str);
				if (_tcslen(str.c_str())!=0)
				{
					int nTrack = _ttoi(str.c_str());
					_stprintf(szTmp, _T("%02d"), nTrack);	
				}
				break;
			}

			// Capitalize options
			CapitalizeString(szTmp);

			_tcscat(szFormat, szTmp);
			_tcscat(szFormat, gParam.szSep);
		}

	}
	nLen = _tcslen(szFormat);
	
	// Remove  at the end " - "
	if (nLen > 0)
	{	p = szFormat + nLen - _tcslen(gParam.szSep);
		*p= '\0';
	}

	// Convert options Underscore <--> space
	ConvertString(szFormat);


	ValidateFilename(szFormat);

	pEnd = _tcsrchr(mp3i->GetCFilename(), '\\');
	pStart = (TCHAR*)mp3i->GetCFilename();
	size = pEnd - pStart;
	
	if (size)
	{
		// Copy path
		memcpy(szOutputFile, mp3i->GetCFilename(),  size);
		p= szOutputFile + size;
		*p = '\0';

		// copy new filename
		_tcscat(szOutputFile, _T("\\"));
		_tcscat(szOutputFile, szFormat);
		_tcscat(szOutputFile, _T(".mp3"));
	}
	else
		_stprintf(szOutputFile, _T("%s.mp3"), szFormat);
		

	// Rename MP3 file
	if (MoveFile((LPTSTR)mp3i->GetCFilename(), (LPTSTR)szOutputFile))
	{
		// OK change in memory the new filename
		mp3i->SetFilename(szOutputFile);
		return TRUE;
	}
	else
		return FALSE;


}





/* ************************************************************************ */
void CDlgID3RenameFileFromTag::ValidateFilename(TCHAR *filename)
{
	// Replace invalid character <>:"/\|? by space in the filename
	TCHAR *p = filename;
	int len = _tcslen(filename);

	while (p < filename+len)
	{
		switch(*p)
		{
			case '<':
			case '>':
			case ':':
			case '"':
			case '/':
			case '\\':
			case '|':
			case '?':
				*p = ' ';
				break;
			default:
				break;
		}

		p++;
	}

}
