#include "priv.h"
#include <commctrl.h>

#include "resource.h"
#include "definition.h"
#include "gen.h"
#include "Util.h"
#include "Dlg.h"
#include "Language.h"

// external declaration
extern winampGeneralPurposePlugin plugin;
extern INITSTRUCT gParam;
extern CLanguage    *lng;


// Array OutputFormat 
const TCHAR *CDlgID3Template::m_listFrmt[6];


// Convert table Resource ID <--> Frame ID3v2
const int CDlgID3Template::m_Res2Frame[MAX_RES2FRAME][2] =
{
	{IDC_TAG_EDIT_TITLE2,	ID3TAG_TIT2},
	{IDC_TAG_EDIT_ARTIST2,	ID3TAG_TPE1},
	{IDC_TAG_EDIT_ALBUM2,	ID3TAG_TALB},
	{IDC_TAG_EDIT_YEAR2,	ID3TAG_TYER},
	{IDC_TAG_EDIT_GENRE2,	ID3TAG_TCON},
	{IDC_TAG_EDIT_COMMENT2,	ID3TAG_COMM},
	{IDC_TAG_EDIT_TRACK2,	ID3TAG_TRCK},
	{IDC_TAG_EDIT_INITKEY2,	ID3TAG_TKEY},
	{IDC_TAG_EDIT_BPM2,		ID3TAG_TBPM},
	{IDC_TAG_EDIT_SUBTITLE2,ID3TAG_TIT3},
	{IDC_TAG_EDIT_REMIXED2, ID3TAG_TPE4},
	{IDC_TAG_EDIT_COMPOSER2,ID3TAG_TCOM}

/*	
	
	{IDC_ORCHESTRA, ID3TAG_TPE2},
	{IDC_CONDUCTOR, ID3TAG_TPE3},
	{IDC_URL,		ID3TAG_WXXX},
	{IDC_OARTIST,	ID3TAG_TOPE},
	{IDC_ENCODER,	ID3TAG_TENC},
	{IDC_TCOPYRIGHT, ID3TAG_TCOP},
	{IDC_LYRIC_EDIT,ID3TAG_USLT}
*/
};




/* ************************************************************************ */
CDlgID3Template::CDlgID3Template(CDlgPlaylist* pThis): CDlg()
{
	m_pThisPlaylist = pThis;
	m_hwndList = pThis->m_hwndList;

	m_listFrmt[0] = GS(LNG_ARTIST); 
	m_listFrmt[1] = GS(LNG_TITLE);
	m_listFrmt[2] = GS(LNG_ALBUM);
	m_listFrmt[3] = GS(LNG_YEAR);
	m_listFrmt[4] = GS(LNG_COMMENT);
	m_listFrmt[5] = GS(LNG_TRACK);

}	



/* ************************************************************************ */
void CDlgID3Template::InitFileList(BOOL bShortFileName)
{
	int i, nCount, idx;
	LV_ITEM lvi;
	TCHAR szTemp[1024], *p;
	
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_RESETCONTENT, 0, 0);

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_PARAM; 
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) 
			continue;
	
		if (bShortFileName)
		{
			p = _tcsrchr(mp3i->GetCFilename(), '\\');
	
			if (p)
				_tcscpy(szTemp, p+1);
			else
				_tcscpy(szTemp, mp3i->GetCFilename());
		}
		else
			_tcscpy(szTemp, mp3i->GetCFilename());


		idx = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_ADDSTRING, 0, (LPARAM)(LPTSTR)szTemp);		
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_SETITEMDATA, idx, (LPARAM)mp3i);		

		for (int iSel=0; iSel<m_pThisPlaylist->m_nSelCount; iSel++)
		{
			if (m_pThisPlaylist->m_mp3iSelArray[iSel] == mp3i)
			{
				SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_SETSEL, TRUE, idx);
				break;
			}
		}	

	}	


}



/* ************************************************************************ */
int CDlgID3Template::GetFrameID(int resID)
{
	for(int i=0; i<MAX_RES2FRAME; i++)
		if (m_Res2Frame[i][0] == resID) 
			return(m_Res2Frame[i][1]);
	
	return(0); // not found
}

/* ************************************************************************ */
int CDlgID3Template::GetResID(int fid)
{
	for(int i=0; i<MAX_RES2FRAME; i++)
		if (m_Res2Frame[i][1] == fid) 
			return(m_Res2Frame[i][0]);
	
	return(0);	// not found
}


/* ************************************************************************ */
// Displays the info for the Info of the specific frame
void CDlgID3Template::SetTextMP3info(int fid, CMP3Info *mp3i)
{
	// only supported frames allowed
	if (!ID3Frames[fid].Supported) return;

	CMP3Frame *mp3f = mp3i->GetFrame(fid);
	
	if (ID3Frames[fid].Type != ID3ENC_STRING) 
		return;
	else
	{
		HWND hEdit = GetDlgItem(m_hDlg, GetResID(fid));

		tstring str(_T(""));
		
		if (mp3f)
			str = ((CMP3StringFrame *)mp3f)->GetString();	// handle only the first String

		if (hEdit)
			SetWindowText(hEdit, str.c_str());
	
	}
	
}

/* ************************************************************************ */
// Retrieve the info for a specific frame from Dialog
void CDlgID3Template::GetTextMP3info(int fid, CMP3Info *mp3i)
{
	// only supported frames allowed
	if (!ID3Frames[fid].Supported) return;

	CMP3Frame *mp3f = mp3i->GetFrame(fid);
	
	if (ID3Frames[fid].Type != ID3ENC_STRING)
		return;
	else
	{

		HWND hEdit = GetDlgItem(m_hDlg, GetResID(fid));

		if (hEdit)
		{
			int len = GetWindowTextLength(hEdit);
			LPTSTR str = (LPTSTR)alloca((len+1)*sizeof(TCHAR));
			if (str)
			{
				CMP3StringFrame *sf;
				GetWindowText(hEdit, str, len+1);
			
				sf = new CMP3StringFrame(fid, str);
			
				
				if (sf)
				{	// Update Frame MP3Info
					mp3i->UpdateFrame(sf);
					delete sf;
				}
			}
		}

		
	}	
	
}



/* ************************************************************************ */
void CDlgID3Template::FrmtCheck2List(int idx)
{
	int i,j, nCount;

	idx -= IDC_TAG_CHCKARTIST_FRMT;

	if (IsDlgButtonChecked(m_hDlg,IDC_TAG_CHCKARTIST_FRMT + idx))	
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_ADDSTRING, 0, (LPARAM)(LPSTR)m_listFrmt[idx]);
	else
	{
		TCHAR szTmp[40];
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_GETCOUNT, 0, 0);

		for (i = 0; i < nCount; i++)
		{	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_GETTEXT, i, (LPARAM) (LPCTSTR)szTmp );
			for (j = 0; j < MAX_COL_FRMT; j++)
			{	if (_tcscmp(m_listFrmt[idx], szTmp)== 0)
				{
					SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_DELETESTRING, i, 0);	
					SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_SETCURSEL, --nCount, 0);
					return;
				}
			}	
		}

	}

}


/* ************************************************************** */
void  CDlgID3Template::FrmtRemoveCol()
{	
	int i,j;
	TCHAR szTmp[40];

	i = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_GETCURSEL, 0, 0);

	if (i == LB_ERR) 
		return;

	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_GETTEXT, i, (LPARAM) (LPCTSTR)szTmp );
	
	for (j = 0; j < MAX_COL_FRMT; j++)
	{	if (_tcscmp(m_listFrmt[j], szTmp) == 0)
		{
			CheckDlgButton(m_hDlg,IDC_TAG_CHCKARTIST_FRMT + j, BST_UNCHECKED);
			SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_DELETESTRING, i, 0);	
			SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_SETCURSEL, i, 0);
			return;
		}
	}	
	
}


/* ************************************************************************ */
void CDlgID3Template::FrmtUpdateSeparator()
{
	TCHAR szTmp[4];

	HWND hEditSep = GetDlgItem(m_hDlg, IDC_EDITSEPARATOR);
	int len = GetWindowTextLength(hEditSep);
	GetWindowText(hEditSep, szTmp, len+1);

	_tcscpy(gParam.szSep, szTmp);
	FrmtUpdate();

}

/* ************************************************************************ */
void CDlgID3Template::FrmtUpdate()
{
	int i, nCount, nLen;
	TCHAR szTmp[40];
	TCHAR szFormat[256];
	TCHAR *p;

	nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT,  LB_GETCOUNT, 0, 0);

	szFormat[0] = '\0';

	for (i = 0; i <nCount; i++)
	{
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_GETTEXT, i, (LPARAM) (LPCTSTR)szTmp );
		_tcscat(szFormat, _T("<"));
		_tcscat(szFormat, szTmp);
		_tcscat(szFormat, _T(">"));
		_tcscat(szFormat, gParam.szSep);

	}
	
	nLen = _tcslen(szFormat);
	
	// Remove  at the end " - "
	if (nLen > 0)
	{	
		
		p = szFormat + nLen - _tcslen(gParam.szSep);
		*p = '\0';
	}
	
	// Update  format zone
	SetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_FRMT), (LPCTSTR)szFormat);	

	// Enable GO Button if at least one item is selected
	EnableWindow(GetDlgItem(m_hDlg, IDC_TAG_BTN_GO), (nCount != 0) ? TRUE : FALSE);

}

/* ************************************************************************ */
void CDlgID3Template::FrmtSelectAll(int nID)
{
	int i;
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_RESETCONTENT, 0, 0);
	for (i = 0; i < MAX_COL_FRMT; i++)
	{	
		CheckDlgButton(m_hDlg,IDC_TAG_CHCKARTIST_FRMT+i, nID == IDC_TAG_SELECTALL ? 
			BST_CHECKED : BST_UNCHECKED);	

		if (nID == IDC_TAG_SELECTALL)
			SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_ADDSTRING, 0, (LPARAM)(LPSTR)m_listFrmt[i]);		
	}
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_SETCURSEL, 0, 0);
}


/* ************************************************************************ */
void CDlgID3Template::LOGEnable()
{
	BOOL bEnable;

	bEnable = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_LOG)==BST_CHECKED);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_EDIT_LOG), bEnable);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_LOG), bEnable);
}

/* ************************************************************************ */
void CDlgID3Template::LOGChooseFile()
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256]; 
	TCHAR szFile[256];
	TCHAR szFileTitle[256]; 
    TCHAR szFilterTmp[MAX_PATH];
	
	_tcscpy(szFilterTmp,  GS(LNG_FILTEROPENDIALOG_LOG) );
	TCHAR *szFilter = szFilterTmp;
    
	SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), (LPTSTR)szDirName);
	*szFile = '\0'; 

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = szFile; 
	ofn.nMaxFile = sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.lpstrTitle = GS(LNG_MSG_CHOOSELOG); 
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 

	if (GetSaveFileName(&ofn)) 
	{ 
		if(ofn.nFilterIndex==1) 
			_tcscat(szFile, _T(".txt"));	
		else
			_tcscat(szFile, _T(".log")); 
			
		SetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG), szFile);
	} 

}


/* ************************************************************************ */
BOOL CDlgID3Template::MsgConfirm(int nCount, int *pArray, int nPage)
{
	TCHAR szTemp[1024];
	TCHAR szMsg[8192];
	BOOL bLimitList = FALSE;
	int nIdMsg;
	
	switch(nPage)
	{	
		case 0:	nIdMsg = LNG_MSG_CONFIRM_MASS;					break;
		case 1:	nIdMsg = LNG_MSG_CONFIRM_RENAME;				break;
		case 2:	nIdMsg = LNG_MSG_CONFIRM_WRITETAGFROMFILENAME;	break;
		case 3:	nIdMsg = LNG_MSG_CONFIRM_COPYID3V1TOID3V2;		break;
		case 4:	nIdMsg = LNG_MSG_CONFIRM_COPYID3V2TOID3V1;		break;
		case 5:	nIdMsg = LNG_MSG_CONFIRM_REMOVEID3V1;			break;
		case 6:	nIdMsg = LNG_MSG_CONFIRM_REMOVEID3V2;			break;
		case 7:	nIdMsg = LNG_MSG_CONFIRM_REMOVEID3V1ID3V2;		break;
		case 8:	nIdMsg = LNG_MSG_CONFIRM_ORGANIZE;				break;
	}

	_stprintf(szMsg, GS(nIdMsg), nCount);
	_tcscat(szMsg, _T("\n\n") );

	BOOL bSelect = (pArray != NULL);

	// Afiche uniquement les 10 premiers fichiers
	if (nCount > 10)
	{	nCount = 10;
		bLimitList = TRUE;
	}

	for (int i = 0; i<nCount; i++)
	{
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETTEXT, (WPARAM)(bSelect ? pArray[i] : i), (LPARAM)(LPCTSTR)szTemp);
		_tcscat(szMsg, szTemp);
		_tcscat(szMsg, _T("\n"));
	}
	
	if (bLimitList)
	{	_tcscat(szMsg, _T("\n\n"));
		_tcscat(szMsg, GS(LNG_MOREFILES));

	}
	_tcscat(szMsg, _T("\n\n"));
	_tcscat(szMsg, GS(LNG_PROCEEDTHEM) );

	if (MessageBox(m_hDlg, szMsg, GS(LNG_QUESTION), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) 
		== IDYES)
		return(TRUE);
	else
		return(FALSE);
	

}


/* ************************************************************************ */
BOOL  CDlgID3Template::PrepareItemList(BOOL& bSelected, int& nCount, int **pArray, int nAction)
{

	bSelected = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ONLYSELECTED)== BST_CHECKED);

	if (!bSelected)
	{	// All Items 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETCOUNT, 0, 0);
	}
	else
	{	// Only selected item 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELCOUNT, 0, 0);
		if (nCount == LB_ERR || nCount == 0)
		{	MessageBox(m_hDlg, GS(LNG_MSG_SELECT_FILE), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
			return FALSE; 
		}
	
		*pArray = (int*)malloc(nCount*sizeof(int));
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELITEMS, (WPARAM)(nCount), (LPARAM)(*pArray));
	}

	
	// Msgbox confirm before execute ?
	if (MsgConfirm(nCount, *pArray, nAction) == FALSE)
	{
		if (bSelected == TRUE)
			free(*pArray);
		return FALSE;
	}

	return TRUE;
}


/* ************************************************************************ */
BOOL  CDlgID3Template::PrepareLogFile(BOOL &bLog, FILE *filelog)
{

	bLog = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_LOG)== BST_CHECKED);
		
	if (bLog)
	{
		TCHAR szFileLog[MAX_PATH];
		szFileLog[0] = '\0';
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG), (LPTSTR)szFileLog, sizeof(szFileLog));
		
		if (_tcslen(szFileLog) == 0)
		{	MessageBox(m_hDlg, GS(LNG_MSG_CHOOSELOG), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
			return FALSE;
		}
		if ((filelog = _tfopen(szFileLog, _T("w+t"))) == NULL)
		{	MessageBox(m_hDlg, GS(LNG_MSG_ERRLOG), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
			return FALSE;
		}
	
		_ftprintf(filelog, GS(LNG_APPNAME) );
		_ftprintf(filelog, _T("\n----------------------\n") );
		_ftprintf(filelog, GS(LNG_HTML_DATE), GetFormattedDateTime());
		_ftprintf(filelog, _T("\n\n"));
	}

	return TRUE;
}

/* ************************************************************************ */
void CDlgID3Template::InitProgressBar(HWND hProgress, int nCount)
{
	ShowWindow(hProgress, SW_SHOW);
	SendMessage(hProgress, PBM_SETPOS, 1, 0);	// these two are required to display the empty bar
	SendMessage(hProgress, PBM_SETPOS, 0, 0);
	SendMessage(hProgress, PBM_SETRANGE, 0, MAKELPARAM(0, nCount)); 
	SendMessage(hProgress, PBM_SETSTEP, (WPARAM) 1, 0);
}
/* ************************************************************************ */
void CDlgID3Template::SaveSelectedItem()
{
	int *pArray = NULL;
	int nSelCount, i;
	CMP3Info *mp3i = NULL;

	
	nSelCount = m_pThisPlaylist->m_nSelCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELCOUNT, 0, 0);

	pArray = (int*)malloc(nSelCount*sizeof(int));
	if (pArray == NULL) return;


	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELITEMS, (WPARAM)nSelCount, (LPARAM)pArray);

	if (m_pThisPlaylist->m_mp3iSelArray != NULL)
		free(m_pThisPlaylist->m_mp3iSelArray);

	m_pThisPlaylist->m_mp3iSelArray = (CMP3Info **)malloc(nSelCount*sizeof(CMP3Info *));

	for (i = 0; i < nSelCount; i++)
	{
		mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETITEMDATA, (WPARAM)pArray[i], 0);

		if (mp3i != NULL)
			m_pThisPlaylist->m_mp3iSelArray[i] = mp3i;

	}

	if (pArray != NULL)
		free(pArray);
}




/* ************************************************************************ */
void CDlgID3Template::OwnerDrawBtn(LPDRAWITEMSTRUCT lpdis)
{
	HDC hDC;
    RECT rect;

    hDC = lpdis->hDC;
    CopyRect (&rect, &(lpdis->rcItem));    

	if (lpdis->CtlID == IDC_TAG_OPTIONCAPITALIZE)
	{
		UINT uState = DFCS_SCROLLRIGHT;
		
		switch (lpdis->itemAction) 
		{	
			case ODA_DRAWENTIRE:  
			case ODA_FOCUS:
				break;

			case ODA_SELECT:
				if (lpdis->itemState & ODS_SELECTED)
					uState = DFCS_SCROLLRIGHT | DFCS_PUSHED;
				break;

			default:
				return;

		}

		DrawFrameControl(hDC, &rect, DFC_SCROLL, uState);
	}

}

/* ************************************************************ */
void CDlgID3Template::MenuButton(int nIDButton, int nIDMenu, int nSubMenu) 
{ 
	HMENU hMenuLoad, hMenu; 
	HWND hwndButton;
	RECT rect;
	POINT pt;

	hMenuLoad = LoadMenu(plugin.hDllInstance, MAKEINTRESOURCE(nIDMenu)); 
	
	hMenu = GetSubMenu(hMenuLoad, nSubMenu); 
			
	// Initialise Menu Option 'Capitalize/Convert'
	if (nIDButton == IDC_TAG_OPTIONCAPITALIZE)	
	{
		MenuSetText( hMenu, 0, GS(LNG_MENU_CAPITALIZE_UNCHANGED));                     
		MenuSetText( hMenu, 1, GS(LNG_MENU_CAPITALIZE_ALLLOWERCASE));                  
		MenuSetText( hMenu, 2, GS(LNG_MENU_CAPITALIZE_ALLUPPERCASE));                  
		MenuSetText( hMenu, 3, GS(LNG_MENU_CAPITALIZE_FIRSTLETTEROFSECTIONUPPERCASE)); 
		MenuSetText( hMenu, 4, GS(LNG_MENU_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE));
		MenuSetText( hMenu, 6, GS(LNG_MENU_CONVERT_NO));
		MenuSetText( hMenu, 7, GS(LNG_MENU_CONVERT_UNDERSCORE_TO_SPACE));
		MenuSetText( hMenu, 8, GS(LNG_MENU_CONVERT_SPACE_TO_UNDERSCORE));

		CheckMenuRadioItem((HMENU) hMenu, 
		   ID_CAPITALIZE_UNCHANGED, 
		   ID_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE,
		   ID_CAPITALIZE_UNCHANGED + gParam.nOptionCapitalize , MF_BYCOMMAND);
	
		CheckMenuRadioItem((HMENU) hMenu, 
		   ID_CONVERT_NO, 
		   ID_CONVERT_SPACE_TO_UNDERSCORE,
		   ID_CONVERT_NO + gParam.nOptionConvert , 
		   MF_BYCOMMAND);
	}


	if(nIDMenu==IDR_MENUSWAP)// Swap Field (EditTag)
	{
		switch(nSubMenu)
		{
			case 0:// IDC_TAG_EDIT_SWAPFIELD_ID3V1_TITLE
			case 3:// IDC_TAG_EDIT_SWAPFIELD_ID3V2_TITLE
				MenuSetText(hMenu, 0, GS(LNG_MENU_SWAPID3_TITLE_ARTIST));                 
				MenuSetText(hMenu, 1, GS(LNG_MENU_SWAPID3_TITLE_ALBUM));                 
				break;
			case 1:// IDC_TAG_EDIT_SWAPFIELD_ID3V1_ARTIST
			case 4:// IDC_TAG_EDIT_SWAPFIELD_ID3V2_ARTIST
				MenuSetText(hMenu, 0, GS(LNG_MENU_SWAPID3_ARTIST_TITLE));                 
				MenuSetText(hMenu, 1, GS(LNG_MENU_SWAPID3_ARTIST_ALBUM));                 
				break;
			case 2:// IDC_TAG_EDIT_SWAPFIELD_ID3V1_ALBUM
			case 5:// IDC_TAG_EDIT_SWAPFIELD_ID3V2_ALBUM
				MenuSetText(hMenu, 0, GS(LNG_MENU_SWAPID3_ALBUM_TITLE));                 
				MenuSetText(hMenu, 1, GS(LNG_MENU_SWAPID3_ALBUM_ARTIST));                 
				break;
		}
		
	}

	
	hwndButton = GetDlgItem(m_hDlg, nIDButton); 
	GetClientRect(hwndButton, &rect);
	pt.x  = rect.right;
	pt.y  = rect.top;
	ClientToScreen(hwndButton, &pt);

	TrackPopupMenu(hMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
		pt.x, pt.y, 0, m_hDlg, NULL); 

	DestroyMenu(hMenuLoad); 
} 

/* ************************************************************ */
void CDlgID3Template::CapitalizeString(TCHAR* szTemp)
{


	switch(gParam.nOptionCapitalize)
	{
		case 0:	// Unchanged
			break;
		
		case 1:	// all lowercase
			CharLower(szTemp);
			break;
		
		case 2:	// all Uppercase
			CharUpper(szTemp);
			break;
		
		case 3: // Capitilize only 1st letter 
			{
				CharLower(szTemp);

				TCHAR *p = &szTemp[0];
				*p = toupper(*p);
			}
			break;
		case 4: // Capitilize all words
			{
				CharLower(szTemp);

				int nLen = _tcslen(szTemp);

				TCHAR *p, *q;

				for (int i = 0; i < nLen-1; i++)
				{
					p = &szTemp[i];
					q = &szTemp[i+1];

					if (i ==0)
						*p = toupper(*p);

					if (*p ==  ' ' && *q  != ' ')
						*q = toupper(*q);


				}

			}
			break;

	}

}


/* ************************************************************ */
void CDlgID3Template::ConvertString(TCHAR* szTemp)
{


	switch(gParam.nOptionConvert)
	{
		case 0:	// Unchanged
			break;
		
		case 1:	// Convert Underscore to space
			{
				int nLen = _tcslen(szTemp);
				TCHAR *p;
				for (int i = 0; i < nLen; i++)
				{
					p = &szTemp[i];
					if (*p == '_')	*p = ' ';
				}

			}
			break;

		case 2:	// Convert Space to Underscore 
			{
				int nLen = _tcslen(szTemp);
				TCHAR *p;
				for (int i = 0; i < nLen; i++)
				{
					p = &szTemp[i];
					if (*p == ' ')	*p = '_';
				}

			}
			break;

	}

}




/* ****************************************************************** */
void CDlgID3Template::GetFrameInfo(CMP3Info *mp3i, int fid, tstring &str)
{
	// only supported frames allowed
	if (!ID3Frames[fid].Supported) return;

	if (ID3Frames[fid].Type == ID3ENC_STRING)
	{
		CMP3Frame *mp3f = mp3i->GetFrame(fid);
		
		if (mp3f)
			str = ((CMP3StringFrame *)mp3f)->GetString();	// handle only the first String
	}

}
