#include "priv.h"
#include <commctrl.h>

#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"
#include "Language.h"

// external declaration
extern INITSTRUCT gParam;
extern CLanguage    *lng;


// Use to WriteTAGFromFile
extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);




/* ************************************************************************ */
BOOL CDlgID3WriteTagFromFile::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		InitLanguage();
		LoadParam();
		InitFileList();
		return FALSE;
	
	case WM_CLOSE:
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_DESTROY:
		SaveParam();
		return 0;

	case WM_DRAWITEM:
		OwnerDrawBtn((LPDRAWITEMSTRUCT)lParam);		
		return TRUE;

	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDOK:
		case IDCANCEL:
			EndDialog(m_hDlg,0);
			return TRUE;

		case IDC_TAG_OPTIONCAPITALIZE:
			MenuButton(IDC_TAG_OPTIONCAPITALIZE, IDR_MENU1,5); // 5:Capitalize SubMenu
			return FALSE;

		case ID_CAPITALIZE_UNCHANGED:
		case ID_CAPITALIZE_ALLLOWERCASE:
		case ID_CAPITALIZE_ALLUPPERCASE:
		case ID_CAPITALIZE_FIRSTLETTEROFSECTIONUPPERCASE:
		case ID_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE:
			gParam.nOptionCapitalize  = LOWORD(wParam) - ID_CAPITALIZE_UNCHANGED;
			return TRUE;

		case ID_CONVERT_NO:
		case ID_CONVERT_UNDERSCORE_TO_SPACE:
		case ID_CONVERT_SPACE_TO_UNDERSCORE:
			gParam.nOptionConvert  = LOWORD(wParam) - ID_CONVERT_NO;
			return TRUE;


		case IDC_EDITSEPARATOR:	
			if (HIWORD(wParam) == EN_CHANGE) 
				FrmtUpdateSeparator();
			return TRUE;

		case IDC_TAG_LIST_FILE:	    
			if (HIWORD(wParam) == LBN_SELCHANGE)
				SaveSelectedItem();
			return TRUE;

		case IDC_TAG_CHCKARTIST_FRMT:
		case IDC_TAG_CHCKTITLE_FRMT:
		case IDC_TAG_CHCKALBUM_FRMT:
		case IDC_TAG_CHCKYEAR_FRMT:
		case IDC_TAG_CHCKCOMMENT_FRMT:
		case IDC_TAG_CHCKTRACK_FRMT:
			if (HIWORD(wParam) == BN_CLICKED)
			{	FrmtCheck2List(LOWORD(wParam));
				FrmtUpdate();
			}
			return TRUE;

		case IDC_TAG_SELECTALL:
		case IDC_TAG_NONE:
			FrmtSelectAll(LOWORD(wParam));
			FrmtUpdate();
			return TRUE;

		case IDC_TAG_MOVEUP:
		case IDC_TAG_MOVEDOWN:
			ListboxMoveItem(GetDlgItem(m_hDlg, IDC_TAG_LIST_FRMT), LOWORD(wParam) == IDC_TAG_MOVEUP);
			FrmtUpdate();
			return TRUE;

		case IDC_TAG_LIST_FRMT:
			if (HIWORD(wParam) == LBN_DBLCLK)
			{	FrmtRemoveCol();
				FrmtUpdate();
			}
			return TRUE;

		case IDC_TAG_CHK_LOG:
			LOGEnable();
			return TRUE;
		case IDC_TAG_BTN_LOG:
			LOGChooseFile();
			return TRUE;
		case IDC_TAG_BTN_GO:
			BtnGO();
			return TRUE;

		}
	}
	return FALSE;
}

// **********************************************
void CDlgID3WriteTagFromFile::InitLanguage()
{
	LNGRESSTRUCT lrs[15] = 
	{
		{ IDC_FILEFORMAT, LNG_DLGID3TAG_FILE_FORMAT },
		{ IDC_SEPARATOR, LNG_DLGID3TAG_SEPARATOR },
		{ IDC_TAG_CHCKARTIST_FRMT, LNG_ARTIST_SHORTCUT },
		{ IDC_TAG_CHCKTITLE_FRMT, LNG_TITLE_SHORTCUT },
		{ IDC_TAG_CHCKALBUM_FRMT, LNG_ALBUM_SHORTCUT },
		{ IDC_TAG_CHCKYEAR_FRMT, LNG_YEAR_SHORTCUT },
		{ IDC_TAG_CHCKCOMMENT_FRMT, LNG_COMMENT_SHORTCUT },
		{ IDC_TAG_CHCKTRACK_FRMT, LNG_TRACK_SHORTCUT },
		{ IDC_TAG_MOVEUP, LNG_DLGID3TAG_BTN_MOVEUP },
		{ IDC_TAG_MOVEDOWN, LNG_DLGID3TAG_BTN_MOVEDOWN },
		{ IDC_TAG_SELECTALL, LNG_DLGID3TAG_BTN_SELECTALL },
		{ IDC_TAG_NONE, LNG_DLGID3TAG_BTN_NONE },
		{ IDC_TAG_BTN_GO, LNG_BTN_GO },
		{ IDC_TAG_CHK_LOG, LNG_DLGID3TAG_TRACE_LOG },
		{ IDC_TAG_CHK_ONLYSELECTED, LNG_DLGID3TAG_ONLY_SELECTED_ITEM }
	};                                                                        
                                                                              
	for (int i = 0 ; i<NUMELEMENTS(lrs); i++)                                                
		DlgSetText(m_hDlg, lrs[i].idRes, GS(lrs[i].idLNG) );          

}



/* ************************************************************** */
void CDlgID3WriteTagFromFile::LoadParam()
{
	int i;

	CheckDlgButton(m_hDlg, IDC_TAG_CHK_ONLYSELECTED, BST_CHECKED); 

	for (i = 0; i < 2; i++)
		CheckDlgButton(m_hDlg,IDC_TAG_CHK_ID3V1+i,(gParam.nWriteTagfromFile & (1<<i))?BST_CHECKED:BST_UNCHECKED);	

	SendDlgItemMessage( m_hDlg, IDC_EDITSEPARATOR, EM_SETLIMITTEXT , 3L, 0L);
	SetWindowText(GetDlgItem(m_hDlg, IDC_EDITSEPARATOR), gParam.szSep);

	// Format Column options 
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_RESETCONTENT, 0, 0);
	
	for (i = 0; i < MAX_COL_FRMT; i++)
	{
		int k = gParam.FrmtCol[i];

		if (k != -1)
		{	
			SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_ADDSTRING, 0, (LPARAM)(LPSTR)m_listFrmt[k]);
			CheckDlgButton(m_hDlg,IDC_TAG_CHCKARTIST_FRMT+k,BST_CHECKED);
		}
	}

	FrmtUpdate();
}


/* ************************************************************** */
void CDlgID3WriteTagFromFile::SaveParam()
{
	int i, k, nCount ;
	TCHAR szTemp[40];

	gParam.nWriteTagfromFile = 0;
	for (i = 0; i < 2; i++)
		if (IsDlgButtonChecked(m_hDlg,IDC_TAG_CHK_ID3V1+i))
			gParam.nWriteTagfromFile |= 1<<i;


	// Save Order format columns
	for (i = 0; i < MAX_COL_FRMT; i++)	
		gParam.FrmtCol[i] = -1;

	nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_GETCOUNT, 0, 0);
	for (i = 0; i < nCount; i++)
	{	
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FRMT, LB_GETTEXT, i, (LPARAM) (LPCTSTR)szTemp );
		
		for (k = 0; k < MAX_COL_FRMT; k++)
		{	if (_tcscmp(m_listFrmt[k], szTemp)== 0)
			{	gParam.FrmtCol[i] = k;
				break;
			}				
		}	
	}

}




/* ************************************************************************ */
void CDlgID3WriteTagFromFile::BtnGO()
{
	HCURSOR hcurOld;
	int i, nCount;
	int *pArray = NULL;
	FILE *filelog = NULL;
	BOOL bOK, bSelected, bLog;
	const TCHAR *Msg_label[] = {GS(LNG_FAILED), GS(LNG_SUCCESS)};
	TCHAR szMsg[1024];

	SaveParam();

	if (_tcslen(gParam.szSep) == 0)
	{	MessageBox(m_hDlg, GS(LNG_SEPARATORNOTNULL), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
		SetFocus(GetDlgItem(m_hDlg, IDC_EDITSEPARATOR));
		return;
	}

	if (gParam.nWriteTagfromFile == 0)
	{	MessageBox(m_hDlg, GS(LNG_CHECKID3), GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);
		return;
	}
	
	bOK = PrepareItemList(bSelected, nCount, (int**)&pArray, 2);
	if(!bOK)
		return;

	bOK = PrepareLogFile(bLog, filelog);
	if(!bOK)
		return;
	
	if (bLog)
	{	_ftprintf(filelog, GS(LNG_MSG_WRITETAG_LOG), ID3TagName[gParam.nWriteTagfromFile], nCount);
		_ftprintf(filelog, _T("\n\n"));
	}

	hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), FALSE);


	// Init Progress bar
	HWND hProgress = GetDlgItem(m_hDlg, IDC_PROGRESS1);
	HWND hStatus = GetDlgItem(m_hDlg, IDC_TAG_WRITETAGFROMFILE_STATUS);
	ShowWindow(hStatus, SW_SHOW);

	InitProgressBar(hProgress, nCount);

	// Scan file item	
	for (i = 0; i<nCount; i++)
	{
		int iSel = (bSelected ? pArray[i] : i);

		CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETITEMDATA , (WPARAM)iSel, 0);
		
		bOK = WriteTAGFromFile(mp3i, iSel);
		
		if (bLog)
		{	_ftprintf(filelog, GS(LNG_LOG_FILE), i+1, Msg_label[bOK], mp3i->GetCFilename());	
			_ftprintf(filelog, _T("\n") );
		}
		
		SendMessage(hProgress, PBM_STEPIT, 0, 0);
		_stprintf(szMsg, GS(LNG_STATUS_WORKING), i+1, nCount, mp3i->GetCFilename());
		SetWindowText(hStatus, szMsg);
	}
	
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), TRUE);
	SetCursor(hcurOld);

	if (bSelected)
		free(pArray);
	
	if (bLog)
		fclose(filelog);

	// Show resume action in MsgBox
	_stprintf(szMsg, GS(LNG_MSG_WRITETAG_RESULT), nCount);
	MessageBox(m_hDlg, szMsg, GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);

	ShowWindow(hProgress, SW_HIDE);
	ShowWindow(hStatus, SW_HIDE);
}

/* ************************************************************************ */
BOOL CDlgID3WriteTagFromFile::WriteTAGFromFile(CMP3Info *mp3i, int iItem)
{
	int nToken;
	TCHAR szTemp[1024];
	TCHAR szBuf[1024];
	BOOL bExist = TRUE;
	TCHAR *p;
	TCHAR *sep = &gParam.szSep[0];
	TCHAR *pSep;
	int size;
	int nAddressLength;
	int nCurrentAddress = 0;
	struct ID3V1MP3Info id3i;

	if (!mp3i) 
		return FALSE;

	CMP3Info *mp3iSave = new CMP3Info(mp3i->GetFilename());

	if (!mp3iSave)
		return FALSE;	

	// Get filename without path
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETTEXT, (WPARAM)iItem, (LPARAM)(LPCTSTR)szBuf);
	_tcscpy(szTemp, szBuf);


	// Prepare and set Tag Version 
	if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ID3V1)== BST_CHECKED)
	{
		struct ID3V1MP3Info *id3iSave = NULL;
		id3iSave = mp3iSave->GetID3v1();
		
		if (id3iSave != NULL)
			memcpy(&id3i, id3iSave, sizeof(struct ID3V1MP3Info));
		else
			memset(&id3i, 0, sizeof(struct ID3V1MP3Info));

		mp3iSave->SetTagVersion(VERSION_ID3V1);

	}
	if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ID3V2)== BST_CHECKED)
	{
		mp3iSave->SetTagVersion(VERSION_ID3V2 +  mp3iSave->GetTagVersion());
	}


	// Remove  extension '.mp3'
	nAddressLength = _tcslen(szTemp);
	nAddressLength -= 4,
	szTemp[nAddressLength] = '\0';

	nToken = 0;

	while(nCurrentAddress < nAddressLength)
	{
		// find the next separator
		pSep = _tcsstr(&szTemp[nCurrentAddress], sep);
		p = &szTemp[nCurrentAddress];
		
		if (pSep)
		{	// separator found
			size = pSep - p;
		}
		else
		{	// no found separator
			size = _tcslen(&szTemp[nCurrentAddress]);
			nCurrentAddress = nAddressLength; // force exit while loop
		}

		if (!IsStringEmpty((LPSTR)p)) 
		{	
			TCHAR szFrame[1024];

			_tcsncpy(szFrame,  (LPTSTR)p, size);
			szFrame[size] = '\0';

			// Capitalize options
			CapitalizeString(szFrame);

			// Convert options Underscore <--> space
			ConvertString(szFrame);

	
			//  ** ID3v1 **
			if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ID3V1)== BST_CHECKED)
			{
				switch(gParam.FrmtCol[nToken])
				{
					case 0:	_tcsncpy((LPTSTR)&id3i.Artist, szFrame, sizeof(id3i.Artist));	break;
					case 1:	_tcsncpy((LPTSTR)&id3i.SongName, szFrame, sizeof(id3i.SongName));	break;
					case 2:	_tcsncpy((LPTSTR)&id3i.Album, szFrame, sizeof(id3i.Album));	break;
					case 3:	_tcsncpy((LPTSTR)&id3i.Year, szFrame, sizeof(id3i.Year));	break;
					case 4:	_tcsncpy((LPTSTR)&id3i.Comment, szFrame, sizeof(id3i.Comment));	break;
					case 5:	id3i.Track = _ttoi(szFrame); break;
				}

			}
			
			
			//  ** ID3v2 **
			if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ID3V2)== BST_CHECKED)
			{
				CMP3Frame *mp3f = NULL;
				tstring str(szFrame);

				switch(gParam.FrmtCol[nToken])
				{
					case 0:	mp3f = new CMP3StringFrame(ID3TAG_TPE1, str);	break;
					case 1:	mp3f = new CMP3StringFrame(ID3TAG_TIT2, str);	break;
					case 2:	mp3f = new CMP3StringFrame(ID3TAG_TALB, str);	break;
					case 3:	mp3f = new CMP3StringFrame(ID3TAG_TYER, str);	break;
					case 4:	mp3f = new CMP3StringFrame(ID3TAG_COMM, str);	break;
					case 5:	
						{
							int nTrack = _ttoi(szFrame);
							_stprintf(szFrame, _T("%d"), nTrack);
							tstring sTrack(szFrame);
							mp3f = new CMP3StringFrame(ID3TAG_TRCK, sTrack);	break;
						}
				}

				if (mp3f)
					mp3iSave->ReplaceFrame(mp3f);

			}
		}

		// Increment the address for next iteration. 
		nCurrentAddress += size + _tcslen(sep); 
		

		nToken++;		
	}


	// Set the new ID3v1 in MP3Info
	if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ID3V1)== BST_CHECKED)
		mp3iSave->SetID3v1(&id3i); 	
	
	// Write modification ID3TAG
	if (mp3iSave->SaveTag() != ID3V2_ERR_OK)
		return FALSE;			

	mp3i->CopyAll(mp3iSave);
	delete mp3iSave;


	return TRUE;			
}


