#include "priv.h"

#include <winuser.h>   // GetClassLong
#include <commctrl.h>

#include <wingdi.h>		// Header proc
#include "gen.h"
#include "util.h"
#include "definition.h"
#include "Dlg.h"
#include "ProgressBar.h"
#include "ID3V1.h"
#include "Volume.h"
#include "Language.h"
#include "settings.h"
#include "resource.h"

#define MAX_ITEM_SIZ 512  // Taille maxi pour l'affichage d'une colonne pour le listview

#define BMWIDTH				16
#define BMHEIGHT			16
#define RGBREPLACE			0x00FF0000 // solid blue 0x00BBGGRR

extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);
// external declaration util.cpp
void SetHookAccelMain(BOOL bActive, int idAccel, HWND hDlg);
// external declaration ShBrowseFolder.cpp
int CALLBACK BrowseRecurseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData);
// external declaration
extern winampGeneralPurposePlugin plugin;
extern TCHAR szAppName[];
extern HWND ghWndMain;
extern CLanguage    *lng;
extern CSetting     *setting;
extern INITSTRUCT gParam;
extern COLUMNSSTRUCT gDefault_column[COLUMN_MAX];
extern TEMPLATESTRUCT gaTemplateDefault[MAX_TEMPLATE];


// for listview header 
// dessine triangle Tri Croissant/dessendant dans l'entete de colonne
static LRESULT CALLBACK Header_WndProc(HWND, UINT, WPARAM, LPARAM);
static void             Header_SetSortInfo(HWND hwndList, int nCol, BOOL bAsc);
static void             Header_Initialize(HWND hwndList);
static BOOL             Header_ListCtrlOnErase(HWND hWnd, HDC hDC);

// for Filter Thread
static DWORD WINAPI ThreadFilterProc(CDlgPlaylist *pdlg);

/* sort columns in reverse order */
static BOOL reverse_sort = FALSE;
static UINT m_uHeaderSortCol = 0;
static BOOL m_fHeaderSortAsc = TRUE;


#ifdef STRICT 
	static WNDPROC g_lpHeaderWndProc = NULL;
#else 
	static FARPROC  g_lpHeaderWndProc = NULL;
#endif 


BOOL gbCheckRecurseDir;		// Recursive subdirectories


STATGENRESTRUCT gStatGenre[ID3TAG_MAX_GENRE+2];







#define POPMENU_SEARCH_OPTION 0
#define POPMENU_FILE          1
#define POPMENU_ADDFILE       2
#define POPMENU_PLAYLIST      3


/* ************************************************************************ */
// Constructor initialze member variables
CDlgPlaylist::CDlgPlaylist() 
{
	m_hwndList = NULL;
	m_hwndStat = NULL;
	m_hwndSplitter = NULL;
	m_hwndTree = NULL;

	m_bShowDuplicate = FALSE;
	m_mp3iSelArray = NULL; 
	m_nSelCount = 0;	

	m_nSelLength = 0;
	m_filterArtist.assign(_T(""));
	m_filterAlbum.assign(_T(""));
	m_filterComment.assign(_T(""));
	m_filterMode = -1;
	m_filterID3Tag = -1;
	m_nFilterBy = NO_FILTER;

	m_bRefreshSelLength = TRUE;
	m_last_sort = 0;
	
	m_bShowTree = FALSE;
	m_bSplitFirstTime = TRUE;	
	m_bTracking = FALSE;

	m_hThread = NULL;
	m_hQuitEvent = NULL;
	m_bThreadActive = FALSE;

	m_nTotalLength=0;
	m_nAvgLength=0;
	m_nSelLength=0;
	m_nFilterBy=0;

}

/* ************************************************************************ */
CDlgPlaylist::~CDlgPlaylist()
{
	int i;
	i = 1;	// ## FOR TRACE/DEBUG
}


/* ************************************************************************ */
BOOL CDlgPlaylist::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	WORD wmId;


	switch (uMsg)
	{
	case WM_INITDIALOG:
		m_hwndList = GetDlgItem(m_hDlg, IDC_LVW_PLAYLIST);
		m_hwndTree = GetDlgItem(m_hDlg, IDC_TREEVIEW_EXPLORER);

		m_hQuitEvent   = CreateEvent(NULL, FALSE, FALSE, _T("QuitEvent") );

		ghWndMain = m_hDlg;

		// HOWTO: Set the Title Bar Icon in a Dialog Box
		// Last reviewed: December 30, 1998 - Article ID: Q179582 
		m_hIcon = (HICON)LoadImage(plugin.hDllInstance, MAKEINTRESOURCE(IDI_ICON0), IMAGE_ICON, 
                       GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), 0);
		if(m_hIcon)
           SendMessage(m_hDlg, WM_SETICON, ICON_SMALL, (LPARAM)m_hIcon);

		LoadParam();
		Init();
		Splitter_Init();
		RefreshAllCtrl_Lang();

		return TRUE;
	
/*		case WM_GETMINMAXINFO: 
		// limit minimum size of the main window 
		((MINMAXINFO FAR*)lParam)->ptMinTrackSize.x =  450;
		((MINMAXINFO FAR*)lParam)->ptMinTrackSize.y = 180; 
		return TRUE;
*/
	
	case WM_NOTIFY:
		{	// Dispatch les message NOTIFY au fenetre fille
			LPNMHDR lpNmHdr = (LPNMHDR)lParam;
			/*	// Fetch tooltip text
			if (lpNmHdr->code == TTN_NEEDTEXT)
			{	LPTOOLTIPTEXT lpttt = (LPTOOLTIPTEXT)lParam;
				CopyToolTipText( lpttt );
			}	*/
			if (lpNmHdr->hwndFrom == m_hwndList) return NotifyListView(lParam);
			if (lpNmHdr->hwndFrom == m_hwndTree) return NotifyTreeView(lParam);
		}
		break;

    case WM_LBUTTONDOWN:
        Splitter_OnLButtonDown(m_hDlg, (UINT)wParam, MAKEPOINTS(lParam));
        return TRUE;

    case WM_MOUSEMOVE:
        Splitter_OnMouseMove(m_hDlg, (UINT)wParam, MAKEPOINTS(lParam));
        return TRUE;

    case WM_LBUTTONUP:
        Splitter_OnLButtonUp(m_hDlg, (UINT)wParam, MAKEPOINTS(lParam));
        return TRUE;

	case WM_CLOSE:
		LV_ColumnSaveSetting();
		SaveParam();
		DestroyWindow ();
	    return 0;

	case WM_ACTIVATE :
		if ((LOWORD(wParam) == WA_ACTIVE)  || (LOWORD(wParam) == WA_CLICKACTIVE))          
			SetHookAccelMain(TRUE, IDR_ACCELERATOR1, m_hDlg);
		else
			SetHookAccelMain(FALSE, IDR_ACCELERATOR1, NULL);
		return TRUE;

	case WM_DESTROY :
		ghWndMain = NULL;
		if (m_hQuitEvent)
			CloseHandle(m_hQuitEvent);
		if (m_hThread)
			CloseHandle(m_hThread);
		if (m_mp3iSelArray != NULL)
			free(m_mp3iSelArray);

		LV_Playlist(COMDLG_SAVE);
		ListView_DeleteAllItems(m_hwndList); 
		TreeView_DeleteAllItems(m_hwndTree);
		//PostQuitMessage (0);	// MODELESS
		//EndDialog(m_hDlg,0);	// MODAL
		return TRUE;

	case WM_DRAWITEM:
	    OwnerDrawBtn((LPDRAWITEMSTRUCT)lParam);		
		return TRUE;

   case WM_CONTEXTMENU: 
		if(doContextMenu(wParam, lParam)) 
			return FALSE; 
		break; 

   case WM_INITMENU:
	   {
		int nCount = lng->GetNumberLang();
		int nSel   = lng->GetLanguageID();
		   CheckMenuRadioItem((HMENU) wParam, ID_OPTION_VIEW_ID3V1, ID_OPTION_VIEW_ID3V2_ID3V1,
		   ID_OPTION_VIEW_ID3V1+gParam.nShowID3v1, MF_BYCOMMAND);
		CheckMenuRadioItem((HMENU) wParam, 10000, 10000+nCount,
		   10000+nSel, MF_BYCOMMAND);
	   }
	   return TRUE;

	case WM_SIZE:
		OnSize(wParam, LOWORD(lParam), HIWORD(wParam));
		return TRUE;
	
	case WM_HSCROLL:
		VolumeSlider(m_hDlg, (HWND)lParam, TRUE);
		return TRUE;

	case WM_DROPFILES:  
		LV_DropFiles((HDROP)wParam);
		return TRUE;

	case WM_COMMAND:
		wmId    = LOWORD(wParam); 
		// Handling custom menus
		// Language
		if(wmId >= 10000 && wmId <10256)
		{
			lng->SetLanguageID(wmId-10000);
			lng->ReadLanguage();
			_tcscpy(gParam.szLanguage , lng->GetLanguage() );
			RefreshAllCtrl_Lang();
			return TRUE;
		}
		
		switch (LOWORD(wParam))
		{
	
		case ID_FILE_QUIT:
		case IDK_QUIT:
			DestroyWindow ();
			return TRUE;

		case ID_FILE_PROPERTY:
		case IDK_PROPERTY:
			{
				CDlgProperty dlg(this, TRUE);
				dlg.DoModal(IDD_DLGPROPERTY, m_hDlg);
			}
			return TRUE;

		case ID_PLAYLIST_PROPERTY:
		case IDK_PROPERTYLIST:
			{	// Selection properties
				CDlgProperty dlg(this, FALSE);
				dlg.DoModal(IDD_DLGPROPERTY, m_hDlg);
			}
			return TRUE;
		
		case ID_FILE_VOLUME_PROPERTY:
		case IDK_VOLUME_PROPERTY:
			{	// Drive Volume information
				CDlgDriveVolume dlg(this);
				dlg.DoModal(IDD_DLGVOLUMEDRIVE, m_hDlg);
			}
			return TRUE;
		
		case ID_OPTION_COLUMN:
		case IDK_COLUMN:
			{
				LV_ColumnSaveSetting();

				CDlgColumn dlg(this);
				if(dlg.DoModal(IDD_DLGCOLUMN, m_hDlg)==TRUE)
				{
					LV_ColumnResetDisplay(FALSE);
				}
			}
			return TRUE;

		case ID_OPTION_RESETTODEFAULT:
			{
				CDlgReset dlg;
				if (dlg.DoModal(IDD_RESET, m_hDlg) == TRUE)
				{
					SaveParam();
					LV_ColumnResetDisplay(FALSE);	
				}
				
			}
			return TRUE;

		case ID_OPTION_MATCHTIME:
		case IDK_MATCHTIME:
			{
				CDlgMatchTime dlg;
				int nLength = 0;
				
				nLength = dlg.DoModal(IDD_MATCHTIME, m_hDlg);
				if (nLength != 0)
					LV_MatchTime(nLength);
					
			}
			return TRUE;

		case ID_OPTION_VIEW_ID3V1:
		case IDK_VIEW_ID3V1:
			gParam.nShowID3v1 = 0;
			LV_StatGenre();
			TV_Update();
			GetMP3Selected();
			LV_UpdatePlaylist();
			SetMP3Selected();
			return TRUE;
		case ID_OPTION_VIEW_ID3V2:
		case IDK_VIEW_ID3V2:
			gParam.nShowID3v1 = 1;
			LV_StatGenre();
			TV_Update();
			GetMP3Selected();
			LV_UpdatePlaylist();
			SetMP3Selected();
			return TRUE;
		case ID_OPTION_VIEW_ID3V2_ID3V1:
		case IDK_VIEW_ID3V2_ID3V1:
			gParam.nShowID3v1 = 2;
			LV_StatGenre();
			TV_Update();
			GetMP3Selected();
			LV_UpdatePlaylist();
			SetMP3Selected();
			return TRUE;

		case ID_OPTION_OPTION:
		case IDK_OPTION:
			{
				CDlgOption dlg;
				dlg.DoModal(IDD_DLGOPTION, m_hDlg);
			}
			return TRUE;

		case ID_OPTION_HTMLCSSCLASSEDITOR:
		case IDK_HTMLCSSEDITOR:
			{
				CDlgStyleHTML dlg;
				dlg.DoModal(IDD_DLGSTYLEHTML, m_hDlg);
			}
			return TRUE;

		case ID_OPTION_CONFIGURE:
		case IDK_CONFIGURE:
			{
				CDlgSysTray dlg;
				dlg.DoModal(IDD_DLGSYSTRAY, m_hDlg);
			}
			return TRUE;
	
			
		case ID_OPTION_EXPLORER:
		case IDK_TREEEXPLORER:
			if (m_bShowTree==FALSE)
				TV_Create();
			else
				TV_Destroy();

			return TRUE;

		case ID_OPTION_EXPLORER_OPTION:
			{
				CDlgOptionExplorer dlg;
				if(dlg.DoModal(IDD_DLGOPTIONEXPLORER, m_hDlg)==TRUE)
				{
					LV_StatGenre();
					TV_Update();
					GetMP3Selected();
					LV_UpdatePlaylist();
					SetMP3Selected();
				}
			}
			return TRUE;


		case ID_FILE_RELOAD:
		case IDK_RELOAD_REFRESH:
			LV_PlaylistWinamp(COMDLG_OPEN);			
			return TRUE;


		case ID_OPTION_FILTER:
		case IDK_FILTER:
			{
				CDlgFilter dlg(this);
				if(dlg.DoModal(IDD_DLGFILTER, m_hDlg)==TRUE)
				{
					LV_DetermineFilter();
					LV_TitleWindowFilter();
					LV_UpdatePlaylist();
				}
			}
			return TRUE;
		
		case ID_PLAYLIST_EDITTAG:
			SendMessage(m_hDlg,WM_COMMAND, (WPARAM)ID_ID3TAG_EDITID3TAG, NULL);
			return TRUE;

		case ID_ID3TAG_EDITID3TAG:
		case ID_ID3TAG_MASSSETID3:
		case ID_ID3TAG_RENAMEFILEFROMID3:
		case ID_ID3TAG_WRITEID3FROMFILE:
		case ID_ID3TAG_CONVERSION:
		case ID_ID3TAG_ORGANIZE:
		case IDK_EDITTAG:
			{
				int nPage;
				
				if (LOWORD(wParam) == IDK_EDITTAG)
					nPage = 0;
				else
					nPage = (LOWORD(wParam) - ID_ID3TAG_EDITID3TAG);
				
				
				GetMP3Selected();
							
				CDlgEditMain dlg(this, nPage);
				dlg.DoModal(IDD_DLGEDITMAIN, m_hDlg);
				
				LV_StatGenre();
				TV_Update();
				LV_UpdatePlaylist();
				LV_ColumnSort(m_uHeaderSortCol, FALSE, FALSE); 
				SetMP3Selected();
				
				
			}
			return TRUE;

		case ID__ABOUT:
		case IDK_ABOUTHELP:
			{
				CDlgAbout dlg;
				dlg.DoModal(IDD_DLGABOUT, m_hDlg);
			}
			return TRUE;

		case IDC_BUTTON_FILE:
		case IDC_BUTTON_ADD:
		case IDC_BUTTON_OPTION:
			MenuButton(LOWORD(wParam));				
			return FALSE;

		case ID_SEARCH_ARTIST:
		case ID_SEARCH_TITLE:
		case ID_SEARCH_ALBUM:
		case ID_SEARCH_COMMENT:
		case ID_SEARCH_FILENAME:
		case ID_SEARCH_MATCHCASEMODE:
			MenuCheckItem(LOWORD(wParam)-ID_SEARCH_ARTIST);				
			LV_UpdatePlaylist();
			return FALSE;

		case ID_SEARCH_SELECTALL:
			gParam.nOptionSearch = 31;
			LV_UpdatePlaylist();
			return FALSE;
		case ID_SEARCH_SELECTNONE:
			gParam.nOptionSearch = 0;
			LV_UpdatePlaylist();
			return FALSE;
		case IDK_FIND:
			SetFocus(GetDlgItem(m_hDlg, IDC_EDIT_SEARCH));
			return TRUE;

		case IDC_EDIT_SEARCH:
			if (HIWORD(wParam) == EN_UPDATE)
			{
/*
				TCHAR szTRACE[MAX_PATH];
				_stprintf(szTRACE, TEXT("%x\t EN_UPDATE\n"), GetTickCount());
				OutputDebugString(szTRACE);
*/
			//	LV_UpdatePlaylist();
				LaunchThreadFilter();
			}
			return TRUE;

		case IDK_SEARCH_ARTIST:
		case IDK_SEARCH_TITLE:
		case IDK_SEARCH_ALBUM:
		case IDK_SEARCH_COMMENT:
		case IDK_SEARCH_FILENAME:
			MenuCheckItem(LOWORD(wParam)-IDK_SEARCH_ARTIST);				
			return FALSE;

		case ID_FILENEW:
		case IDK_NEW:
			LV_New();
			return FALSE;

		case ID_FILEOPEN:
		case IDK_OPEN:
			LV_ComDlg(COMDLG_OPEN);
			return FALSE;
		
		case ID_FILESAVE:
		case ID_FILESAVEAS:
		case IDK_SAVE:
			LV_ComDlg(COMDLG_SAVE);
			return FALSE;
		
		case ID_FILE_INSERT:
			LV_ComDlg(COMDLG_INSERT);
			return FALSE;

		case ID_FILEIMPORT:
			LV_Import();
			return FALSE;

		case ID_FILEEXPORT:
			LV_Export();
			return FALSE;

		case ID_ADD_ADDFILES:
		case IDK_ADDFILES:
			LV_AddFiles();
			return FALSE;

		case ID_ADD_ADDDIRECTORY:
		case IDK_ADDDIRECTORY:
			LV_AddDirectory();
			return FALSE;

		case ID_PLAYLIST_PLAY:
		case IDK_PLAY:
			winamp_enqueue_to_playlist(TRUE);
			return FALSE;

		case ID_PLAYLIST_PLAYALL:
			winamp_enqueue_all_to_playlist(TRUE);
			return FALSE;

		case ID_PLAYLIST_ENQUEUE:
			winamp_enqueue_to_playlist(FALSE);
			return FALSE;

		case ID_PLAYLIST_ENQUEUEALL:
			winamp_enqueue_all_to_playlist(FALSE);
			return FALSE;

		case ID_PLAYLIST_SHOWHIDEPLAYLIST:
			SendMessage(plugin.hwndParent, WM_COMMAND, (WPARAM)40040,0);
			return FALSE;

		case ID_PLAYLIST_CLEARCURRENTPLAYLIST:
			SendMessage(plugin.hwndParent, WM_USER, (WPARAM)NULL, 101);
			return FALSE;

		case ID_PLAYLIST_SELECTALL:
		case IDK_SELECTALL:
			LV_SelectAll(TRUE);
			ST_UpdateLengthSelected();
			return FALSE;

		case ID_PLAYLIST_SELECTNONE:
			LV_SelectAll(FALSE);
			ST_UpdateLengthSelected();
			return FALSE;

		case ID_PLAYLIST_SELECTTOGGLE:
		case IDK_SELECTTOGGLE:
			LV_SelectToggle();
			return FALSE;

		case ID_PLAYLIST_SELECTWITHSAME_ARTIST: 
		case ID_PLAYLIST_SELECTWITHSAME_TITLE:  
		case ID_PLAYLIST_SELECTWITHSAME_GENRE:  
		case ID_PLAYLIST_SELECTWITHSAME_YEAR:   
		case ID_PLAYLIST_SELECTWITHSAME_ALBUM:  
		case ID_PLAYLIST_SELECTWITHSAME_COMMENT:
		case ID_PLAYLIST_SELECTWITHSAME_MODE:   
		case ID_PLAYLIST_SELECTWITHSAME_BITRATE:
		case ID_PLAYLIST_SELECTWITHSAME_FREQSAMPLING:
		case ID_PLAYLIST_SELECTWITHSAME_LAYER:
		case ID_PLAYLIST_SELECTWITHSAME_TIME:
		case ID_PLAYLIST_SELECTWITHSAME_FILESIZE:
		case ID_PLAYLIST_SELECTWITHSAME_FILENAME:
			{
			int nCol = LOWORD(wParam)-ID_PLAYLIST_SELECTWITHSAME_ARTIST;
			LV_SelectWithSame(nCol);
			}
			return FALSE;

		case ID_PLAYLIST_SELECTCROP:
		case IDK_CROP:
			LV_SelectCrop();
			return FALSE;

		case ID_PLAYLIST_DEL:
		case IDK_DEL:
			LV_SelectDel();
			return FALSE;

		case ID_PLAYLIST_REMOVEFILE:
			LV_SelectRemoveFile();
			return FALSE;


		case ID_PLAYLIST_SELECTALLDEADFILES:
			LV_DeadFiles();
			return FALSE;

		case ID_PLAYLIST_SHOWDUPLICATE:
			m_bShowDuplicate = !m_bShowDuplicate; 

			if (m_bShowDuplicate)
			{
				CDlgDuplicate dlg;
				if(dlg.DoModal(IDD_DLGDUPLICATE, m_hDlg)==TRUE)
				{
					LV_ShowDuplicate();
				}
			}
			else
				LV_UpdatePlaylist();

			return FALSE;
	

		default:
			return FALSE;
		}
		



	}
	return FALSE;
}




/****************************************************************************
* 
*    FUNCTION: NotifyListView(LPNMHDR nm)
*
*    PURPOSE: This function is the handler for the WM_NOTIFY that is 
*    sent to the parent of the list view window.
*
****************************************************************************/
LRESULT CDlgPlaylist::NotifyListView(LPARAM lParam)
{
LV_DISPINFO FAR * pnmv = (LV_DISPINFO FAR *) lParam;
	NM_LISTVIEW *pNm = (NM_LISTVIEW *)lParam;
	
	switch(((LPNMHDR) lParam)->code)
	{

		// listview
        case NM_DBLCLK: 
		case NM_RETURN: 
			switch (gParam.nDblClic)
			{
				case 0:	winamp_enqueue_to_playlist(TRUE); break;
				case 1:	winamp_enqueue_to_playlist(FALSE); break;
				case 2: SendMessage(m_hDlg,WM_COMMAND, (WPARAM)ID_ID3TAG_EDITID3TAG, NULL); break;
			}

			return(TRUE); 
		
		case LVN_ITEMCHANGED :
			{
				if (m_bRefreshSelLength == FALSE)
					return TRUE;
				
				
				NM_LISTVIEW *pnmv = (LPNMLISTVIEW) lParam; 
					
				if ((pnmv->uOldState & LVIS_SELECTED) && !(pnmv->uNewState & LVIS_SELECTED))
				{
					LVITEM lvi; 
					ZeroMemory(&lvi, sizeof(lvi)); 
					lvi.mask  = LVIF_PARAM; 
					lvi.iItem = pnmv->iItem;
					ListView_GetItem(m_hwndList, &lvi);
				
					CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

					m_nSelLength -= mp3i->GetPlayLength();
					
					TCHAR szTimeFrmt[100];
					TCHAR szTemp[100];
					Time2Str(m_nSelLength, &szTimeFrmt[0], TRUE);
					_stprintf(szTemp, GS(LNG_STATUS_SELLENGTH), szTimeFrmt);
					ST_ChangeText((LPTSTR)szTemp, 4);
					return TRUE;
				}
				
				if (!(pnmv->uOldState & LVIS_SELECTED) && (pnmv->uNewState & LVIS_SELECTED))
				{
					LVITEM lvi; 
					ZeroMemory(&lvi, sizeof(lvi)); 
					lvi.mask  = LVIF_PARAM; 
					lvi.iItem = pnmv->iItem;
					ListView_GetItem(m_hwndList, &lvi);
					
					CMP3Info *mp3i = (CMP3Info *)lvi.lParam;
					
					m_nSelLength += mp3i->GetPlayLength();
					
					TCHAR szTimeFrmt[100];
					TCHAR szTemp[100];
					Time2Str(m_nSelLength, &szTimeFrmt[0], TRUE);
					_stprintf(szTemp, GS(LNG_STATUS_SELLENGTH), szTimeFrmt);
					ST_ChangeText((LPTSTR)szTemp, 4);
					return TRUE;
				}

			}
			return TRUE;

		case LVN_GETDISPINFO:
			// Provide the item or subitem's text, if requested. 
		    if (pnmv->item.mask & LVIF_TEXT) 
			{ 
				TCHAR szCol[MAX_ITEM_SIZ];
				LV_GetInfoCol((CMP3Info*)pnmv->item.lParam, pnmv->item.iSubItem, szCol, pnmv->item.iItem + 1) ;
				_tcscpy(pnmv->item.pszText, szCol);
			}
			break;


		case LVN_COLUMNCLICK:
			LV_ColumnSort(pNm->iSubItem, FALSE);
			break;

		default:
			break;
	}
	return 0L;
}




/****************************************************************************
* 
*    FUNCTION: NotifyTreeView(LPNMHDR nm)
*
*    PURPOSE: This function is the handler for the WM_NOTIFY that is 
*    sent to the parent of the Treeview Explorer window.
*
****************************************************************************/
LRESULT CDlgPlaylist::NotifyTreeView(LPARAM lParam)
{
	LPNMHDR nm  = (LPNMHDR)lParam;
    LPNMTREEVIEW lpNmtv = (LPNMTREEVIEW)nm;

    switch (nm->code)
    {
    case TVN_SELCHANGED :
        {
			TV_SetCurrentItem(); 
			LV_TitleWindowFilter();
			LV_UpdatePlaylist(); 
            return TRUE;
        }
        break;
    }

    return FALSE;
}


/**************************************************************************** */
int CALLBACK CDlgPlaylist::ListViewCompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{   
	int iCmp; 
	tstring str1 = _T("");
	tstring str2 = _T("");
	

	CMP3Info *mi1 = (CMP3Info *)lParam1;
	CMP3Info *mi2 = (CMP3Info *)lParam2;
	
	CDlgPlaylist *pThis = (CDlgPlaylist *)lParamSort;

	int nSubItem = 	pThis->m_realColumn[pThis->m_last_sort];


	// Compare the specified column. 
	switch(nSubItem)
	{
		case 0: // Artist
			GetFrameInfo(mi1, ID3TAG_TPE1, str1);
			GetFrameInfo(mi2, ID3TAG_TPE1, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
		case 1:
			// Title songname
			GetFrameInfo(mi1, ID3TAG_TIT2, str1);
			GetFrameInfo(mi2, ID3TAG_TIT2, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;

		case 2:  // Genre
			GetFrameInfo(mi1, ID3TAG_TCON, str1);
			GetFrameInfo(mi2, ID3TAG_TCON, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
		
		case 3: // Year
			GetFrameInfo(mi1, ID3TAG_TYER, str1);
			GetFrameInfo(mi2, ID3TAG_TYER, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
		
		case 4: // Album
			GetFrameInfo(mi1, ID3TAG_TALB, str1);
			GetFrameInfo(mi2, ID3TAG_TALB, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;

		case 5: // Comment
			GetFrameInfo(mi1, ID3TAG_COMM, str1);
			GetFrameInfo(mi2, ID3TAG_COMM, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
			
		case 6: // Mode
			str1 = ModeName[mi1->GetMode()];;
			str2 = ModeName[mi2->GetMode()];;
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
		
		case 7: // Bitrate
			if (mi1->GetBitrate() == mi2->GetBitrate())
				iCmp = 0;
			else if (mi1->GetBitrate() > mi2->GetBitrate())
				iCmp = 1;
			else
				iCmp = -1;
			break;
		
		case 8: // Frequency
			if (mi1->GetFrequency() == mi2->GetFrequency())
				iCmp = 0;
			else if (mi1->GetFrequency() > mi2->GetFrequency())
				iCmp = 1;
			else
				iCmp = -1;
		
		case 9: // Layer
			{
				if (mi1->GetLayer() == mi2->GetLayer())
					iCmp = 0;
				else if (mi1->GetLayer() > mi2->GetLayer())
					iCmp = 1;
				else
					iCmp = -1;
				
				break;
			}
			
		case 10:  // Length Time
			if (mi1->GetPlayLength() == mi2->GetPlayLength())
					iCmp = 0;
				else if (mi1->GetPlayLength() > mi2->GetPlayLength())
					iCmp = 1;
				else
					iCmp = -1;
			break;

		case 11: // File size
			if (mi1->GetMP3Size() == mi2->GetMP3Size())
					iCmp = 0;
				else if (mi1->GetMP3Size() > mi2->GetMP3Size())
					iCmp = 1;
				else
					iCmp = -1;
			break;

		case 12:  // File name
			iCmp = _tcsicmp(mi1->GetCFilename(), mi2->GetCFilename());
			break;
		
		case 13: // Track #
			{
				GetFrameInfo(mi1, ID3TAG_TRCK, str1);
				GetFrameInfo(mi2, ID3TAG_TRCK, str2);
				int track1 = _ttoi(str1.c_str());
				int track2 = _ttoi(str2.c_str());

				if (track1 == track2)
					iCmp = 0;
				else if (track1 > track2)
					iCmp = 1;
				else
					iCmp = -1;
			}
				
			break;

		case 14: // ID3 Tag version
			if (mi1->GetTagVersion() == mi2->GetTagVersion())
					iCmp = 0;
				else if (mi1->GetTagVersion() > mi2->GetTagVersion())
					iCmp = 1;
				else
					iCmp = -1;
			break;


		case 15: // B.P.M.
			{
				GetFrameInfo(mi1, ID3TAG_TBPM, str1);
				GetFrameInfo(mi2, ID3TAG_TBPM, str2);
				int track1 = _ttoi(str1.c_str());
				int track2 = _ttoi(str2.c_str());

				if (track1 == track2)
					iCmp = 0;
				else if (track1 > track2)
					iCmp = 1;
				else
					iCmp = -1;
			}
				
			break;

		case 16: // Volume drive
			{
				TCHAR *p1=NULL;
				TCHAR *p2=NULL;

				p1 = pThis->m_minfo.GetInfoDriveName(mi1);
				p2 = pThis->m_minfo.GetInfoDriveName(mi2);
				iCmp = _tcsicmp(p1, p2);
			}
			break;

	}

	if (reverse_sort == TRUE)
		iCmp *= -1;
	
	return (iCmp); 

	
}

/* ************************************************************ */
void CDlgPlaylist::LV_GetInfoCol(CMP3Info *mp3i, int col, TCHAR *szCol, int iItem) 
{
	
	tstring str = _T(" ");
	
	if (!mp3i) 
	{
		_tcscpy(szCol, _T(" "));
		return;
	}


	switch(m_realColumn[col])
	{
	case 0:  // Artist
		GetFrameInfo(mp3i, ID3TAG_TPE1, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 1: // Title
		GetFrameInfo(mp3i, ID3TAG_TIT2, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 2: // Genre
		GetFrameInfo(mp3i, ID3TAG_TCON, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 3: // Year
		GetFrameInfo(mp3i, ID3TAG_TYER, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 4: // Album
		GetFrameInfo(mp3i, ID3TAG_TALB, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 5: // Comment
		GetFrameInfo(mp3i, ID3TAG_COMM, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 6: // Mode Stereo...
		str = ModeName[mp3i->GetMode()];
		_tcscpy(szCol, (LPTSTR)str.c_str());
		break;
	case 7: // Bitrate
		if (mp3i->isVBR())
			_stprintf(szCol, _T("%dkb/s (VBR)"), mp3i->GetBitrate());
		else
			_stprintf(szCol, _T("%dkb/s"), mp3i->GetBitrate());
		break;
	case 8: // Frequency
		_stprintf(szCol, _T("%dHz"), mp3i->GetFrequency());
		break;
	case 9: // Version & Mode
		{
			TCHAR mpeglayer[100];
			mp3i->GetMPEGVersionLayer(mpeglayer);
			_tcscpy(szCol, (LPTSTR)mpeglayer);
		}
		break;
	case 10: // Length Time
		Time2Str(mp3i->GetPlayLength(), szCol);
		break;
	case 11: // File size
		_stprintf(szCol, _T("%u "), mp3i->GetMP3Size());
		break;
	case 12: // File name
		str = mp3i->GetCFilename();
		_tcscpy(szCol, (LPTSTR)str.c_str());
		break;
	case 13: // Track #
		GetFrameInfo(mp3i, ID3TAG_TRCK, str);
		_tcscpy(szCol, (LPTSTR)str.c_str());
		break;
	case 14: // ID3Tag
		str = ID3TagName[mp3i->GetTagVersion()];
		_tcscpy(szCol, (LPTSTR)str.c_str());
		break;
	case 15: // B.P.M.
		GetFrameInfo(mp3i, ID3TAG_TBPM, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 16: // Volume Drive
		_tcsncpy(szCol, (LPTSTR)m_minfo.GetInfoDriveName(mp3i), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	default:
		_tcscpy(szCol, GS(LNG_NOTHING) );
		break;
	}

	// Show number for 1st column
	if (iItem != -1)
	{
		if ((gParam.bShowNumber == 1) && (col == 0))
		{	TCHAR szTemp[MAX_ITEM_SIZ];
			_stprintf(szTemp, _T("%d. %s"), iItem, szCol);       
			_tcscpy(szCol, szTemp);
		}
	}

}


/* ****************************************************************** */
void CDlgPlaylist::GetFrameInfo(CMP3Info *mp3i, int fid, tstring &str)
{
//	nShowID3v1 == 0  ID3V1
//	nShowID3v1 == 1  ID3V2
//	nShowID3v1 == 2  ID3V2 if null --> ID3V1

	// ///////////////////////////////
	// Retreive information from ID3v2
	// ///////////////////////////////
	if (gParam.nShowID3v1 == 1 || gParam.nShowID3v1 == 2)
	{
		// only supported frames allowed
		if (!ID3Frames[fid].Supported) 
			return; 
		if (ID3Frames[fid].Type != ID3ENC_STRING)	
			return;
		
		CMP3Frame *mp3f = mp3i->GetFrame(fid);
			
		if (mp3f)
			str = ((CMP3StringFrame *)mp3f)->GetString();	// handle only the first String


		
		if (gParam.nShowID3v1 == 2)
		{
			if (str.compare(_T("")) == 0 || str.compare(_T(" ")) == 0)
			{
				str.erase();
				goto showID3v1;
			}

		}
		return;
	}
	
showID3v1:
	
	// ///////////////////////////////
	// Retreive information from ID3v1
	// ///////////////////////////////
	{
		struct ID3V1MP3Info *id3i = mp3i->GetID3v1();

		if (!id3i)
			return;

		switch(fid)
		{
		
		case ID3TAG_COMM: // read the Comment
			if (!IsStringEmpty((LPTSTR)id3i->Comment)) 
				a2tcpy(str, (LPTSTR)id3i->Comment);
			break;
	
		case ID3TAG_TYER: // read the Year
			if (!IsStringEmpty((LPTSTR)id3i->Year)) 
				a2tcpy(str, (LPTSTR)id3i->Year);
			break;

		case ID3TAG_TALB: // read the Album
			if (!IsStringEmpty((LPTSTR)id3i->Album)) 
				a2tcpy(str, (LPTSTR)id3i->Album);
			break;
	
		case ID3TAG_TPE1: // read the Artist
			if (!IsStringEmpty((LPTSTR)id3i->Artist)) 
				a2tcpy(str, (LPTSTR)id3i->Artist);
			break;
	
		case ID3TAG_TIT2:  // read the Title:Song name
			if (!IsStringEmpty((LPTSTR)id3i->SongName)) 
				a2tcpy(str, (LPTSTR)id3i->SongName);
			break;
	
		case ID3TAG_TCON: // read the genre
			a2tcpy(str, (LPTSTR)GenreInt2Str(id3i->Genre));
			break;

		case ID3TAG_TRCK: // read the Track
			if (id3i->Track != 0) 
			{
				TCHAR szTrack[4];
				_stprintf(szTrack, _T("%d"), id3i->Track);
				a2tcpy(str, (LPTSTR)szTrack);
			}

		}
	
	}
}


/* ****************************************************************** */
void CDlgPlaylist::OnSize(UINT nState, int nWidth, int nHeight)
{
 	ResizeControls();
}


/* ****************************************************************** */
void CDlgPlaylist::ResizeControls()
{
	RECT rc, rcMain;
	int wWidth;  
	int wHeight; 
	int wHeightStatus;
	int wHeightEdit;
	HWND hwndEdit;
	POINT pt;
   
	GetClientRect(m_hDlg, &rcMain);
	wWidth = rcMain.right - rcMain.left;
	wHeight = rcMain.bottom - rcMain.top;


	hwndEdit = GetDlgItem(m_hDlg, IDC_EDIT_SEARCH);
	GetClientRect(hwndEdit, &rc);
	wHeightEdit  = (rc.bottom - rc.top) + 20;

	GetClientRect(m_hwndStat, &rc);
	wHeightStatus = rc.bottom - rc.top;
	
	if (m_bShowTree == FALSE)
		//Listview seul
		MoveWindow(m_hwndList, 0, wHeightEdit, wWidth , wHeight - wHeightEdit - wHeightStatus, TRUE); 
	else
	{
		if (m_bSplitFirstTime == TRUE)
		{
			m_nSplitterOffset = wWidth * 25 / 100; 
			m_bSplitFirstTime = FALSE;
		}

		
		int nWidthTree = m_nSplitterOffset;
		int nWidthSplitter = 4;
		
		//Treeview
		MoveWindow(m_hwndTree, 0, wHeightEdit, nWidthTree, wHeight - wHeightEdit - wHeightStatus, TRUE); 
		//Splitter
		MoveWindow(GetDlgItem(m_hDlg, IDC_SPLITTER), nWidthTree, wHeightEdit, nWidthSplitter, wHeight - wHeightEdit - wHeightStatus, TRUE); 
		//Listview
		MoveWindow(m_hwndList, nWidthTree+nWidthSplitter, wHeightEdit, wWidth-nWidthTree-nWidthSplitter , wHeight - wHeightEdit - wHeightStatus, TRUE); 

		Splitter_Recalc(); 
		
	}
	
	
	// Statusbar
	MoveWindow(m_hwndStat, 0, wHeight - wHeightStatus, wWidth, wHeightStatus, TRUE);

	// Resize Volume Slider 
	hwndEdit = GetDlgItem(m_hDlg, IDC_SLIDER_VOLUME);
	GetWindowRect(hwndEdit, &rc);
	
	pt.x = rc.left;
	pt.y = rc.top;
	ScreenToClient(m_hDlg, &pt);
	wWidth = wWidth -  pt.x  - 4;

	if (wWidth > 0)
		MoveWindow(hwndEdit, pt.x, pt.y, wWidth, rc.bottom - rc.top, TRUE);
}

/* ************************************************************************ */
void CDlgPlaylist::Init()
{
	LPARAM lParam;
	RECT rect;	

	HCURSOR hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));

	
	// Initialize Volume 
	VolumeInit(m_hDlg, GetDlgItem(m_hDlg, IDC_SLIDER_VOLUME));

	SendDlgItemMessage(m_hDlg, IDC_EDIT_SEARCH, EM_SETLIMITTEXT , 30L, 0L);

	ST_CreateStatus(0);
	
	// Initialize listview
	Header_Initialize(m_hwndList);
	LV_InitIcon();
	LV_ColumnResetDisplay(TRUE);
	GetColumnOrder(m_realColumn);


	if (gParam.nOnStartup == 0)
		LV_Playlist(COMDLG_OPEN);
	else
		LV_PlaylistWinamp(COMDLG_OPEN);

	
	GetClientRect(m_hDlg, &rect);
	lParam = rect.right - rect.left;  
	lParam |= (rect.bottom - rect.top) << 16; 
	SendMessage(m_hDlg, WM_SIZE , (WPARAM)SIZE_RESTORED, (LPARAM)lParam);


	// Selectionne le 1er de la liste
	ListView_SetItemState(m_hwndList, 0, LVIS_SELECTED , LVIS_SELECTED);

	SetCursor(hcurOld);
}

/* ************************************************************ */
void CDlgPlaylist::OwnerDrawBtn (LPDRAWITEMSTRUCT lpdis)
{                                                    
    HDC hDC;
    RECT rect;
	HFONT hFontButton, hOldFont;
	LOGFONT lf;	
	HRGN hRgn;
	POINT pt[4];

    hDC = lpdis->hDC;
    CopyRect (&rect, &(lpdis->rcItem));    

	if (lpdis->CtlID == IDC_BUTTON_OPTION)
	{
		UINT uState = DFCS_SCROLLRIGHT;
		
		switch (lpdis->itemAction) 
		{	
			case ODA_DRAWENTIRE:  
			case ODA_FOCUS:
				break;
			case ODA_SELECT:
				if (lpdis->itemState & ODS_SELECTED)
					uState = DFCS_SCROLLRIGHT | DFCS_PUSHED;
				break;
			default:
				return;
		}

		DrawFrameControl(hDC, &rect, DFC_SCROLL, uState);
		return;
	}
	

	if (lpdis->CtlID == IDC_BUTTON_FILE || lpdis->CtlID == IDC_BUTTON_ADD)
	{
		switch (lpdis->itemAction) 
		{	
			case ODA_DRAWENTIRE:  
			case ODA_FOCUS:
				DrawFrameControl(hDC, &rect, DFC_BUTTON, DFCS_BUTTONPUSH);
				InflateRect(&rect, -2, -2);
				rect.left += 8;
				rect.top  += 1;
				break;

			case ODA_SELECT:
				if (lpdis->itemState & ODS_SELECTED)
				{
					DrawFrameControl(hDC, &rect, DFC_BUTTON, DFCS_BUTTONPUSH|DFCS_PUSHED);
					InflateRect(&rect, -2, -2);
					rect.left += 9;
					rect.top  += 2;
				}
				else
				{
					DrawFrameControl(hDC, &rect, DFC_BUTTON, DFCS_BUTTONPUSH);
					InflateRect(&rect, -2, -2);
					rect.left += 8;
					rect.top  += 1;
				}
				
				break;

			default:
				return;

		}

		ZeroMemory(&lf,sizeof(lf));
		_tcscpy (lf.lfFaceName, _T("MS Sans Serif"));
		lf.lfHeight = 5;
		lf.lfWidth  = 5;
		lf.lfUnderline = FALSE;
		lf.lfOutPrecision = OUT_TT_PRECIS;
		hFontButton = CreateFontIndirect ((LPLOGFONT) &lf);
		if (hFontButton!=NULL) 
		{
			TCHAR szTitle[20];

			if (lpdis->CtlID == IDC_BUTTON_FILE)
				_tcscpy(szTitle, GS(LNG_BTN_FILE));
			else
				_tcscpy(szTitle, GS(LNG_MENU_ADD));

			hOldFont = (HFONT)SelectObject(hDC, hFontButton);
			SetBkMode(hDC,TRANSPARENT);
			SetTextAlign(hDC,TA_LEFT);
			TextOut(hDC,rect.left,rect.top ,szTitle, _tcslen(szTitle));

			SelectObject(hDC, hOldFont);
			DeleteObject (hFontButton); 
		}

		pt[0].x  = (lpdis->rcItem).right - 16;
		pt[0].y  = (lpdis->rcItem).top + 8;
		pt[1].x = pt[0].x + 7;
		pt[1].y = pt[0].y;
		pt[2].x = pt[0].x + 3;
		pt[2].y = pt[0].y + 4;
		pt[3].x = pt[0].x + 3;
		pt[3].y = pt[0].y + 3;
		hRgn = CreatePolygonRgn((CONST POINT*) &pt,4,WINDING);
		FillRgn(hDC, hRgn, (HBRUSH)GetStockObject(BLACK_BRUSH));
		DeleteObject(hRgn);
	}	

}                                                    



/* ************************************************************************ */
void CDlgPlaylist::LoadParam()
{
	/*
	WINDOWPLACEMENT wpNew;

	CopyMemory(&wpNew, &gParam.wp, sizeof(WINDOWPLACEMENT) );
	wpNew.length  = sizeof(WINDOWPLACEMENT);
	wpNew.showCmd = SW_HIDE;
	wpNew.flags   = WPF_RESTORETOMAXIMIZED | WPF_SETMINPOSITION;
	SetWindowPlacement(m_hDlg, &gParam.wp);
	*/

}

/* ************************************************************************ */
void CDlgPlaylist::SaveParam()
{
	// Window Position 
	gParam.wp.length = sizeof(WINDOWPLACEMENT);
	gParam.wp.flags  = WPF_RESTORETOMAXIMIZED | WPF_SETMINPOSITION;
	GetWindowPlacement(m_hDlg, &gParam.wp);

	ZeroMemory(&gParam.szLanguage, sizeof(gParam.szLanguage));
	_tcscpy(gParam.szLanguage, lng->GetLanguage());

	setting->Save();
}


/* ************************************************************************ */
void CDlgPlaylist::LV_InitIcon()
{
	HIMAGELIST hSmall;
	HICON hIcon[5];	
	int i;

	// initialise le ListView / Colonnes
	ListView_SetExtendedListViewStyle(m_hwndList, LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

	// Create Imagelist / small Icon
	// Associate the image lists with the list view control.
	hSmall = ImageList_Create (16, 16,	 ILC_COLOR16 | ILC_MASK, 3, 0);

	for (i=0; i<NUMELEMENTS(hIcon);i++)
	{	
		hIcon[i] = LoadIcon (plugin.hDllInstance, MAKEINTRESOURCE (IDI_ICON_ID3_0+i));
		ImageList_AddIcon (hSmall, hIcon[i]);
	}
	
	ListView_SetImageList (m_hwndList, hSmall, LVSIL_SMALL);

}


/* ************************************************************************ */
void CDlgPlaylist::LV_ColumnsGrid()
{
	DWORD dwExStyle = ListView_GetExtendedListViewStyle(m_hwndList);

	if (gParam.bShowGrid)
		dwExStyle |= LVS_EX_GRIDLINES;
	else
		dwExStyle &= ~LVS_EX_GRIDLINES;

	ListView_SetExtendedListViewStyle(m_hwndList, dwExStyle);
}

/* ************************************************************************ */
void CDlgPlaylist::LV_New()
{
	LV_Redraw(FALSE);

	TreeView_DeleteAllItems(m_hwndTree);
	ListView_DeleteAllItems(m_hwndList); 
	m_minfo.ResetAll();

	m_nTotalLength = 0;
	m_nAvgLength   = 0;
	m_nSelLength   = 0;
	UpdateStatus();
	LV_StatGenre();
	TV_Update();
	
	LV_Redraw(TRUE);
}

/* ************************************************************************ */
void CDlgPlaylist::LV_ComDlg(int nMode)
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256];
	TCHAR szFile[256];
	TCHAR szFileTitle[256]; 
	TCHAR szFilter[] = _T("EZ-Playlist (*.ez3)|*.ez3|");
    
    SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), szDirName);
	*szFile = '\0'; 

	memset(&ofn, 0, sizeof(OPENFILENAME)); 

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = szFile; 
	ofn.nMaxFile = sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 

	switch(nMode)
	{
		case COMDLG_INSERT:
			ofn.lpstrTitle = (LPCTSTR)_T("Insert EZ-Playlist"); 
			if (GetOpenFileName(&ofn) == 0) 
				return;
			LV_LoadEZ3(szFile, FALSE);
			break;

		case COMDLG_OPEN:
			ofn.lpstrTitle = (LPCTSTR)_T("Open EZ-Playlist"); 
			if (GetOpenFileName(&ofn) == 0) 
				return;
			LV_LoadEZ3(szFile);
			break;
		
		case COMDLG_SAVE:
			ofn.lpstrTitle = (LPCTSTR)_T("Save EZ-Playlist"); 
			if (GetSaveFileName(&ofn)) 
			{
				AddFileExtension(szFile, _T(".ez3")); 
				LV_WriteEZ3(szFile);
			}
			break;
		
	}

}

/* ************************************************************************ */
void CDlgPlaylist::LV_Playlist(int nOpen)
{
	TCHAR filename[255];
	TCHAR  *p;
	
	GetModuleFileName(plugin.hDllInstance,filename,sizeof(filename));
	p = filename + _tcslen(filename);
	while (p >= filename && *p != '.') p--;
	p++;
	*p++ = 'E';
	*p++ = 'Z';
	*p++ = '3';
	*p = '\0';
	
	
	if (nOpen == COMDLG_OPEN)
		LV_LoadEZ3(filename);
	else
		LV_WriteEZ3(filename);
}

/* ************************************************************************ */
void CDlgPlaylist::LV_PlaylistWinamp(int nOpen)
{
	TCHAR filename[255];
	HMODULE hModuleWinamp;
	TCHAR *p;
	
	hModuleWinamp = (HMODULE)GetClassLong(plugin.hwndParent, GCL_HMODULE);
	GetModuleFileName(hModuleWinamp,filename,sizeof(filename));
	p = filename + _tcslen(filename);
	while (p >= filename && *p != '.') p--;
	p++;
	*p++ = 'm';
	*p++ = '3';
	*p++ = 'u';
	*p = '\0';
	
	if (nOpen == COMDLG_OPEN)
	{
		// Writes out the current playlist to Winampdir \winamp.m3u
		SendMessage(plugin.hwndParent, WM_USER, 0,120);
		LV_LoadM3U(filename);
	}
	else
	{
		COPYDATASTRUCT cds;
		
		if (LV_WriteM3U(filename) == FALSE)
			return;

		// Efface l'ancienne playlist de WinAMP et ajoute la nvelle
		SendMessage(plugin.hwndParent, WM_USER, 0,101);

		cds.dwData = 100;
		cds.lpData = (void*)filename;
		cds.cbData = _tcslen((TCHAR*)cds.lpData)+1;
		SendMessage(plugin.hwndParent, WM_COPYDATA, (WPARAM)NULL,(LPARAM)&cds);
	}

}


/* ************************************************************************ */
void CDlgPlaylist::LV_LoadEZ3(TCHAR *filename, BOOL bInitList)
{
	FILE *file;	
	EZ3HEADER header;
	LV_ITEM lvi;
	BOOL bNewFormat;
	int i;

	if ( (file = _tfopen(filename, _T("rb"))) == NULL)
		return;

	if (fread(&header, 1, sizeof(header), file) != sizeof(header))
	{	fclose(file);
		MessageBox(m_hDlg, GS(LNG_ERR_FILEPLAYLIST), GS(LNG_ERROR), MB_OK|MB_ICONSTOP);
		return;
	}

	if (_tcsnccmp(header.tag, _T("EZ3DATA"), 7) != 0)
	{	fclose(file);
		MessageBox(m_hDlg, GS(LNG_ERR_BADFILE), GS(LNG_ERROR), MB_OK|MB_ICONSTOP);
		return;
	}

	if (_tcsnccmp(header.tag, _T("EZ3DATA1"), 8) == 0)	
		bNewFormat = FALSE;
	else if (_tcsnccmp(header.tag, _T("EZ3DATA2"), 8) == 0)	
		bNewFormat = TRUE;
	else
		bNewFormat = FALSE;

	//Turn the listview's redrawing off 
	LV_Redraw(FALSE);
	
	
	if (bInitList == TRUE)
	{	
		ListView_DeleteAllItems(m_hwndList); 
		m_minfo.ResetAll();

		m_nTotalLength = 0;
		m_nSelLength   = 0;
	}
	
    // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK; 


	// Ajout pour le nouveau format le SerialNumber 'DriveVolume'
	if (bNewFormat == TRUE)
	{
		int nCountVolumeDrive=0;
		fread(&nCountVolumeDrive, 1, sizeof(nCountVolumeDrive), file);

		for (i=0;i<nCountVolumeDrive;i++)
		{
			DRIVEVOLUMESTRUCT dvs;
			fread(&dvs, 1, sizeof(dvs), file);
			m_minfo.SetFileEZ3(&dvs);
		}
	}

	
	// Scan EZPlaylist 
	for (int iItem = 0; iItem < header.nCountFile; iItem++)
	{
		EZ3HEADERMP3INFO headerfile;
		CMP3Info *mp3i = NULL;
		
		if (fread(&headerfile, 1, sizeof(headerfile), file) != sizeof(headerfile))
			goto err_loadEZ3;

		TCHAR *szFilename = (TCHAR*)malloc(sizeof(TCHAR)*(headerfile.nLenFilename+1));

		if (szFilename == NULL)
			goto err_loadEZ3;

		*szFilename = '\0';
		fread(szFilename, sizeof(TCHAR), headerfile.nLenFilename, file);
		szFilename[headerfile.nLenFilename] = '\0';

		tstring strfile(szFilename);
		mp3i = new CMP3Info(strfile, FALSE);

		if (mp3i == NULL)
			goto err_loadEZ3;

		// Set CoreMP3, init MP3Info
		mp3i->SetFileEZ3(&headerfile);

		// Ajout pour le nouveau format le SerialNumber 'DriveVolume'
		if (bNewFormat == TRUE)
		{
			DWORD dwSerial=0;
			fread(&dwSerial, 1, sizeof(dwSerial), file);
			mp3i->SetDriveVolumeSerialNumber(dwSerial);
		}


		// Read ID3v1 if exist
		if (mp3i->HaveID3v1())
		{
			struct ID3V1MP3Info id3i;
			
			if (fread(&id3i, 1, sizeof(id3i), file) != sizeof(id3i))
			{	delete mp3i;
				goto err_loadEZ3;
			}
			if (mp3i->SetID3v1(&id3i) == FALSE)
			{	delete mp3i;
				goto err_loadEZ3;
			}
			
		}

		// Read ID3v2 frames
		for (int i = 0; i<headerfile.nCountFrame; i++)
		{
			EZ3HEADERFRAME headerframe;
			
			if (fread(&headerframe, 1, sizeof(headerframe), file) != sizeof(headerframe))
			{	delete mp3i;
				goto err_loadEZ3;
			}

			
			LPTSTR str = (LPTSTR)malloc((headerframe.size+1)*sizeof(TCHAR));
			if (str)
			{
				fread(str, sizeof(TCHAR), headerframe.size, file);
				str[headerframe.size] = '\0';

				CMP3StringFrame *sf;
				int fid = headerframe.fid;

				if (fid == ID3TAG_NUMTAGS || !ID3Frames[fid].Supported)	// unknown TAG or unsupported
				{	delete mp3i;
					goto err_loadEZ3;
				}

				sf = new CMP3StringFrame(fid, str);
				
				if (sf == NULL)
				{	free(str);
					delete sf;
					delete mp3i;

					goto err_loadEZ3;
				}
				mp3i->AddFrame(sf, FALSE);

				//free(str); car nettoyage lors du destructor
			}



		}
		
		free(szFilename);
		
		int pos = m_minfo.AddInfoQuick(mp3i);
		int nImage =  mp3i->GetTagVersion();

		// Initialize item-specific LVITEM members.         
		// Add the item.         
		lvi.iItem = iItem; 
		lvi.iImage = nImage;
		lvi.iSubItem = 0;         
		lvi.lParam = (LPARAM)mp3i;    // item data  
		ListView_InsertItem(m_hwndList, &lvi);  
		
		m_nTotalLength += mp3i->GetPlayLength();

	}

	fclose(file);


	UpdateStatus();
	LV_Redraw(TRUE);
	LV_StatGenre();
	TV_Update();

	return;

err_loadEZ3:
	// vide la playlist incomplete ou endommage  ou mal charge
	ListView_DeleteAllItems(m_hwndList); 
	m_minfo.ResetAll();
	m_nTotalLength = 0;
	m_nSelLength   = 0;
	// reactualise le listview & treeview
	UpdateStatus();
	LV_Redraw(TRUE);
	LV_StatGenre();
	TV_Update();
	
	fclose(file);
	MessageBox(m_hDlg, GS(LNG_ERR_CORRUPTFILE), GS(LNG_ERROR), MB_OK|MB_ICONSTOP);
	return;
	
}


/* ************************************************************************ */
void CDlgPlaylist::LV_LoadM3U(TCHAR *filename)
{
	TCHAR szTemp[4096];
	FILE *filePlaylist;		
	BOOL bExtM3U;
	TCHAR *p;
	int iItem, nImage = 0;
	LV_ITEM lvi;
	int index;
	
	if ( (filePlaylist = _tfopen(filename, _T("r+t") )) == NULL)
		return;
	
	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 
	m_minfo.ResetAll();
	
    // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

	m_nTotalLength = 0;
	m_nSelLength   = 0;

	
	// Scan playlist Winamp
	for (iItem = 0; 
		_fgetts(szTemp, NUMELEMENTS(szTemp), filePlaylist);
		iItem++)
	{

		// 1st detect is extented playlist  ?
		if (iItem == 0)
		{	bExtM3U = (_tcsncmp(szTemp, _T("#EXTM3U"), 7) == 0 ? TRUE :FALSE);
			continue;
		}

		// Skip extended information if exist
		if (bExtM3U)
			if(_tcsncmp(szTemp, _T("#EXTINF"), 7)==0)
				_fgetts(szTemp, NUMELEMENTS(szTemp), filePlaylist);

		// Read info media filename	 
		p = szTemp + _tcslen(szTemp) -1;
		*p = '\0';
		
		tstring filename(szTemp);

		m_minfo.AddInfoDriveVolume(szTemp); 

		index = InsertFile(filename);

		if (index >= 0)		
		{
			CMP3Info *mp3i = m_minfo.GetInfo(index);

			nImage = mp3i->GetTagVersion();
			// Initialize item-specific LVITEM members.         
			// Add the item.         
			lvi.iItem = iItem; 
			lvi.iImage = nImage;
			lvi.iSubItem = 0;         
			lvi.lParam = (LPARAM)mp3i;    // item data  
			ListView_InsertItem(m_hwndList, &lvi);  
			
			m_nTotalLength += mp3i->GetPlayLength();
		}
	}

	fclose(filePlaylist);
	

	UpdateStatus();
	LV_Redraw(TRUE);
	LV_StatGenre();
	TV_Update();
}

/* ************************************************************************ */
void CDlgPlaylist::LV_LoadPLS(TCHAR *filename)
{
	TCHAR szTemp[4096];
	FILE *filePlaylist;		
	TCHAR *p, *szFile;
	int iItem, nImage=0;
	LV_ITEM lvi;
	int index;
	
	if ( (filePlaylist = _tfopen(filename, _T("r+t"))) == NULL)
		return;
	
	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 
	m_minfo.ResetAll();
	
    // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

	m_nTotalLength = 0;
	m_nSelLength   = 0;

	
	// Scan playlist Winamp
	for (iItem = 0; 
		_fgetts(szTemp, NUMELEMENTS(szTemp), filePlaylist);
		iItem++)
	{

		// 1st detect is extented playlist  ?
		if (iItem == 0)
		{	if(_tcsnccmp(szTemp, _T("[playlist]"), 10) != 0)
			{
				MessageBox(m_hDlg, GS(LNG_ERR_BADFILEPLS), GS(LNG_ERROR), MB_OK|MB_ICONSTOP);
				return;
			}
		}

		// Skip extended information if exist
		if(_tcsnccmp(szTemp, _T("File"), 4) !=0)
			continue;

		p = _tcschr(szTemp, '=');
		if (p==0)
			continue;
		else
			p++;	
		
		szFile = p;

		// Read info media filename	 
		p = szFile + _tcslen(szFile) -1;
		*p = '\0';
		
		tstring filename(szFile);

		m_minfo.AddInfoDriveVolume(szFile); 

		index = InsertFile(filename);
		
		if (index >= 0)		
		{
			CMP3Info *mp3i = m_minfo.GetInfo(index);
			
			nImage = mp3i->GetTagVersion();
			// Initialize item-specific LVITEM members.         
			// Add the item.         
			lvi.iItem = iItem; 
			lvi.iImage = nImage;
			lvi.iSubItem = 0;         
			lvi.lParam = (LPARAM)mp3i;    // item data  
			ListView_InsertItem(m_hwndList, &lvi);  
			
			m_nTotalLength += mp3i->GetPlayLength();
		}
	}

	fclose(filePlaylist);


	UpdateStatus();
	LV_Redraw(TRUE);
	LV_StatGenre();
	TV_Update();
}


/* ************************************************************************ */
int CDlgPlaylist::InsertFile(tstring& file)
{
	// Verif if file is *.MP3 format extension 
	// check the extensions
	if (CheckExtension(file))
	{
		CMP3Info *mp3i;
		mp3i = new CMP3Info(file);
		
		if (mp3i)
		{
			int pos = m_minfo.AddInfo(mp3i);
			
			return (pos);
		}
	}
	
	return (-1);
}

/* ************************************************************************ */
void CDlgPlaylist::LV_WriteEZ3(TCHAR* filename)
{
	FILE *file;		
	EZ3HEADER header;
	int i;
	int nCount = m_minfo.GetCount();
	int nCountDriveVolume = m_minfo.GetCountDriveVolume();
	
	if ( (file = _tfopen(filename, _T("wb"))) == NULL)
		return;

	_tcscpy(header.tag, _T("EZ3DATA2"));
	header.nCountFile = nCount;
	fwrite (&header, sizeof(header), 1, file);
	fwrite (&nCountDriveVolume, sizeof(nCountDriveVolume), 1, file);

	for (i = 0; i< nCountDriveVolume; i++)
	{
		DRIVEVOLUMESTRUCT *dvs;
		dvs = m_minfo.GetFileEZ3(i);
		fwrite(dvs, 1, sizeof(DRIVEVOLUMESTRUCT), file);
	}
	
	for (i = 0; i< nCount; i++)
	{
		CMP3Info *mp3i = m_minfo.GetInfo(i);
	
		if (mp3i)
		{
			EZ3HEADERMP3INFO hmi;
			DWORD dwSerial=0;

			mp3i->GetFileEZ3(&hmi);	

			// Save file info header CoreMP3
			fwrite (&hmi, sizeof(hmi), 1, file);	
			fwrite (mp3i->GetCFilename(), sizeof(TCHAR), hmi.nLenFilename, file);
			
			dwSerial = mp3i->GetDriveVolumeSerialNumber();
			fwrite(&dwSerial, 1, sizeof(dwSerial), file);
			

			// Save ID3v1 Tag
			if (mp3i->HaveID3v1())
			{
				struct ID3V1MP3Info *id3i = mp3i->GetID3v1();
				if (id3i)
					fwrite (id3i, sizeof(struct ID3V1MP3Info), 1, file);	
			}
			
			// Save ID3v2 Frames
			for (int j=0; j<mp3i->GetFrameCount(); j++)
			{
				CMP3Frame *mp3f = mp3i->GetIndexFrame(j);

				if (mp3f)
				{
					// Ne sauvegarde dans la plyalist EZ3 que les frame de type STRING
					// Compte uniquement les frames de type STRING uniquement
					int nFrameType = mp3f->GetType();

					if(nFrameType == ID3ENC_STRING)
					{
						tstring str = _T("");
						EZ3HEADERFRAME hfr;						

						str = ((CMP3StringFrame *)mp3f)->GetString();
						
						hfr.fid = mp3f->GetID();
						hfr.size = str.length();

						// Save frame header
						fwrite (&hfr, sizeof(hfr), 1, file);	
						// Save frame
						fwrite (str.c_str(), sizeof(TCHAR), hfr.size, file);
					}	
					
				}
				
			}

		
		}
	}

	fclose(file);
	
}

/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteM3U(TCHAR *filename)
{
	FILE *file;		// winamp.m3u playlist
	int i, nCount;
	LV_ITEM lvi;
	tstring s1, s2;
	
	if ( (file = _tfopen(filename, _T("w+t"))) == NULL)
		return FALSE;

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 
	 
	_ftprintf(file, _T("#EXTM3U\n"));

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;

		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		GetFrameInfo(mp3i, ID3TAG_TPE1, s1);	// Artist
		GetFrameInfo(mp3i, ID3TAG_TIT2, s2);	// Title
		
		_ftprintf(file, _T("#EXTINF,%d,%s - %s\n"),
			mp3i->GetPlayLength(),
			s1.c_str(),
			s2.c_str());

		s1 = mp3i->GetCFilename();
		_ftprintf(file, _T("%s\n"), s1.c_str());
	
	}
	fclose(file);

	return TRUE;
}


/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WritePLS(TCHAR *filePLS)
{
	FILE *file;		// winamp.m3u playlist
	int i, nCount;
	LV_ITEM lvi;
	tstring filename, artist, title;
	
	if ( (file = _tfopen(filePLS, _T("w+t") )) == NULL)
		return FALSE;

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 
	
	_ftprintf(file, _T("[playlist]\n") );

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) 
			continue;

		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		GetFrameInfo(mp3i, ID3TAG_TPE1, artist);	// Artist
		GetFrameInfo(mp3i, ID3TAG_TIT2, title);		// Title
		filename = mp3i->GetCFilename();			// filename
		
		_ftprintf(file, _T("File%d=%s\nTitle%d=%s - %s\nLength%d=%d\n"), 
			i+1, 
			filename.c_str(),
			i+1,
			artist.c_str(), 
			title.c_str(),
			i+1,
			mp3i->GetPlayLength());
			
		
	
	}
	_ftprintf(file, _T("NumberOfEntries=%i\nVersion=2"), nCount);
	fclose(file);

	return TRUE;
}

/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteHTML_Table(TCHAR* filename)
{
	FILE *file;		// playlist HTML format
	int i,k,nCount,nCountSelect;
	int iColArtist;
	LV_ITEM lvi;
	TCHAR *title, *p; 
	TCHAR szCol[8192];
	unsigned long uTotalSize = 0;
	long nTotalLength = 0;
	int nOldShowID3v1;

	if ( (file = _tfopen(filename, _T("w+t") )) == NULL)
		return FALSE;


	if (gParam.bExportBothID3)
		nOldShowID3v1 = gParam.nShowID3v1;


	title = _tcschr(filename, '\\');
	if (!title)
		title = filename;
	else
		title++;

	p = _tcschr(title, '.');
	if (p) 	*p = '\0';

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED;  

	// Detect Artist column for add hyperlink
	if (gParam.bExportHTML_Link==1) 
	{	for (i = 0; i < gParam.NbCol; i++)
		{	if(gParam.column_order[i]==0)	
			{	iColArtist = i;
				break;
			}
		}	
	}		
	
	// Header / Body / CSS
	LV_WriteHTML_CSS(file, title);
	
	
	// Initialize Table format
	_ftprintf(file, _T("<table border=1 cellpadding=5 cellspacing=0>\n"));
	_ftprintf(file, _T("<tr class=\"ColumnHeader\">\n"));
	
	// Table header , columns header
	_ftprintf(file, _T("<th>N</th>\n"));	
	for (i = 0; i < gParam.NbCol; i++)
	{	k = gParam.column_order[i];		//$ UseCol
		
		if ((gParam.bExportBothID3) && (gDefault_column[k].nID3 == 1))
		{
			_ftprintf(file, _T("<th>%s(id3v1)</th>\n"), gDefault_column[k].szName);
			_ftprintf(file, _T("<th>%s(id3v2)</th>\n"), gDefault_column[k].szName);
		}
		else
			_ftprintf(file, _T("<th>%s</th>\n"), gDefault_column[k].szName);
	}
	_ftprintf(file, _T("</tr>\n"));

	nCountSelect = 0;

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;
		
		if (!mp3i)
			continue;

		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		// New begin row
		_ftprintf(file, _T("<tr class=\"ColumnText\">\n"));


		nCountSelect++;
		uTotalSize   += (mp3i->GetFileSize() / 1024 / 1024);
		nTotalLength += mp3i->GetPlayLength();
		
		_ftprintf(file, _T("<td>%i</td>\n"), i+1);

		// Scan all columns
		for (k = 0; k < gParam.NbCol; k++)
		{
			int h = gParam.column_order[k];

			if ((gParam.bExportBothID3) && (gDefault_column[h].nID3 == 1))
			{
				//recupere info ID3v1
				gParam.nShowID3v1 = 0;  
				LV_WriteHTML_TableColumn(file, mp3i, k, iColArtist);

				//recupere info ID3v2
				gParam.nShowID3v1 = 1;  
				LV_WriteHTML_TableColumn(file, mp3i, k, iColArtist);
			}
			else
			{
				LV_WriteHTML_TableColumn(file, mp3i, k, iColArtist);
			}
				

		}

		// End begin row
		_ftprintf(file, _T("</tr>\n") );
	
	}


	// End table
	_ftprintf(file, _T("</table>\n<BR>\n"));
	
	
	// Pied de page resume / copyright
	LV_WriteHTML_Footer(file, szCol, uTotalSize, nTotalLength, nCountSelect);

	fclose(file);

	if (gParam.bExportBothID3)
		gParam.nShowID3v1 = nOldShowID3v1;


	return TRUE;
}

/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteHTML_Listing(TCHAR* filename)
{
	FILE *file;		// playlist HTML format
	int i,k,nCount;
	LV_ITEM lvi;
	TCHAR *title, *p; 
	TCHAR szCol[8192];
	unsigned long uTotalSize = 0;
	long nTotalLength = 0;
	int nCountSelect = 0;
	int nOldShowID3v1;
	
	if ( (file = _tfopen(filename, _T("w+t") )) == NULL)
		return FALSE;

	if (gParam.bExportBothID3)
		nOldShowID3v1 = gParam.nShowID3v1;
	

	title = _tcsrchr(filename, '\\');
	if (!title)
		title = filename;
	else
		title++;

	p = _tcsrchr(title, '.');
	if (p) 	*p = '\0';

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED;  


	// Header / Body / CSS
	LV_WriteHTML_CSS(file, title);


	// Contenu en ligne	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;

		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;
		
		nCountSelect++;
		uTotalSize   += (mp3i->GetFileSize() / 1024 / 1024); 
		nTotalLength += mp3i->GetPlayLength();

		if (gParam.bExportBothID3)
		{
			tstring artist = _T("");
			tstring title  = _T("");

			//recupere info ID3v1
			gParam.nShowID3v1 = 0;  
			GetFrameInfo(mp3i, ID3TAG_TPE1, artist);	// Artist ID3v1
			GetFrameInfo(mp3i, ID3TAG_TIT2, title);		// Title ID3v1

			// Artist & title
			if (gParam.bExportHTML_Link == 1) 
			{	_ftprintf(file, _T("<a href =\"%s\">(id3v1) %s - %s</a><BR>\n"),
					mp3i->GetCFilename(), 
					artist.c_str(), 
					title.c_str()); 
			}
			else
			{
				_ftprintf(file, _T("<B>(id3v1) %s - %s</B><BR>\n"),
					artist.c_str(), 
					title.c_str()); 
			}

			//recupere info ID3v2
			gParam.nShowID3v1 = 1;  
			GetFrameInfo(mp3i, ID3TAG_TPE1, artist);	// Artist ID3v1
			GetFrameInfo(mp3i, ID3TAG_TIT2, title);		// Title ID3v1

			// Artist & title
			if (gParam.bExportHTML_Link == 1) 
			{	_ftprintf(file, _T("<a href =\"%s\">(id3v2) %s - %s</a><BR>\n"),
					mp3i->GetCFilename(), 
					artist.c_str(), 
					title.c_str()); 
			}
			else
			{
				_ftprintf(file, _T("<B>(id3v2) %s - %s</B><BR>\n"),
					artist.c_str(), 
					title.c_str()); 
			}

		}
		else
		{
			tstring artist = _T("");
			tstring title  = _T("");
			GetFrameInfo(mp3i, ID3TAG_TPE1, artist);	// Artist
			GetFrameInfo(mp3i, ID3TAG_TIT2, title);		// Title

			// Artist & title
			if (gParam.bExportHTML_Link == 1) 
			{
				_ftprintf(file, _T("<a href =\"%s\">%s - %s</a><BR>\n"),
					mp3i->GetCFilename(), 
					artist.c_str(), 
					title.c_str()); 
			}
			else
			{
				_ftprintf(file, _T("<B>%s - %s</B><BR>\n"),
					artist.c_str(), 
					title.c_str()); 
			}
		}


		// Scan all columns except Artist and title
		for (k = 0; k < gParam.NbCol; k++)
		{
			int h  = gParam.column_order[k];
			
			if (h > 1)	
			{
				if ((gParam.bExportBothID3) && (gDefault_column[h].nID3 == 1))
				{
					//recupere info ID3v1
					gParam.nShowID3v1 = 0;  
					LV_GetInfoCol(mp3i, k, szCol); 	
					_ftprintf(file, _T("- %s (id3v1): %s<BR>\n"), gDefault_column[h].szName,szCol); 

					//recupere info ID3v2
					gParam.nShowID3v1 = 1;  
					LV_GetInfoCol(mp3i, k, szCol); 	
					_ftprintf(file, _T("- %s (id3v2): %s<BR>\n"), gDefault_column[h].szName,szCol); 
				}
				else
				{
					LV_GetInfoCol(mp3i, k, szCol); 	
					_ftprintf(file, _T("- %s: %s<BR>\n"), gDefault_column[h].szName,szCol); 
				}
			}
		}
		
		_ftprintf(file, _T("<BR>\n") );		// seperate line between songs
			
	}

	_ftprintf(file, _T("<BR>\n") );		// seperate line between songs

	// Pied de page resume / copyright
	LV_WriteHTML_Footer(file, szCol, uTotalSize, nTotalLength, nCountSelect);

	fclose(file);

	if (gParam.bExportBothID3)
		gParam.nShowID3v1 = nOldShowID3v1;
		
	return TRUE;
}


/* ************************************************************************* */
void CDlgPlaylist::LV_WriteHTML_TableColumn(FILE *file, CMP3Info *mp3i, int iCol, int iColArtist)
{
	TCHAR szTemp[8192];
	
	LV_GetInfoCol(mp3i, iCol, szTemp); 	
	
	int retVide = (_tcscmp(szTemp, _T(" "))==0) ||  (_tcslen(szTemp) == 0);
	
	if (gParam.bExportHTML_Link == 1 && iCol == iColArtist) 
	{
		tstring filename(_T(""));
		filename = mp3i->GetCFilename();
		
		if (!retVide)
			//  nowrap
			_ftprintf(file, _T("<td nowrap><a href =\"%s\">%s</a></td>\n"),
				filename.c_str(), szTemp);
		else
			_ftprintf(file, _T("<td><a href =\"%s\">&nbsp;</a></td>\n"), 
				filename.c_str());
	}
	else
	{
		if (!retVide)
			//  nowrap
			_ftprintf(file, _T("<td nowrap>%s</td>\n"), szTemp);
		else
			_ftprintf(file, _T("<td>&nbsp;</td>\n") );
	}


}




/* ************************************************************************* */
void CDlgPlaylist::LV_WriteHTML_CSS(FILE *file, TCHAR *title)
{
	TEMPLATESTRUCT *tpl;
	int nCSSStyle[4][2]    =  {{0,0},{0,1},{1,0},{1,1}};
	TCHAR *CSSText[MAX_CSS] =  { _T("TextPage"), _T("TitlePage"), _T("ColumnHeader"), _T("ColumnText")};

	
	// Header /Body structure
	_ftprintf(file, _T("<html>\n<head>\n<title>%s</title>\n"), title);
	_ftprintf(file, _T("<style type=\"text/css\">\n"));
	
	tpl = &gParam.Template[gParam.nCurrentTemplate];

	// Insert Style CSS
	for (int i = 0; i< MAX_CSS;i++)
	{
		int nStyle  = nCSSStyle[ tpl->css[i].nFontStyle ][0];
		int nWeight = nCSSStyle[ tpl->css[i].nFontStyle ][1];
	
		_ftprintf(file, _T(".%s {font-family:%s; font-size:%s; font-style:%s; font-weight:%s; color:#%02X%02X%02X; background-color:#%02X%02X%02X }\n"),
			CSSText[i],
			CSSFontName[tpl->css[i].nFontName],
			CSSFontSize[tpl->css[i].nFontSize],
			CSSFontStyle[nStyle],
			CSSFontWeight[nWeight],
			GetRValue(tpl->css[i].cColor),
			GetGValue(tpl->css[i].cColor),
			GetBValue(tpl->css[i].cColor),
			GetRValue(tpl->css[i].cColorBack),
			GetGValue(tpl->css[i].cColorBack),
			GetBValue(tpl->css[i].cColorBack));
			
	}
	
	_ftprintf(file, _T("</style>\n</head>\n<body class=\"TextPage\" link=#%02X%02X%02X>\n"),
		GetRValue(tpl->cLinkColor),
		GetGValue(tpl->cLinkColor),
		GetBValue(tpl->cLinkColor));
		
	_ftprintf(file, _T("<center><h1 class=\"TitlePage\">Playlist page:%s</h1></center>\n"), title);

}




/* ************************************************************************* */
void CDlgPlaylist::LV_WriteHTML_Footer(FILE *file, TCHAR *szCol, unsigned long uTotalSize, long nTotalLength, int nCountSelect)
{
	long nAvgLength = 0;
	
	// Description 
	_ftprintf(file, GS(LNG_STATUS_TOTALFILE), nCountSelect);
	_ftprintf(file, _T("<br>\n") );
	_ftprintf(file, GS(LNG_STATUS_TOTALSIZE), uTotalSize);
	_ftprintf(file, _T("<br>\n") );
	
	Time2Str(nTotalLength, szCol, FALSE);
	_ftprintf(file, GS(LNG_STATUS_TOTALLENGTH), szCol);
	_ftprintf(file, _T("<br>\n") );

	// Avoid Divide per zero
	if (nCountSelect == 0) nCountSelect =1;

	nAvgLength = nTotalLength / nCountSelect;
	Time2Str(nAvgLength, szCol, TRUE);
	_ftprintf(file, GS(LNG_STATUS_AVGLENGTH), szCol);
	_ftprintf(file, _T("<br>\n\n") );

	_ftprintf(file, _T("<hr>This list was created with <a href=\"http://perso.club-internet.fr/boyerjl\">EZ-Playlist for Winamp</a>") );
	_ftprintf(file, _T(" version 1.50b1, &copy;") );
	_ftprintf(file, _T("<a href=\"mailto:boyerjl@club-internet.fr\">") );
	_ftprintf(file, _T(" JL Boyer </a> france.\n\n") );

	_ftprintf(file, GS(LNG_HTML_DATE), GetFormattedDateTime());

	_ftprintf(file, _T("</body></html>\n"));
	
}


/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteCSV(TCHAR* filename)
{
	FILE *file;		// playlist CSV MS-EXCEL  (Comma-Separated-Values)
	int i,k,nCount;
	LV_ITEM lvi;
	TCHAR szCol[8192];
	TCHAR *p;


	if ( (file = _tfopen(filename, _T("w+t") )) == NULL)
		return FALSE;

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i < gParam.NbCol; i++)
	{	k = gParam.column_order[i];		// UseCol
		if (i != gParam.NbCol -1)	
			_ftprintf(file, _T("\"%s\","), gDefault_column[k].szName);
		else
			_ftprintf(file, _T("\"%s\"\n"), gDefault_column[k].szName);
	}

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;
		
		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		// Scan all columns
		for (k = 0; k < gParam.NbCol; k++)
		{
			LV_GetInfoCol(mp3i, k, szCol); 	

			p = szCol;

			_fputtc('"', file);
			while (*p != '\0')
			{
				if (*p == '"')
					_fputtc('"', file);

				_fputtc(*p, file);
				p++;
			}

			if (k != gParam.NbCol -1)
				_ftprintf(file, _T("\","));	// end part
			else
				_ftprintf(file, _T("\"\n"));	//new line (last column)

		}
	
	}
	
	fclose(file);
	return TRUE;
}


/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteASCII(TCHAR* filename)
{
	FILE *file;		// playlist ASCII text format
	int i,k,nCount;
	LV_ITEM lvi;
	TCHAR szCol[8192];

	if ( (file = _tfopen(filename, _T("w+t") )) == NULL)
		return FALSE;

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i < gParam.NbCol; i++)
	{	k = gParam.column_order[i];		// UseCol
		if (i != gParam.NbCol -1)	
			_ftprintf(file, _T("%s\t"), gDefault_column[k].szName);
		else
			_ftprintf(file, _T("%s\n"), gDefault_column[k].szName);
	}

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;
		
		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		// Scan all columns
		for (k = 0; k < gParam.NbCol; k++)
		{
			LV_GetInfoCol(mp3i, k, szCol); 	
			if (k != gParam.NbCol -1)
				_ftprintf(file, _T("%s\t"),szCol);
			else
				_ftprintf(file, _T("%s\n"), szCol);

		}
	
	}
	
	fclose(file);
	return TRUE;
}

/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteASCIIFixed(TCHAR* filename)
{
	FILE *file;		// playlist ASCII text format
	int i,k,nCount;
	LV_ITEM lvi;
	TCHAR szCol[8192];
	TCHAR szFormat[8192];
	int *nColMaxArray;
	int nLen;
	
	nCount = ListView_GetItemCount(m_hwndList);

	nColMaxArray = (int*)calloc(gParam.NbCol, sizeof(int));

	if (nColMaxArray == NULL)
		return FALSE;

	
	// Determine les largeurs max
	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;
		
		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		// Scan all columns
		for (k = 0; k < gParam.NbCol; k++)
		{
			LV_GetInfoCol(mp3i, k, szCol); 	
			
			nLen = _tcslen(szCol);
			
			if (nLen > nColMaxArray[k])
				nColMaxArray[k] =  nLen;
	
		}
	
	}
	
	// Si lles titres de colonnes sont plus grand que les informations
	for (i = 0; i < gParam.NbCol; i++)
	{
		k = gParam.column_order[i];	
		
		nLen = _tcslen(gDefault_column[k].szName);
		
		if (nLen > nColMaxArray[i])
			nColMaxArray[k] =  nLen;
	}


	
	if ( (file = _tfopen(filename, _T("w+t") )) == NULL)
		return FALSE;

	nCount = ListView_GetItemCount(m_hwndList);
	

	// Header
	nLen = 0;
	for (i = 0; i < gParam.NbCol; i++)
	{	k = gParam.column_order[i];	

		_stprintf(szFormat, _T("%%-%ds  | "), nColMaxArray[k]);
		_stprintf(szCol, szFormat, gDefault_column[k].szName);

		_ftprintf(file, szCol);

		if (i == gParam.NbCol -1)	
			_ftprintf(file, _T("\n"));

		nLen += _tcslen(szCol);
	}

	// Ligne separation header
	for (i = 0; i < gParam.NbCol; i++)
	{	k = gParam.column_order[i];	

		_stprintf(szFormat, _T("%%-%ds"), nColMaxArray[k]);
		_stprintf(szCol, szFormat, gDefault_column[k].szName);
		_tcsset(szCol, '-');
		_ftprintf(file,  _T("%s--+-"), szCol);

		if (i == gParam.NbCol -1)	
			_ftprintf(file, _T("\n"));

		nLen += _tcslen(szCol);
	}


	// Sauvegarde des colonnes
	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;
		
		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		// Scan all columns
		for (k = 0; k < gParam.NbCol; k++)
		{
			LV_GetInfoCol(mp3i, k, szCol); 	
			
			_stprintf(szFormat, _T("%%-%ds  | "), nColMaxArray[k]);
			_ftprintf(file, szFormat, szCol);

			if (k == gParam.NbCol -1)	
				_ftprintf(file, _T("\n"));

		}
	
	}
	
	fclose(file);
	return TRUE;
}



/* ************************************************************************ */
void CDlgPlaylist::LV_AddFiles()
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256];
	TCHAR szFile[16384];
	TCHAR szFileTitle[256]; 
	TCHAR *szFilter;
	TCHAR szFilterTmp[MAX_PATH];
	
	_tcscpy(szFilterTmp,  GS(LNG_FILTERADDFILE) );
	szFilter = szFilterTmp;
 
	SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), szDirName);
	*szFile = '\0'; 	
	
	memset(&ofn, 0, sizeof(OPENFILENAME)); 

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = NULL; //m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = szFile; 
	ofn.nMaxFile = sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.lpstrTitle = (LPCTSTR) GS(LNG_ADDFILE_TITLE); 
	ofn.Flags = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_ALLOWMULTISELECT | OFN_EXPLORER; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 	
	
	if (GetOpenFileName(&ofn) == 0) 
	{
		if (CommDlgExtendedError() == FNERR_BUFFERTOOSMALL)
			MessageBox(m_hDlg, GS(LNG_ADDFILE_TOOMANYFILE), GS(LNG_ADDFILE_ERROR), MB_OK|MB_ICONSTOP); 
		return;
	}

	else
	{
		TCHAR szCurDirName[MAX_PATH];
		TCHAR szPath[MAX_PATH];
		size_t size;
		int iItem, nImage=0;
		LV_ITEM lvi;
			
		// Initialize LVITEM members that are common to all items. 
		lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
		lvi.state = 0;     
		lvi.stateMask = 0; 
		lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

		GetCurrentDirectory(sizeof(szCurDirName), szCurDirName );
		size = _tcslen(szFile);

		m_minfo.AddInfoDriveVolume(szCurDirName); 

		if (_tcslen(szCurDirName) != size)
		{
			//Only one file selected
			
			tstring filename(szFile);

			int index = InsertFile(filename);
			
			if (index >= 0)		
			{	
				CMP3Info *mp3i = m_minfo.GetInfo(index);

				nImage = mp3i->GetTagVersion();

				iItem = ListView_GetItemCount(m_hwndList);
	
				lvi.iItem = iItem; 
				lvi.iImage = nImage;
				lvi.iSubItem = 0;         
				lvi.lParam = (LPARAM)mp3i;    // item data  
				ListView_InsertItem(m_hwndList, &lvi);  
					
				m_nTotalLength += mp3i->GetPlayLength();
			}
		}
		else
		{
			// multiselected files
			BOOL bExit = FALSE;
			TCHAR *p;

			_tcscpy(szPath, szFile);
			p = szPath +  size - 1;	
			if ( *p != '\\')
				_tcscat(szPath, _T("\\"));

			while (!bExit)
			{
				
				TCHAR *pFile = szFile + size + 1;
				int filesize = _tcslen(pFile);
				
				if (!filesize)
					bExit = TRUE;
				else
				{
					TCHAR szTemp[1024];
					
					int index;

					size += (filesize + 1);

					_tcscpy(szTemp, szPath);
					_tcscat(szTemp, pFile);
	
					tstring filename(szTemp); 

					index = InsertFile(filename);
				
					if (index >= 0)		
					{	
						CMP3Info *mp3i = m_minfo.GetInfo(index);

						nImage = mp3i->GetTagVersion();

						iItem = ListView_GetItemCount(m_hwndList);
			
						lvi.iItem = iItem; 
						lvi.iImage = nImage;
						lvi.iSubItem = 0;         
						lvi.lParam = (LPARAM)mp3i;    // item data  
						ListView_InsertItem(m_hwndList, &lvi);  
							
						m_nTotalLength += mp3i->GetPlayLength();
					}
	
				}
			
			
			}
	
		}

		UpdateStatus();
		LV_StatGenre();
		TV_Update();
	}

}



/* ************************************************************************ */
void CDlgPlaylist::LV_AddDirectory()
{
	static TCHAR szDir[MAX_PATH]= {0};
	LPMALLOC pMalloc; 

	// Get's the Shell's default allocator 
	if (SHGetMalloc(&pMalloc) == NOERROR) 
	{ 
		BROWSEINFO bi; 
		LPITEMIDLIST pidl; 
		
		if (szDir[0] == '\0')
			GetCurrentDirectory(MAX_PATH, szDir);

		// Zero the BROWSEINFO structure to start off with 
		ZeroMemory( &bi, sizeof(bi) ); 
		
		bi.hwndOwner      = m_hDlg;
		bi.pidlRoot       = NULL;
		bi.pszDisplayName = szDir;
		bi.ulFlags        = BIF_RETURNONLYFSDIRS; //BIF_STATUSTEXT|BIF_RETURNONLYFSDIRS; 
		bi.lpfn           = BrowseRecurseCallbackProc;		// Set Callback function 
		bi.lParam         = (LPARAM)szDir;
		bi.iImage         = 0;

		// This next call issues the dialog box 
		if ((pidl = ::SHBrowseForFolder(&bi)) != NULL) 
		{ 
			if (::SHGetPathFromIDList(pidl, szDir)) 
			{ 
				// At this point szBuffer contains the selected path 
				TCHAR *p;
				TCHAR szDirSelect[MAX_PATH];
				HCURSOR hcurOld;
			
				_tcscpy(szDirSelect, szDir);
				p = szDirSelect + _tcslen(szDirSelect) - 1;
				if (*p == '\\')	*p = '\0';

				LV_Redraw(FALSE);
				
				ST_CreateStatus(2);
				ST_ChangeText((LPTSTR)_T("Scanning..."), 0);
				ST_ChangeText((LPTSTR)_T(""), 1);
				
				
				hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));

				m_minfo.AddInfoDriveVolume(szDirSelect); 

				LV_AddDir2List(szDirSelect);

				SetCursor(hcurOld);
			
				LV_Redraw(TRUE);
				LV_StatGenre();
				TV_Update();
				UpdateStatus();	
			} 
			
			// Free the PIDL allocated by SHBrowseForFolder 
			pMalloc->Free(pidl); 
		} 
		
		// Release the shell's allocator 
		pMalloc->Release(); 
	} 

	ST_CreateStatus(1);
	UpdateStatus();
}

/* ************************************************************************ */
void CDlgPlaylist::LV_DropFiles(HDROP hFilesInfo)
{    
	// Handle to file info structure when files
	// are dropped on our application
	int nTotalFiles; // Number of files dropped on our application
	int nIndex;		// Counter of files
	TCHAR szFileName[MAX_PATH];  // Buffer for filename
	
	// get number of files dropped
	nTotalFiles = DragQueryFile(hFilesInfo, -1, NULL, 0);
	
	// No files quit stop drag
	if (nTotalFiles == 0)
	{	DragFinish(hFilesInfo);
		return;
	}
	
	LV_Redraw(FALSE);
	ST_ChangeText((LPTSTR)_T("Scanning..."), 0);
	ST_ChangeText((LPTSTR)_T(""), 1);
	HCURSOR hcurOld = SetCursor(LoadCursor(plugin.hDllInstance,	(nTotalFiles > 1) ?	MAKEINTRESOURCE(IDC_DRPFIL_MULTIPLE) : MAKEINTRESOURCE(IDC_DRPFIL_SINGLE)));
	
	gbCheckRecurseDir = TRUE;

	// add the file names to the listbox
	for (nIndex = 0; nIndex < nTotalFiles; nIndex++)
	{
		// get the next file name
		DragQueryFile(hFilesInfo, nIndex, szFileName, MAX_PATH*sizeof(TCHAR));

		DWORD attr = GetFileAttributes(szFileName);

		if (attr & FILE_ATTRIBUTE_DIRECTORY)
		{
			// add directory name to the playlist
			if (_tcscmp(_T("."), szFileName) != 0 && _tcscmp(_T(".."), szFileName) != 0)
			{
				m_minfo.AddInfoDriveVolume(szFileName); 
				LV_AddDir2List(szFileName);
			}
		}
		else
		{	// add mp3 file name to the playlist
			TCHAR *p = NULL;
			TCHAR szFilenamelowercase[MAX_PATH];
			_tcscpy(szFilenamelowercase, szFileName);
			CharLower(szFilenamelowercase);
            
			p = _tcsstr(szFilenamelowercase, _T(".mp3"));
            if (p) 
			{	tstring filename(szFileName);

				m_minfo.AddInfoDriveVolume(szFileName); 
				
				int index = InsertFile(filename);
				
				if (index >= 0)		
				{	
					LV_ITEM lvi;
					int iItem, nImage=0;

					CMP3Info *mp3i = m_minfo.GetInfo(index);

					nImage = mp3i->GetTagVersion();
					iItem = ListView_GetItemCount(m_hwndList);
					lvi.iItem = iItem; 
					lvi.iImage = nImage;
					lvi.iSubItem = 0;         
					lvi.lParam = (LPARAM)mp3i;    // item data  
					ListView_InsertItem(m_hwndList, &lvi);  
						
					m_nTotalLength += mp3i->GetPlayLength();
					ST_ChangeText((LPTSTR)szFileName, 1);
				}
			}
		}

	}

	// release memory Windows allocated for transferring 
	// filenames to app
	DragFinish(hFilesInfo);

	SetCursor(hcurOld);

	LV_UpdatePlaylist(); 
	LV_Redraw(TRUE);
	LV_StatGenre();
	TV_Update();
	
	ST_CreateStatus(1);
	UpdateStatus();	
}

/* ************************************************************************ */
void CDlgPlaylist::LV_AddDir2List(TCHAR* dir)
{    
	WIN32_FIND_DATA data;
	HANDLE hFile;
    TCHAR* buffer;
	
	buffer = (TCHAR*)malloc(MAX_PATH*sizeof(TCHAR));    
	_stprintf(buffer, _T("%s\\*"), dir);
    
	hFile = FindFirstFile(buffer, &data);

    if(hFile != INVALID_HANDLE_VALUE) 
	{        
		do 
		{
            if (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) 
			{
				if (gbCheckRecurseDir == FALSE)
					continue;

                if (_tcscmp(_T("."), data.cFileName) != 0 && _tcscmp(_T(".."), data.cFileName) != 0) 
				{
                    TCHAR* subDir;
					
					subDir = (TCHAR*)malloc(MAX_PATH*sizeof(TCHAR));
                    _stprintf(subDir, _T("%s\\%s"), dir, data.cFileName);
                    
					LV_AddDir2List(subDir);
                    
					free(subDir);
                }
            } 
			else 
			{
				TCHAR *p = NULL;
				
				TCHAR szFilenamelowercase[MAX_PATH];

				_tcscpy(szFilenamelowercase, data.cFileName);
				CharLower(szFilenamelowercase);
                
				p = _tcsstr(szFilenamelowercase, _T(".mp3"));
                if (p) 
				{
					LV_ITEM lvi;
					int iItem, nImage=0;
					TCHAR szTemp[256];
					int index;

					// Initialize LVITEM members that are common to all items. 
					lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
					lvi.state = 0;     
					lvi.stateMask = 0; 
					lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 
					
					_stprintf(szTemp, _T("%s\\%s"), dir, data.cFileName);

					tstring filename(szTemp); 

					index = InsertFile(filename);
				
					if (index >= 0)		
					{	
						CMP3Info *mp3i = m_minfo.GetInfo(index);
						
						nImage = mp3i->GetTagVersion();			
					
						iItem = ListView_GetItemCount(m_hwndList);

						lvi.iItem = iItem; 
						lvi.iImage = nImage;
						lvi.iSubItem = 0;         
						lvi.lParam = (LPARAM)mp3i;    // item data  
						ListView_InsertItem(m_hwndList, &lvi);  
							
						m_nTotalLength +=  mp3i->GetPlayLength();

						ST_ChangeText((LPTSTR)data.cFileName, 1);
					}
                }            
			}

		}while(FindNextFile(hFile, &data)); 

        FindClose(hFile);    
	}    
	
	free(buffer);
}





/* ************************************************************ */
void CDlgPlaylist::LV_Import()
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256]; 
	TCHAR szFile[256];
	TCHAR szFileTitle[256]; 
    TCHAR *szFilter;
  	TCHAR szFilterTmp[MAX_PATH];
	
	_tcscpy(szFilterTmp,  GS(LNG_FILTEROPENDIALOG_IMPORT) );
	szFilter = szFilterTmp;
   
	SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), (LPTSTR)szDirName);
	*szFile = '\0'; 

	memset(&ofn, 0, sizeof(OPENFILENAME)); 

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = szFile; 
	ofn.nMaxFile = sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.lpstrTitle = (LPCTSTR) GS(LNG_IMPORTPLAYLIST); 
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 

	if (GetOpenFileName(&ofn)) 
	{ 
		switch (ofn.nFilterIndex) 
		{
			case 1:
				LV_LoadM3U(szFile);
				break;
			case 2:
				LV_LoadPLS(szFile);
				break;
		}
	} 
}


/* ************************************************************ */
void CDlgPlaylist::LV_Export()
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256]; 
	TCHAR szFile[256];
	TCHAR szFileTitle[256]; 
	TCHAR *szFilter;
  	TCHAR szFilterTmp[MAX_PATH];
	
	_tcscpy(szFilterTmp,  GS(LNG_FILTEROPENDIALOG_EXPORT) );
	szFilter = szFilterTmp;
   
	SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), szDirName);
	*szFile = '\0'; 

	memset(&ofn, 0, sizeof(OPENFILENAME)); 

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = szFile; 
	ofn.nMaxFile = sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.lpstrTitle = (LPCTSTR) GS(LNG_EXPORTPLAYLIST); 
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 

	if (GetSaveFileName(&ofn)) 
	{ 
		switch (ofn.nFilterIndex) 
		{
		case 1:
			AddFileExtension(szFile, _T(".html")); 
			if (gParam.bExportHTML_Format == 0)
				LV_WriteHTML_Table(szFile);
			else
				LV_WriteHTML_Listing(szFile);
			break;
		case 2:
			AddFileExtension(szFile, _T(".m3u")); 
			LV_WriteM3U(szFile);
			break;
		case 3:
			AddFileExtension(szFile, _T(".pls")); 
			LV_WritePLS(szFile);
			break;
		case 4:
			AddFileExtension(szFile, _T(".txt")); 			
			LV_WriteASCII(szFile);
			break;
		case 5:
			AddFileExtension(szFile, _T(".txt")); 			
			LV_WriteASCIIFixed(szFile);
		case 6:
			AddFileExtension(szFile, _T(".csv")); 			
			LV_WriteCSV(szFile);
			break;
		}
	} 
}


/* ************************************************************ */
void CDlgPlaylist::AddFileExtension(TCHAR *szFile, TCHAR *szExt) 
{ 
	int nLen = _tcslen(szFile);
	int nLenExt  = _tcslen(szExt);

	if (nLen > nLenExt)
	{
		TCHAR *szFileExt = NULL;
		TCHAR *p = &szFile[nLen-nLenExt];

		// nLenExt+1: terminate string by '\0'
		szFileExt = (TCHAR*)malloc((nLenExt+1)*sizeof(TCHAR));		
		if (szFileExt == NULL)
		{	_tcscat(szFile, szExt);
			return;
		}
		_tcscpy(szFileExt, p);
		_tcslwr(szFileExt);

		// there is no extension so add it
		if (_tcscmp(szFileExt,szExt) != 0)
			_tcscat(szFile, szExt);

		free(szFileExt);
	}
	else
		_tcscat(szFile, szExt);

}

/* ************************************************************ */
BOOL CDlgPlaylist::doContextMenu(WPARAM wParam, LPARAM lParam) 
{ 
	HWND  hwndListView = (HWND)wParam; 
	HMENU hMenuLoad, hMenu; 

	if(hwndListView != GetDlgItem(m_hDlg, IDC_LVW_PLAYLIST)) 
		return FALSE; 

	hMenuLoad = LoadMenu(plugin.hDllInstance, MAKEINTRESOURCE(IDR_MENU1)); 
	
	// subMenu WinAMP
	hMenu = GetSubMenu(hMenuLoad, 1); 
	SetTextMenuPop(hMenu,POPMENU_PLAYLIST);
	UpdateMenu(hMenu); 

	TrackPopupMenu(hMenu, 
				  TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
				  LOWORD(lParam), 
				  HIWORD(lParam), 
				  0, 
				  m_hDlg, 
				  NULL); 

	DestroyMenu(hMenuLoad); 

	return TRUE; 
} 

/* ************************************************************ */
void CDlgPlaylist::MenuButton(int nButton) 
{ 
	HMENU hMenuLoad, hMenu; 
	HWND hwndButton;
	RECT rect;
	POINT pt;

	hMenuLoad = LoadMenu(plugin.hDllInstance, MAKEINTRESOURCE(IDR_MENU1)); 
	
	// subMenu File
	switch (nButton)
	{ 
	case IDC_BUTTON_FILE:
		hMenu = GetSubMenu(hMenuLoad, 0); 
		SetTextMenuPop(hMenu,POPMENU_FILE);
		break;
	case IDC_BUTTON_ADD:
		hMenu = GetSubMenu(hMenuLoad, 3); 
		SetTextMenuPop(hMenu,POPMENU_ADDFILE);
		break;
	case IDC_BUTTON_OPTION:
		{
			int i;

			hMenu = GetSubMenu(hMenuLoad, 4); 
			SetTextMenuPop(hMenu,POPMENU_SEARCH_OPTION);
			for (i=0; i<6; i++)
			{
				CheckMenuItem(hMenu, ID_SEARCH_ARTIST + i,  
					(gParam.nOptionSearch & (1<<i) )? 
					MF_BYCOMMAND | MF_CHECKED :
					MF_BYCOMMAND | MF_UNCHECKED);
				
			}
			
		}
		break;
	default:
		return;
	}

	hwndButton = GetDlgItem(m_hDlg, nButton); 
	GetClientRect(hwndButton, &rect);
	pt.x  = rect.left;
	pt.y  = rect.bottom;
	ClientToScreen(hwndButton, &pt);

	TrackPopupMenu(hMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
		pt.x, pt.y, 0, m_hDlg, NULL); 

	DestroyMenu(hMenuLoad); 

} 

/* ************************************************************ */
void CDlgPlaylist::MenuCheckItem(int nItem)
{
	gParam.nOptionSearch ^=  1<<nItem;
}

/* ************************************************************ */
void CDlgPlaylist::UpdateMenu(HMENU hMenu) 
{ 
	CheckMenuItem(hMenu, ID_PLAYLIST_SHOWDUPLICATE,  (m_bShowDuplicate == TRUE ? MF_BYCOMMAND | MF_CHECKED: MF_BYCOMMAND | MF_UNCHECKED));
} 

/* ************************************************************ */
void CDlgPlaylist::winamp_enqueue_to_playlist(BOOL bPlay)
{
	LVITEM lvi; 
	int i,nCount;
	char *file;
	COPYDATASTRUCT cds;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) continue;

		if ((lvi.state & LVIS_SELECTED)== LVIS_SELECTED)
		{
			if (bPlay)
				SendMessage(plugin.hwndParent, WM_USER, (WPARAM)NULL, 101);

			// Ajoute a la playlist la playlist Test.m3u identique pour un MP3 Titi.mp3
			
			file = (char*)mp3i->GetCFilename();

			cds.dwData = 100;
			cds.lpData = (void*)file;
			cds.cbData = _tcslen((TCHAR*)cds.lpData)+1;
			SendMessage(plugin.hwndParent, WM_COPYDATA, (WPARAM)NULL,(LPARAM)&cds);

			if (bPlay)
			{
				// Set the 1st song and play
				SendMessage(plugin.hwndParent, WM_USER, (WPARAM)0, 121);
				SendMessage(plugin.hwndParent, WM_USER, (WPARAM)NULL, 102);
			}
		}
		
	}

}

/* ************************************************************ */
void CDlgPlaylist::winamp_enqueue_all_to_playlist(BOOL bPlay)
{
	int i, nCount;
	LV_ITEM lvi;
	COPYDATASTRUCT cds;
	int iItem;

	iItem = ListView_GetNextItem(m_hwndList, -1, LVNI_ALL | LVNI_SELECTED); 

	nCount = ListView_GetItemCount(m_hwndList);

	lvi.mask =  LVIF_PARAM; 

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) continue;

		cds.dwData = 100;
		cds.lpData = (void*)mp3i->GetCFilename();
		cds.cbData = _tcslen((TCHAR*)cds.lpData)+1;
		SendMessage(plugin.hwndParent, WM_COPYDATA, (WPARAM)NULL,(LPARAM)&cds);
	}

	if (bPlay)
	{
		SendMessage(plugin.hwndParent, WM_USER, (WPARAM)0, 121);
		SendMessage(plugin.hwndParent, WM_USER, (WPARAM)NULL, 102);
	}


}


/* ************************************************************ */
void CDlgPlaylist::LV_SelectAll(BOOL bSelect)
{
	// See ST_UpdateLengthSelected
	m_bRefreshSelLength = FALSE;

	ListView_SetItemState(m_hwndList, -1, (bSelect ? LVIS_SELECTED : 0) , LVIS_SELECTED);  

	m_bRefreshSelLength = TRUE;;
}

/* ************************************************************ */
void CDlgPlaylist::LV_SelectToggle(void)
{
	LVITEM lvi; 
	int i,nCount;

	m_bRefreshSelLength = FALSE;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_STATE;
	lvi.stateMask = LVIS_SELECTED; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		ListView_SetItemState(m_hwndList, i, (lvi.state & LVIS_SELECTED)? 0: LVIS_SELECTED , LVIS_SELECTED);
	}

	m_bRefreshSelLength = TRUE;
	ST_UpdateLengthSelected(); 

}

/* ************************************************************ */
void CDlgPlaylist::LV_SelectWithSame(int nCol)
{
	LV_ITEM lvi;
	int i, nCount, ret;
	BOOL bSame;
	CMP3Info *mp3i1, *mp3i2;
	tstring str1, str2;
	
	// retreive information for the item selected
    i = ListView_GetNextItem(m_hwndList, -1, LVNI_SELECTED); 
    if (i < 0) 
		return;

	lvi.iItem = i; 
    lvi.iSubItem = 0; 
    lvi.mask = LVIF_PARAM; 

    ret = ListView_GetItem(m_hwndList, &lvi); 
	mp3i1 = (CMP3Info *)lvi.lParam;

	if (!mp3i1) return;
	
	// Unselect all item before
	LV_SelectAll(FALSE);
	m_bRefreshSelLength = FALSE;

	// Scan all item in the listview
	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		mp3i2 = (CMP3Info *)lvi.lParam;
		

		// Check the good column ?
		bSame = FALSE;
		switch(nCol)
		{
		case 0: // Artist
			GetFrameInfo(mp3i1, ID3TAG_TPE1, str1);
			GetFrameInfo(mp3i2, ID3TAG_TPE1, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;
		case 1: // Title
			GetFrameInfo(mp3i1, ID3TAG_TIT2, str1);
			GetFrameInfo(mp3i2, ID3TAG_TIT2, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;

		case 2: // Genre
			GetFrameInfo(mp3i1, ID3TAG_TCON, str1);
			GetFrameInfo(mp3i2, ID3TAG_TCON, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;

		case 3: // Year
			GetFrameInfo(mp3i1, ID3TAG_TYER, str1);
			GetFrameInfo(mp3i2, ID3TAG_TYER, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;
		
		case 4: // Album
			GetFrameInfo(mp3i1, ID3TAG_TALB, str1);
			GetFrameInfo(mp3i2, ID3TAG_TALB, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;
		
		case 5: // Comment
			GetFrameInfo(mp3i1, ID3TAG_COMM, str1);
			GetFrameInfo(mp3i2, ID3TAG_COMM, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;

		case 6: // Mode
			bSame = (mp3i1->GetMode() == mp3i2->GetMode()); 
			break;
		
		case 7: // Bitrate
			bSame = (mp3i1->GetBitrate() == mp3i2->GetBitrate()); 
			break;
		
		case 8: // Frequency
			bSame = (mp3i1->GetFrequency() == mp3i2->GetFrequency());
			break;
		
		case 9: // MPEG Layer
			{
				TCHAR mpeglayer1[100], mpeglayer2[100];
				mp3i1->GetMPEGVersionLayer(mpeglayer1);
				mp3i2->GetMPEGVersionLayer(mpeglayer2);
				ret = _tcscmp(mpeglayer1, mpeglayer2); 
				bSame = (ret == 0);
		
			}
			break;
		
		case 10: // Length Time
			bSame = (mp3i1->GetPlayLength() == mp3i2->GetPlayLength());
			break;
		
		case 11: // File size
			bSame = (mp3i1->GetMP3Size() == mp3i2->GetMP3Size());
			break;
		
		case 12: // File name
			str1= mp3i1->GetCFilename();
			str2= mp3i2->GetCFilename();
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;
		
		default:
			bSame = FALSE;
			break;
		}

		
		ListView_SetItemState(m_hwndList, i, ((bSame)?LVIS_SELECTED:0), LVIS_SELECTED);
	}


	m_bRefreshSelLength = TRUE;
	ST_UpdateLengthSelected(); 
}

/* ************************************************************ */
void CDlgPlaylist::LV_SelectCrop(void)
{
	LVITEM lvi; 
	int i,j, nCount, nSelCount;
	
	nSelCount = ListView_GetSelectedCount(m_hwndList);
	if (nSelCount < 1)
		return;

	LV_Redraw(FALSE);

	HCURSOR hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));


	nCount = ListView_GetItemCount(m_hwndList);
	
	CProgressBar *pb;
	pb = new CProgressBar(m_hDlg, m_hwndStat);
	pb->Init(plugin.hDllInstance, nCount);
	pb->Show();

	j = i = 0;
	while (nCount > nSelCount)
	{
		pb->StepIt(j);
		j++;
		
		ZeroMemory(&lvi, sizeof(lvi)); 
		lvi.mask = LVIF_STATE | LVIF_PARAM; 
		lvi.stateMask = LVIS_SELECTED; 
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		if ((lvi.state & LVIS_SELECTED)== LVIS_SELECTED)
			// skip select item
			i++;		
		else
		{
			CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

			if (mp3i) 
			{
				// delete and clear data for unselect item
				int pos = m_minfo.FindInfo(mp3i);
				
				if (pos >=0)
				{
					ListView_DeleteItem(m_hwndList, i);
					m_minfo.DeleteInfo(pos);
					

					nCount--;
				}
			}	
		}
	}

	
	LV_Redraw(TRUE);
	SetCursor(hcurOld);

	delete pb;	// destroy progress bar
	ST_CreateStatus(1);
	ST_UpdateLengthSelected();

	LV_UpdateStatus();
	LV_StatGenre();
	TV_Update();

	
}

/* ************************************************************ */
void CDlgPlaylist::LV_SelectDel(void)
{
	LVITEM lvi; 
	int i,j,  nTotalProgress, nSelCount, nCount;
	
	
	nSelCount = ListView_GetSelectedCount(m_hwndList);
	nCount = ListView_GetItemCount(m_hwndList);
	

	if (nSelCount < 1)
		return;

	LV_Redraw(FALSE);

	HCURSOR hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));



	CProgressBar *pb;
	pb = new CProgressBar(m_hDlg, m_hwndStat);

	if (nCount>nSelCount)
		nTotalProgress = nCount-nSelCount;
	else
		nTotalProgress = nCount;
	
	pb->Init(plugin.hDllInstance, nTotalProgress);
	pb->Show();

	
	i = 0;
	j = 0;
	while (nSelCount > 0)
	{
		pb->StepIt(j++);
		
		
		ZeroMemory(&lvi, sizeof(lvi)); 
		lvi.mask = LVIF_STATE | LVIF_PARAM; 
		lvi.stateMask = LVIS_SELECTED; 
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		if ((lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
		{
			// skip unselect item
			i++;
		}
		else
		{
			CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

			if (mp3i) 
			{
				// delete and clear data for unselect item
				int pos  = m_minfo.FindInfo(mp3i);
				
				if (pos >=0)
				{
					ListView_DeleteItem(m_hwndList, i);
					m_minfo.DeleteInfo(pos);
				

					nSelCount--;
				}
			}	
		}
	}

	// Selected the next item
	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_STATE; 
	lvi.stateMask = LVIS_SELECTED; 
	lvi.iItem  = i;
	ListView_SetItemState(m_hwndList, i, LVIS_SELECTED , LVIS_SELECTED);

	LV_Redraw(TRUE);
	SetCursor(hcurOld);

	delete pb;	// destroy progress bar
	ST_CreateStatus(1);
	ST_UpdateLengthSelected();
	
	LV_UpdateStatus();
	LV_StatGenre();
	TV_Update();
}



/* ************************************************************ */
void CDlgPlaylist::LV_SelectRemoveFile(void)
{
	LVITEM lvi; 
	int i, nSelCount;
	
	nSelCount = ListView_GetSelectedCount(m_hwndList);
	if (nSelCount < 1)
		return;


	if (MessageBox( m_hDlg, GS(LNG_CONFIRM_REMOVEFILE), _T("Warning"), MB_YESNO | MB_ICONWARNING ) == IDNO)
		return;


	LV_Redraw(FALSE);

	i = 0;
	while (nSelCount > 0)
	{
		ZeroMemory(&lvi, sizeof(lvi)); 
		lvi.mask = LVIF_STATE | LVIF_PARAM; 
		lvi.stateMask = LVIS_SELECTED; 
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		if ((lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
		{
			// skip unselect item
			i++;
		}
		else
		{
			nSelCount--;
			
			CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

			if (mp3i) 
			{
				LPCTSTR lpFileName;

				lpFileName = mp3i->GetCFilename();
					 
				if (DeleteFile(lpFileName) == 0)
				{
					// Delete file failed, show message error
					LPVOID lpMsgBuf;
					TCHAR szMsgErr[1024];

					FormatMessage( 
						FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						GetLastError(),
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
						(LPTSTR) &lpMsgBuf,
						0,
						NULL);
					
					_stprintf(szMsgErr, GS(LNG_ERROR_REMOVEFILE), lpFileName);
					_tcscat(szMsgErr, _T("\n\n"));
					_tcscat(szMsgErr, (TCHAR*)lpMsgBuf);
					_tcscat(szMsgErr, _T("\n"));
					

					// Display the string.
					MessageBox( m_hDlg, (LPCTSTR)szMsgErr, GS(LNG_ERROR), MB_OK | MB_ICONINFORMATION );
					
					// Free the buffer.
					LocalFree( lpMsgBuf );

					i++;	// va au suivant
				}
				else
				{
					// delete and clear data for unselect item
					int pos  = m_minfo.FindInfo(mp3i);
					
					if (pos >=0)
					{
						m_minfo.DeleteInfo(pos);
						ListView_DeleteItem(m_hwndList, i);
					}
				}	

			}


		}
	}


	LV_Redraw(TRUE);

	ST_UpdateLengthSelected();
	LV_UpdateStatus();
	LV_StatGenre();
	TV_Update();
}




/* ************************************************************ */
void CDlgPlaylist::LV_UpdateStatus(void)
{
	LVITEM lvi; 
	int i,nCount;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_PARAM; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	m_nTotalLength = 0;

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) continue;

		m_nTotalLength += mp3i->GetPlayLength();
	}

	UpdateStatus();

}

/* ************************************************************ */
void CDlgPlaylist::LV_StatGenre(void)
{
	int i, idGenre;
	CMP3Info *mp3i;
	tstring genre;

	for (i = 0; i < ID3TAG_MAX_GENRE+2; i++ )
	{
		gStatGenre[i].bCheck = FALSE;
		gStatGenre[i].nCount = 0;
	}
	
	// Scan all playlist  from memory 
	for (i=0; i < m_minfo.GetCount(); i++)
	{
		mp3i = m_minfo.GetInfo(i);
		
		genre.assign(_T(""));
		GetFrameInfo(mp3i, ID3TAG_TCON, genre);
		
		idGenre = GenreStr2Int(genre.c_str());
		
		if (idGenre >= 0 && idGenre < ID3TAG_MAX_GENRE+2)
		{
			gStatGenre[idGenre].bCheck = TRUE;
			gStatGenre[idGenre].nCount++;
		}

	}


	LV_DetermineFilter();
	LV_TitleWindowFilter();
}

/* ************************************************************ */
void CDlgPlaylist::LV_UpdatePlaylist()
{
	TCHAR szSearch[40];
	LVITEM lvi; 
	int i, iItem;
	BOOL bFind;

	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 
		
    // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

	m_nTotalLength = 0;
	
	// Scan all playlist  from memory 
	GetDlgItemText(m_hDlg, IDC_EDIT_SEARCH, (LPTSTR)szSearch, 40);


	if (_tcslen(szSearch) != 0)
	{
		// /////////////////////////
		// Filter with search string 
		// /////////////////////////

		// Match case checked
		if (!(gParam.nOptionSearch & 32))
			_tcslwr(szSearch);

		iItem = 0;

		for (i=0; i < m_minfo.GetCount(); i++)
		{
			tstring artist =_T("");
			tstring title =_T(""); 
			tstring album =_T(""); 
			tstring comment =_T(""); 
			tstring filename =_T("");
			tstring genre= _T("");
						

			CMP3Info *mp3i = m_minfo.GetInfo(i);
			
			filename = mp3i->GetCFilename();
			GetFrameInfo(mp3i, ID3TAG_TPE1, artist);
			GetFrameInfo(mp3i, ID3TAG_TIT2, title);
			GetFrameInfo(mp3i, ID3TAG_TALB, album);
			GetFrameInfo(mp3i, ID3TAG_COMM, comment);
			GetFrameInfo(mp3i, ID3TAG_TCON, genre);
	
			bFind = FALSE;

			
			tstring artistlwr(artist.c_str());
			tstring titlelwr(title.c_str()); 
			tstring albumlwr(album.c_str()); 
			tstring commentlwr(comment.c_str()); 
			tstring filenamelwr(filename.c_str());
			

			// compare item in lowercase  
			if (!(gParam.nOptionSearch & 32))
			{
				_tcslwr((TCHAR*)artistlwr.c_str());
				_tcslwr((TCHAR*)titlelwr.c_str());
				_tcslwr((TCHAR*)albumlwr.c_str());
				_tcslwr((TCHAR*)commentlwr.c_str());
				_tcslwr((TCHAR*)filenamelwr.c_str());
			}

			if( (gParam.nOptionSearch & 1)  && _tcsstr(artistlwr.c_str(), szSearch)  != 0 ||
				(gParam.nOptionSearch & 2)  && _tcsstr(titlelwr.c_str(), szSearch)   != 0 ||
				(gParam.nOptionSearch & 4)  && _tcsstr(albumlwr.c_str(), szSearch) != 0 ||
				(gParam.nOptionSearch & 8)  && _tcsstr(commentlwr.c_str(), szSearch) != 0 ||
				(gParam.nOptionSearch & 16) && _tcsstr(filenamelwr.c_str(), szSearch) != 0 )
					bFind = TRUE;
			
			if (LV_Filter(mp3i) && bFind)
			{
			
				int nImage = mp3i->GetTagVersion();

				// Initialize item-specific LVITEM members.         
				// Add the item.         
				lvi.iItem = iItem; 
				lvi.iSubItem = 0;         
				lvi.iImage = nImage; 
				lvi.lParam = (LPARAM)mp3i;    // item data  
				ListView_InsertItem(m_hwndList, &lvi);  
				
				m_nTotalLength += mp3i->GetPlayLength();
				iItem++;
			}

		}

	}
	else
	{
		// /////////////////////////
		// No Filter search
		// /////////////////////////

		
		iItem = 0;
		for (i=0; i < m_minfo.GetCount(); i++)
		{
			
			CMP3Info *mp3i = m_minfo.GetInfo(i);

			if (LV_Filter(mp3i))
			{
				int nImage = mp3i->GetTagVersion();
				
				// Initialize item-specific LVITEM members.         
				// Add the item.         
				lvi.iItem = iItem; 
				lvi.iSubItem = 0;         
				lvi.iImage = nImage; 
				lvi.lParam = (LPARAM)mp3i;    // item data  
				ListView_InsertItem(m_hwndList, &lvi);  
				
				m_nTotalLength += mp3i->GetPlayLength();
				iItem++;
			}
			else
				iItem = iItem;

			
		}

	}


	LV_UpdateStatus();
	LV_Redraw(TRUE);
}


/* ************************************************************ */
void CDlgPlaylist::LV_DetermineFilter()
{
	m_nFilterBy = NO_FILTER;

	// Filter by genre ?
	int idGenre;
	BOOL bFilterGenre = FALSE;
	
	for (idGenre=0 ;idGenre<ID3TAG_MAX_GENRE+2; idGenre++)
	{
		if (gStatGenre[idGenre].bCheck == FALSE && gStatGenre[idGenre].nCount != 0)
		{	bFilterGenre = TRUE;
			break;
		}
	}
	
	if (bFilterGenre == TRUE)
		m_nFilterBy = FILTER_GENRE;

	// Filter by Artist
	if (!m_filterArtist.empty())
		m_nFilterBy |= FILTER_ARTIST;

	// Filter by Album
	if (!m_filterAlbum.empty())
		m_nFilterBy |= FILTER_ALBUM;

	// Filter by Comment
	if (!m_filterComment.empty())
		m_nFilterBy |= FILTER_COMMENT;

}


/* ************************************************************ */
BOOL CDlgPlaylist::LV_Filter(CMP3Info *mp3i)
{
	if (m_nFilterBy == NO_FILTER)
		return (TRUE);

	BOOL bFiltered = FALSE;
	
	// Genre OK ?
	if (m_nFilterBy & FILTER_GENRE)
	{
		tstring genre;
		GetFrameInfo(mp3i, ID3TAG_TCON, genre);
		int idGenre = GenreStr2Int(genre.c_str());
	
		if (gStatGenre[idGenre].bCheck == FALSE)
			return(FALSE);
	}
	// Artist OK ?
	if (m_nFilterBy & FILTER_ARTIST)
	{
		tstring artist;
		GetFrameInfo(mp3i, ID3TAG_TPE1, artist);
	
		if (artist.empty())
			return(FALSE);
		if (m_filterArtist.compare(artist))
			return(FALSE);
	}

	// Album OK ?
	if (m_nFilterBy & FILTER_ALBUM)
	{
		tstring album;
		GetFrameInfo(mp3i, ID3TAG_TALB, album);
	
		if (album.empty())
			return(FALSE);
		if (m_filterAlbum.compare(album))
			return(FALSE);
	}

	// Comment OK ?
	if (m_nFilterBy & FILTER_COMMENT)
	{
		tstring comment;
		GetFrameInfo(mp3i, ID3TAG_COMM, comment);
	
		if (comment.empty())
			return(FALSE);
		if (m_filterComment.compare(comment))
			return(FALSE);
	}

	// ID3Tag version OK ?
	if (m_nFilterBy & FILTER_ID3TAG)
	{
		int nID3 = mp3i->GetTagVersion();
		
		if (m_filterID3Tag != nID3)
			return(FALSE);
	}

	// Mode version OK ?
	if (m_nFilterBy & FILTER_MODE)
	{
		int nMode = mp3i->GetMode();
		
		if (m_filterMode != nMode)
			return(FALSE);
	}


	return(TRUE);
}


/* ************************************************************ */
void CDlgPlaylist::LV_TitleWindowFilter()
{
	TCHAR szTitle[MAX_PATH] = {0};
	
	_tcscat(szTitle, szAppName);
	
	if (m_nFilterBy == NO_FILTER)
	{	SetWindowText(m_hDlg,(LPCTSTR)szTitle);
		return;
	}

	_tcscat(szTitle, _T(" ("));
	
	if (m_nFilterBy & FILTER_GENRE)
	{	_tcscat(szTitle, GS(LNG_GENRE));		_tcscat(szTitle, _T(", "));	}
	
	if (m_nFilterBy & FILTER_ARTIST)
	{	_tcscat(szTitle, GS(LNG_ARTIST));		_tcscat(szTitle, _T(", "));	}

	if (m_nFilterBy & FILTER_ALBUM)
	{	_tcscat(szTitle, GS(LNG_ALBUM));		_tcscat(szTitle, _T(", "));	}

	if (m_nFilterBy & FILTER_COMMENT)
	{	_tcscat(szTitle, GS(LNG_COMMENT));		_tcscat(szTitle, _T(", "));	}

	if (m_nFilterBy & FILTER_ID3TAG)
	{	_tcscat(szTitle, GS(LNG_ID3TAG));		_tcscat(szTitle, _T(", "));	}
	
	if (m_nFilterBy & FILTER_MODE)
	{	_tcscat(szTitle, GS(LNG_MODE));		_tcscat(szTitle, _T(", "));	}


	int nLen = _tcslen(szTitle);
	TCHAR *p =  &szTitle[nLen-2];
	*p++ = ')';
	*p = '\0';

	SetWindowText(m_hDlg,(LPCTSTR)szTitle);
}


/* ************************************************************ */
void CDlgPlaylist::LV_ShowDuplicate(void)
{
	LVITEM lvi; 
	int iItem = 0;
	int i,j;
	CMP3Info *mp3i1, *mp3i2;
	int *pDuplicates = NULL;
	int nCount = m_minfo.GetCount();
	BOOL bFindArtistTitle;
	BOOL bFindFilename;
	BOOL bFindGenre;
	BOOL bFindTime;
	BOOL bFindFilesize;
	BOOL bFind;
	tstring artist1 =_T("");
	tstring title1 =_T("");
	tstring genre1 = _T("");
	int delta;
	CProgressBar *pb;

	
	if (nCount < 1)
		return;

	pDuplicates = (int*)calloc(nCount, sizeof(int));

	if (pDuplicates == NULL)
		return;
	
	HCURSOR hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));

	// Force le   reaffichage
	LV_Redraw(TRUE);

	pb = new CProgressBar(m_hDlg, m_hwndStat);
	pb->Init(plugin.hDllInstance, nCount);
	pb->Show();
	
	// /////////////////
	// Detect duplicates
	for (i=0; i<nCount; i++)
	{
	
		pb->StepIt(i);
		
		if (pDuplicates[i]==1)
			continue;
	
		mp3i1 = m_minfo.GetInfo(i);


		// Artist & Title
		if (gParam.nTypeDuplicate & 1)
		{		
			GetFrameInfo(mp3i1, ID3TAG_TPE1, artist1);
			GetFrameInfo(mp3i1, ID3TAG_TIT2, title1);
		}
		// Genre
		if (gParam.nTypeDuplicate & 4)
			GetFrameInfo(mp3i1, ID3TAG_TCON, genre1);


		for (j = i + 1; j<nCount; j++)
		{
			mp3i2 = m_minfo.GetInfo(j);

			bFindArtistTitle = TRUE;
			bFindFilename = TRUE;
			bFindGenre = TRUE;
			bFindTime = TRUE;
			bFindFilesize = TRUE;
			bFind = TRUE;

			// Artist & Title
			if (gParam.nTypeDuplicate & 1)
			{	
				tstring artist2 = _T("");
				tstring title2 = _T("");
				GetFrameInfo(mp3i2, ID3TAG_TPE1, artist2);
				GetFrameInfo(mp3i2, ID3TAG_TIT2, title2);

				if ((artist1.compare(artist2) == 0) && (title1.compare(title2) == 0)) 
					bFindArtistTitle = TRUE;
				else
					bFindArtistTitle = FALSE;
			}

			// Filename
			if (gParam.nTypeDuplicate & 2)
			{	
				if ( (_tcsicmp(mp3i1->GetCFilename(), mp3i2->GetCFilename())) == 0)
					bFindFilename = TRUE;
				else
					bFindFilename = FALSE;
			}


			// Genre
			if (gParam.nTypeDuplicate & 4)
			{	
				tstring genre2 = _T("");
				GetFrameInfo(mp3i2, ID3TAG_TCON, genre2);

				if ((genre1.compare(genre2) == 0)) 
					bFindGenre = TRUE;
				else
					bFindGenre = FALSE;
			}

			
			// Time
			if (gParam.nTypeDuplicate & 8)
			{	
				delta = abs(mp3i1->GetPlayLength() - mp3i2->GetPlayLength());
				
				if (delta <= gParam.nDuplicateTime) 
					bFindTime = TRUE;
				else
					bFindTime = FALSE;
			}


			// File size
			if (gParam.nTypeDuplicate & 16)
			{	
				delta = abs(mp3i1->GetMP3Size() - mp3i2->GetMP3Size());
				delta *= 100;	// x%
				
				if (delta <= gParam.nDuplicateFilesize) 
					bFindFilesize = TRUE;
				else
					bFindFilesize = FALSE;
			}

			bFind = bFindArtistTitle &&
					bFindFilename && 
					bFindGenre && 
					bFindTime && 
					bFindFilesize;
			
			
			if (bFind)
			{
				pDuplicates[i] = 1;				
				pDuplicates[j] = 1;
			}
			

		}

	}

	// ////////////////////////////////////
	// insert into listivew only duplicates
	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 

    lvi.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state     = 0;     
	lvi.iSubItem  = 0;         
	lvi.stateMask = 0; 
    lvi.pszText   = LPSTR_TEXTCALLBACK;

	m_nTotalLength = 0;
	iItem = 0;

	for (i=0; i<nCount; i++)
	{
		if (pDuplicates[i]==1)
		{
			mp3i1 = m_minfo.GetInfo(i);
		
			// Add result items in listview 
			lvi.iItem = iItem; 
			lvi.iImage = 4;		// Duplicate icon (3 ronds)
			lvi.lParam = (LPARAM)mp3i1;    // item data  
			ListView_InsertItem(m_hwndList, &lvi);  
			
			m_nTotalLength += mp3i1->GetPlayLength();
			iItem++;
		}
	}
	
	free(pDuplicates);

	delete pb;	// destroy progress bar
	ST_CreateStatus(1);

	LV_UpdateStatus();
	LV_Redraw(TRUE);

	ST_UpdateLengthSelected();
	SetCursor(hcurOld);
}


/* ************************************************************ */
void CDlgPlaylist::LV_DeadFiles(void)
{
	LVITEM lvi; 
	int i,nCount;
	DWORD attr;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;    // Retreive item data  MP3Info
		
		if (!mp3i) continue;

		attr = GetFileAttributes(mp3i->GetCFilename());
		
		ListView_SetItemState(m_hwndList, i, (attr == 0xFFFFFFFF) ? LVIS_SELECTED : 0 , LVIS_SELECTED);
	}
}


/* ************************************************************ */
void CDlgPlaylist::LV_Redraw(BOOL bUpdate)
{
	//turn the listview's redrawing on/off 
	
	SendMessage(m_hwndList, WM_SETREDRAW, bUpdate, 0); 
	
	if (bUpdate)
	{
		//turn the listview's redrawing back on and force it to draw 
		SendMessage(m_hwndList, WM_SETREDRAW, TRUE, 0); 
		InvalidateRect(m_hwndList, NULL, TRUE); 
		UpdateWindow(m_hwndList); 
	}
}

/* ********************************************************* */
/* Re/initialize the ListControl Columns */
void CDlgPlaylist::LV_ColumnResetDisplay(BOOL firstime)
{
    LV_COLUMN   lvc;
    int         i;
    int         nColumn;
	int			nColumnSort;
    int         widths[COLUMN_MAX];
    int         order[COLUMN_MAX];
    int         shown[COLUMN_MAX];
    
	// Modifie les style etendues
	ListView_SetExtendedListViewStyle(m_hwndList,LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
	LV_ColumnsGrid();


    GetColumnWidths(widths);
    GetColumnOrder(order);
    GetColumnShown(shown);


    if (!firstime)
    {
        nColumn = LV_ColumnGetNum();

        for (i = nColumn; i > 0; i--)
            ListView_DeleteColumn(m_hwndList, i - 1);
    }

   
   
	

    nColumn = 0;
    for (i = 0; i < COLUMN_MAX; i++)
    {
        if (shown[order[i]]==1)
        {
            lvc.mask      = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
			lvc.fmt       = gDefault_column[order[i]].nAlignment;
            lvc.pszText   = gDefault_column[order[i]].szName;
            lvc.iSubItem  = nColumn;
            lvc.cx        = widths[order[i]];
            ListView_InsertColumn(m_hwndList, nColumn, &lvc);

            m_realColumn[nColumn] = order[i];

            nColumn++;
        }
    }

    // Fill this in so we can still sort on columns NOT shown
    for (i = 0; i < COLUMN_MAX && nColumn < COLUMN_MAX; i++)
    {
        if (shown[order[i]]==0)
        {
            m_realColumn[nColumn] = order[i];
            nColumn++;
        }
    }

   
	reverse_sort = FALSE;

	if ((nColumnSort = GetSortColumn()) > 0)
	{
		m_last_sort = -1;
	}
	else
	{
		nColumnSort = abs(nColumnSort);
		m_last_sort = nColumnSort;
	}

	LV_ColumnSort(nColumnSort, TRUE);


	gParam.NbCol = LV_ColumnGetNum();
}


/* ********************************************************* */
void CDlgPlaylist::LV_ColumnSort(int column, BOOL bColumn, BOOL bReverse)
{
    int save_column = 0;
    int use_column = column;

    if (bColumn) // Was a column enum used??
    {
        int i;

		for (i = 0; i < COLUMN_MAX; i++)
		{
			if(m_realColumn[i] == column)
			{
				use_column = i;
				break;
			}
        }
    }

    /* Sort 'em */
    save_column = m_realColumn[use_column] ;

    if (bReverse == TRUE)
		reverse_sort = (column == m_last_sort && !reverse_sort) ? TRUE : FALSE;


    m_last_sort = use_column;
	ListView_SortItems(m_hwndList, ListViewCompareProc, this); //m_realColumn[use_column]);
    

    if (reverse_sort)
        save_column = -(save_column);

	SetSortColumn(save_column);

    Header_SetSortInfo(m_hwndList, use_column, !reverse_sort);

}


/* ********************************************************* */
int CDlgPlaylist::LV_ColumnGetNum()
{
    int  nColumn = 0;
    int  i;
    HWND hwndHeader;
    int  shown[COLUMN_MAX];

    GetColumnShown(shown);

    hwndHeader = GetDlgItem(m_hwndList, 0);

	nColumn = Header_GetItemCount(hwndHeader);
    
	if (nColumn < 1)
    {
        nColumn = 0;
        for (i = 0; i < COLUMN_MAX ; i++ )
        {
            if (shown[i])
                nColumn++;
        }
    }
    return nColumn;
}

/* ********************************************************* */
void CDlgPlaylist::LV_ColumnGetRealOrder(int order[])
{
	int tmpOrder[COLUMN_MAX];
	int nColumnMax;
	int i;
	
	nColumnMax = LV_ColumnGetNum();
	
	/* Get the Column Order and save it */
	ListView_GetColumnOrderArray(m_hwndList, nColumnMax, tmpOrder);
	
	for (i = 0; i < nColumnMax; i++)
	    order[i] = m_realColumn[tmpOrder[i]];

}



/* ********************************************************* */
void CDlgPlaylist::LV_ColumnSaveSetting()
{
	int widths[COLUMN_MAX];
	int order[COLUMN_MAX];
	int shown[COLUMN_MAX];
	int tmpOrder[COLUMN_MAX];
	int nColumnMax = 0;

	GetColumnOrder(order);
    GetColumnShown(shown);
    GetColumnWidths(widths);

    gParam.NbCol = nColumnMax = LV_ColumnGetNum();

    /* Get the Column Order and save it */
    ListView_GetColumnOrderArray(m_hwndList, nColumnMax, tmpOrder);

    for (int i = 0; i < nColumnMax; i++)
    {
        widths[m_realColumn[i]] = ListView_GetColumnWidth(m_hwndList, i);
        order[i] = m_realColumn[tmpOrder[i]];
    }
    
    SetColumnWidths(widths);
    SetColumnOrder(order);
}



/* ************************************************************ */
void CDlgPlaylist::LV_MatchTime(int nLengthMax)
{
	LV_SelectAll(FALSE);
	
	MatchTime(m_hwndList, nLengthMax);

	ST_UpdateLengthSelected();
}





/* ************************************************************************ */
void CDlgPlaylist::GetMP3Selected()
{

	// retreive information for the item selected
	CMP3Info *mp3iSel;
	LV_ITEM lvi;
	int i, nCount;
	int iSel = 0;
	
	m_bRefreshSelLength = FALSE;

	m_nSelCount = ListView_GetSelectedCount(m_hwndList);
	if (m_nSelCount < 1)
		return;


	// Vide l'ancienne selection en memoire et la recreer avec 
	// le nombre d'elements selectionne
	if (m_mp3iSelArray != NULL)
		free(m_mp3iSelArray);

	m_mp3iSelArray = (CMP3Info **)calloc(m_nSelCount, sizeof(CMP3Info *));


	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 
	 
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		if ((lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		mp3iSel = (CMP3Info *)lvi.lParam;

		m_mp3iSelArray[iSel] = mp3iSel;
		iSel++;
	}

}

/* ************************************************************ */
void CDlgPlaylist::SetMP3Selected(void)
{
	LVITEM lvi; 
	int i,nCount, iSel;
	
	LV_SelectAll(FALSE);
 
	m_bRefreshSelLength = FALSE;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_PARAM; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		int bFirst = TRUE;
		
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;    // Retreive item data  MP3Info
		
		if (!mp3i) 
			continue;

		for (iSel=0; iSel < m_nSelCount; iSel++)
		{
			if (mp3i == m_mp3iSelArray[iSel])
			{
				ListView_SetItemState(m_hwndList, i, LVIS_SELECTED , LVIS_SELECTED);
	
				if (bFirst)
				{	bFirst = FALSE;	
					ListView_SetItemState(m_hwndList, i, LVIS_FOCUSED , LVIS_FOCUSED);
					ListView_EnsureVisible(m_hwndList, i, TRUE);
				}
				break;
			}
		}
	}

	m_bRefreshSelLength = TRUE;

	SetFocus(m_hwndList);
	ST_UpdateLengthSelected();
}


/* ************************************************************************ */
void CDlgPlaylist::ST_UpdateLengthSelected()
{
	if (m_bRefreshSelLength == FALSE)		
		return;
	
	// retreive length Time for the item selected
	m_nSelLength   = 0;

	int nSelCount = ListView_GetSelectedCount(m_hwndList);
	int nCount = ListView_GetItemCount(m_hwndList);

	if (nSelCount == nCount)
		m_nSelLength = m_nTotalLength;
	else
	{
		if( nSelCount >= 1)
		{
			LVITEM lvi; 
			ZeroMemory(&lvi, sizeof(lvi)); 
			lvi.mask      = LVIF_STATE | LVIF_PARAM; 
			lvi.stateMask = LVIS_SELECTED; 
			
			for (int i = 0; i< nCount; i++)
			{
				lvi.iItem  = i;
				ListView_GetItem(m_hwndList, &lvi);
				
				CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

				if (!mp3i) continue;

				if ((lvi.state & LVIS_SELECTED)== LVIS_SELECTED)
				{
					m_nSelLength += mp3i->GetPlayLength();
				}
				
			}
		}
	}

	TCHAR szTimeFrmt[100];
	TCHAR szTemp[100];
	Time2Str(m_nSelLength, &szTimeFrmt[0], TRUE);
	_stprintf(szTemp, GS(LNG_STATUS_SELLENGTH), szTimeFrmt);
	ST_ChangeText((LPTSTR)szTemp, 4);

} 

/*************************************************************
* Update the status bar
**************************************************************/
void CDlgPlaylist::UpdateStatus()
{
	TCHAR szTemp[255];
	TCHAR szTimeFrmt[100];
	TCHAR *szID3v[] = { _T("ID3v1"), _T("ID3v2"), _T("ID3 ?") };
	
	int nTotalFile = ListView_GetItemCount(m_hwndList);

	_stprintf(szTemp, GS(LNG_STATUS_TOTALFILE), nTotalFile);
	ST_ChangeText((LPTSTR)szTemp, 0);

	_tcscpy(szTemp, szID3v[gParam.nShowID3v1]);
	ST_ChangeText((LPTSTR)szTemp, 1);


	if (nTotalFile != 0)
		m_nAvgLength = m_nTotalLength / nTotalFile;
	else
		m_nAvgLength = 0;

	Time2Str(m_nAvgLength, &szTimeFrmt[0], TRUE);
	_stprintf(szTemp, GS(LNG_STATUS_AVGLENGTH), szTimeFrmt);
	ST_ChangeText((LPTSTR)szTemp, 2);

	Time2Str(m_nTotalLength, &szTimeFrmt[0], FALSE);
	_stprintf(szTemp, GS(LNG_STATUS_TOTALLENGTH), szTimeFrmt);
	ST_ChangeText((LPTSTR)szTemp, 3);
}


/* ********************************************************** */
BOOL CDlgPlaylist::ST_CreateStatus(int nMode)
{
	int aWidth[5];
	int aNumElt;
	
	switch(nMode)
	{
		case 0:	// creation	
			if (m_hwndStat)
				::DestroyWindow(m_hwndStat);

			m_hwndStat = CreateStatusWindow( 
			  WS_CHILD | WS_BORDER | WS_VISIBLE,// window styles
			  _T(""),							// default window text
			  m_hDlg,							// parent window
			  IDC_STATUS);						// ID

			if(!m_hwndStat)
			   return FALSE;

		case 1:	// Define parts
			aWidth[0] = 110;
			aWidth[1] = 148;
			aWidth[2] = 245;
			aWidth[3] = 530;
			aWidth[4] = -1;

			aNumElt = 5;
			break;

		case 2:	// Define parts for scanning
			aWidth[0] = 110;
			aWidth[1] = -1;
			
			aNumElt = 2;
			break;

	}

	SendMessage(m_hwndStat, SB_SETPARTS , (LPARAM)aNumElt, (LPARAM)(LPINT)aWidth);
	return TRUE;
}

/* ********************************************************** */
BOOL CDlgPlaylist::ST_ChangeText(LPTSTR szText, int iPart )
{
    SendMessage(m_hwndStat, SB_SETTEXT, iPart, (LPARAM)szText);
	return TRUE;
}





// *********************************************************
// *********************************************************
/* Header code - Directional Arrows */
// *********************************************************
// *********************************************************
static LRESULT CALLBACK Header_WndProc( HWND hwnd,  UINT uMsg,  WPARAM wParam,  LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_ERASEBKGND:
        return Header_ListCtrlOnErase( hwnd, (HDC)wParam);

    case WM_DRAWITEM:
        {
            LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT)lParam;
            RECT rcLabel;
            int nSavedDC;
            HRGN hrgn;
            int nOffset;
            SIZE size;
            TCHAR tszText[256];
            HD_ITEM hdi;
            UINT uFormat;
            HDC hdc;

            hdc = lpdis->hDC;

            CopyRect(&rcLabel, &(lpdis->rcItem));

            /* save the DC */
            nSavedDC = SaveDC(hdc);

            /* set clip region to column */
            hrgn = CreateRectRgnIndirect(&rcLabel);
            SelectObject(hdc, hrgn);
            DeleteObject(hrgn);

            /* draw the background */
            FillRect(hdc, &rcLabel, GetSysColorBrush(COLOR_3DFACE));

            /* offset the label */
            GetTextExtentPoint32(hdc, _T(" "), 1, &size);
            nOffset = size.cx * 2;

            /* get the column text and format */
            hdi.mask = HDI_TEXT | HDI_FORMAT;
            hdi.pszText = tszText;
            hdi.cchTextMax = 255;
            Header_GetItem(GetDlgItem(hwnd, 0), lpdis->itemID, &hdi);

            /* determine format for drawing label */
            uFormat = DT_SINGLELINE | DT_NOPREFIX | DT_NOCLIP |
                DT_VCENTER | DT_END_ELLIPSIS;

            /* determine justification */
            if (hdi.fmt & HDF_CENTER)
                uFormat |= DT_CENTER;
            else if (hdi.fmt & HDF_RIGHT)
                uFormat |= DT_RIGHT;
            else
                uFormat |= DT_LEFT;

            /* adjust the rect if selected */
            if (lpdis->itemState & ODS_SELECTED)
            {
                rcLabel.left++;
                rcLabel.top += 2;
                rcLabel.right++;
            }

            /* adjust rect for sort arrow */
            if (lpdis->itemID == m_uHeaderSortCol)
            {
                rcLabel.right -= (3 * nOffset);
            }

            rcLabel.left += nOffset;
            rcLabel.right -= nOffset;

            /* draw label */
            if (rcLabel.left < rcLabel.right)
                DrawText(hdc, tszText, -1, &rcLabel, uFormat);

            /* draw the arrow */
            if (lpdis->itemID == m_uHeaderSortCol)
            {
                RECT rcIcon;
                HPEN hpenLight, hpenShadow, hpenOld;

                CopyRect(&rcIcon, &(lpdis->rcItem));

                hpenLight = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DHILIGHT));
                hpenShadow = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DSHADOW));
                hpenOld = (HPEN)SelectObject(hdc, hpenLight);

                if (m_fHeaderSortAsc)
                {
                    /* draw triangle pointing up */
                    MoveToEx(hdc, rcIcon.right - 2 * nOffset, nOffset - 1, NULL);
                    LineTo(hdc, rcIcon.right - 3 * nOffset / 2,
                        rcIcon.bottom - nOffset);
                    LineTo(hdc, rcIcon.right - 5 * nOffset / 2 - 2,
                        rcIcon.bottom - nOffset);
                    MoveToEx(hdc, rcIcon.right - 5 * nOffset / 2 - 1,
                        rcIcon.bottom - nOffset, NULL);

                    SelectObject(hdc, hpenShadow);
                    LineTo(hdc, rcIcon.right - 2 * nOffset, nOffset - 2);
                }
                else
                {
                    /* draw triangle pointing down */
                    MoveToEx(hdc, rcIcon.right - 3 * nOffset / 2, nOffset - 1,
                        NULL);
                    LineTo(hdc, rcIcon.right - 2 * nOffset - 1,
                        rcIcon.bottom - nOffset);
                    LineTo(hdc, rcIcon.right - 2 * nOffset - 1,
                        rcIcon.bottom - nOffset);
                    MoveToEx(hdc, rcIcon.right - 2 * nOffset - 1,
                        rcIcon.bottom - nOffset, NULL);

                    SelectObject(hdc, hpenShadow);
                    LineTo(hdc, rcIcon.right - 5 * nOffset / 2 - 1,
                        nOffset - 1);
                    LineTo(hdc, rcIcon.right - 3 * nOffset / 2,
                        nOffset - 1);
                }
                SelectObject(hdc, hpenOld);
                DeleteObject (hpenLight);
                DeleteObject (hpenShadow);
            }
            RestoreDC(hdc, nSavedDC);
            return 1;
        }

    case WM_NOTIFY:
        {
            HD_NOTIFY   *pHDN = (HD_NOTIFY*)lParam;

            // This code is for using bitmap in the background
            // Invalidate the right side of the control when a column is resized
            if (pHDN->hdr.code == HDN_ITEMCHANGINGW ||
                pHDN->hdr.code == HDN_ITEMCHANGINGA)
            {
               
			/*	
	
			*/
            }
        }
        break;
    }

    /* message not handled */
	return CallWindowProc(g_lpHeaderWndProc, hwnd, uMsg, wParam, lParam);

}

// *********************************************************
/* Subclass the Listview Header */
static void Header_Initialize(HWND hwndList)
{
    /* this will subclass the listview (where WM_DRAWITEM gets sent for
       the header control) */

#ifdef STRICT 
	g_lpHeaderWndProc = (WNDPROC)GetWindowLong(hwndList, GWL_WNDPROC);
#else 
	g_lpHeaderWndProc = (FARPROC)GetWindowLong(hwndList, GWL_WNDPROC);
#endif 

    SetWindowLong(hwndList, GWL_WNDPROC, (LONG)Header_WndProc);
}


// *********************************************************
/* Set Sort information needed by Header */
static void Header_SetSortInfo(HWND hwndList, int nCol, BOOL bAsc)
{
    HWND hwndHeader;
    HD_ITEM hdi;

    m_uHeaderSortCol = nCol;
    m_fHeaderSortAsc = bAsc;

    hwndHeader = GetDlgItem(hwndList, 0);

    /* change this item to owner draw */
    hdi.mask = HDI_FORMAT;
    Header_GetItem(hwndHeader, nCol, &hdi);
    hdi.fmt |= HDF_OWNERDRAW;
    Header_SetItem(hwndHeader, nCol, &hdi);

    /* repaint the header */
    InvalidateRect(hwndHeader, NULL, TRUE);
}

// *********************************************************
static BOOL Header_ListCtrlOnErase(HWND hWnd, HDC hDC)
{
	RECT rcClient;
	
    GetClientRect(hWnd, &rcClient);

	/* Use standard control face color */
	{
		HBRUSH hBrush = CreateSolidBrush(GetSysColor(COLOR_WINDOW));
        HBRUSH hOldBrush;

		hOldBrush = (HBRUSH)SelectObject(hDC, hBrush);
		FillRect(hDC, &rcClient, hBrush);
        SelectObject(hDC, hOldBrush);
		DeleteObject(hBrush);
	}

    return TRUE;
}





/* End of source file */


/* ************************************************************************ */
BOOL CDlgPlaylist::TV_Create()
{
	// ///////////////////	
	// Initialize Treeview
	m_hIml = ImageList_LoadBitmap(plugin.hDllInstance, MAKEINTRESOURCE(IDB_FILTERTYPE), BMWIDTH, 0, RGBREPLACE);

	if (m_hIml != NULL)
		TreeView_SetImageList(m_hwndTree, m_hIml, 0);

	TV_Load();

	ShowWindow(m_hwndTree, SW_SHOW);
	ShowWindow(GetDlgItem(m_hDlg, IDC_SPLITTER), SW_SHOW);
	m_bShowTree = TRUE;

	ResizeControls();



	return TRUE;
}

/* ************************************************************************ */
void CDlgPlaylist::TV_Destroy()
{
	TreeView_DeleteAllItems(m_hwndTree);
	
	if(m_hIml)
	{
		ImageList_Destroy(m_hIml);
		m_hIml = NULL;
	}

	ShowWindow(m_hwndTree, SW_HIDE);
	ShowWindow(GetDlgItem(m_hDlg, IDC_SPLITTER), SW_HIDE);
	m_bShowTree = FALSE;
	
	ResizeControls();
}

/* ************************************************************************ */
void CDlgPlaylist::TV_Update()
{
	if (m_bShowTree) 
	{
		TV_Redraw(FALSE);
		TV_Load();
		TV_Redraw(TRUE);
	}
}



/* ************************************************************ */
void CDlgPlaylist::TV_Redraw(BOOL bUpdate)
{
	//turn the listview's redrawing on/off 
	
	SendMessage(m_hwndTree, WM_SETREDRAW, bUpdate, 0); 
	
	if (bUpdate)
	{
		//turn the listview's redrawing back on and force it to draw 
		SendMessage(m_hwndTree, WM_SETREDRAW, TRUE, 0); 
		InvalidateRect(m_hwndTree, NULL, TRUE); 
		UpdateWindow(m_hwndTree); 
	}
}


/* ************************************************************** */
void CDlgPlaylist::TV_Load()
{
	TV_INSERTSTRUCT	tvis;
	HTREEITEM		hItem,  hRootItem;
	HTREEITEM hItemSelected = NULL;
	int ModeCount[4] = {0,0,0,0};
	int ID3Count[4] = {0,0,0,0};	
	TCHAR szTemp[MAX_PATH];
	tstring str;
	CMP3Info *mp3i;
	HCURSOR hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	HWND m_hwndTree = GetDlgItem(m_hDlg, IDC_TREEVIEW_EXPLORER);
	int i;

/*	int k=0;
	CProgressBar *pb;
	pb = new CProgressBar(m_hDlg, m_hwndStat);
	pb->Init(plugin.hDllInstance, 5);
	pb->Show();
*/
	TreeView_DeleteAllItems(m_hwndTree);
	
	struct tag_Root
	{	int nIDLng;
		int nIndex;
		int nImage;
	} rootItems[] =
		{
			{LNG_ARTIST,  ROOT_ARTIST, 2} ,
			{LNG_GENRE,   ROOT_GENRE,  3} ,
			{LNG_ID3TAG,  ROOT_ID3TAG, 6} ,
			{LNG_COMMENT, ROOT_COMMENT,4} ,
			{LNG_MODE,    ROOT_MODE,   10} ,
			{LNG_ALBUM,   ROOT_ALBUM,  5} 
		};
		
	// Compte le nombre de Mode et ID3Tag
	// Scan all playlist  from memory 
	for (i=0; i < m_minfo.GetCount(); i++)
	{
		mp3i = m_minfo.GetInfo(i);
		GetFrameInfo(mp3i, ID3TAG_COMM, str);

		if (str.empty() == FALSE)
			AddVector(str);
			
		ModeCount[mp3i->GetMode()]++;
		ID3Count[mp3i->GetTagVersion()]++;
	}



// //////////
// ROOTS
// //////////
	ZeroMemory(&tvis, sizeof(tvis)); 
	tvis.item.mask           = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
	tvis.hParent             = NULL;
	tvis.hInsertAfter        = NULL;
	tvis.item.lParam         = NULL;

	_tcscpy(szTemp, _T("All audio"));
	tvis.item.pszText  = szTemp;
	tvis.item.iImage         = 0;	// Image list "Multi CD"
	tvis.item.iSelectedImage = 0;
	hRootItem = TreeView_InsertItem(m_hwndTree, &tvis);

	ZeroMemory(&m_hRoot, sizeof(m_hRoot)); 

	for (i=0; i<NUMELEMENTS(rootItems); i++)
	{
		if ( gParam.nOptionExplorer & (1<<i) )
		{
			_tcscpy(szTemp, GS(rootItems[i].nIDLng));
			tvis.hParent             = hRootItem;
			tvis.item.pszText        = szTemp;
			tvis.item.iImage         = rootItems[i].nImage;
			tvis.item.iSelectedImage = rootItems[i].nImage;
			m_hRoot[rootItems[i].nIndex] = TreeView_InsertItem(m_hwndTree, &tvis);
		}
	}
	

// ///////////////////////////
// ARTIST
// ///////////////////////////
	if ( gParam.nOptionExplorer & (1<<ROOT_ARTIST) )
	{
		m_StrArray.clear();

		// Scan all playlist  from memory 
		for (i=0; i < m_minfo.GetCount(); i++)
		{
			mp3i = m_minfo.GetInfo(i);
			GetFrameInfo(mp3i, ID3TAG_TPE1, str);

			if (str.empty() == FALSE)
				AddVector(str);
		}


		ZeroMemory(&tvis, sizeof(tvis)); 
		tvis.item.mask           = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		tvis.hParent             = m_hRoot[ROOT_ARTIST];
		tvis.item.iImage         = 2;	// Image list "Artist"
		tvis.item.iSelectedImage = 2;

		for (i=0; i < m_StrArray.size(); i++)
		{
			tvis.item.pszText        = (LPTSTR)m_StrArray[i].c_str();
			hItem = TreeView_InsertItem(m_hwndTree, &tvis);
			
			if (m_StrArray[i].compare(m_filterArtist)==0)			
				hItemSelected = hItem;
		}


	//	pb->StepIt(k++);
	}	


// ///////////////////////////
// GENRE
// ///////////////////////////
	if ( gParam.nOptionExplorer & (1<<ROOT_GENRE) )
	{
		ZeroMemory(&tvis, sizeof(tvis)); 
		tvis.item.mask           = TVIF_TEXT | TVIF_PARAM | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		tvis.hParent             = m_hRoot[ROOT_GENRE];
		tvis.hInsertAfter        = TVI_SORT; 
		tvis.item.iImage         = 3;
		tvis.item.iSelectedImage = 3;
		
		for (i=0; i<ID3TAG_MAX_GENRE+2; i++)
		{
			if (gStatGenre[i].nCount > 0)
			{
				tvis.item.pszText = (LPTSTR)GenreInt2Str((unsigned char)i);	// title Genre
				tvis.item.lParam  = i;	 // id Genre
				hItem = TreeView_InsertItem(m_hwndTree, &tvis);	
			}
		}

		//	pb->StepIt(k++);
	}

// ///////////////////////////
// COMMENT
// ///////////////////////////
	if ( gParam.nOptionExplorer & (1<<ROOT_COMMENT) )
	{
		m_StrArray.clear();

		// Scan all playlist  from memory 
		for (i=0; i < m_minfo.GetCount(); i++)
		{
			mp3i = m_minfo.GetInfo(i);
			GetFrameInfo(mp3i, ID3TAG_COMM, str);

			if (str.empty() == FALSE)
				AddVector(str);
		}

		ZeroMemory(&tvis, sizeof(tvis)); 
		tvis.item.mask           = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		tvis.hParent             = m_hRoot[ROOT_COMMENT];
		tvis.item.iImage         = 4;	// Image list "PostIt"
		tvis.item.iSelectedImage = 4;

		for (i=0; i < m_StrArray.size(); i++)
		{
			tvis.item.pszText        = (LPTSTR)m_StrArray[i].c_str();
			hItem = TreeView_InsertItem(m_hwndTree, &tvis);
			
			if (m_StrArray[i].compare(m_filterComment)==0)			
				hItemSelected = hItem;
		}

		//	pb->StepIt(k++);
	}
	

// ///////////////////////////	
// ID3Tag
// ///////////////////////////
	if ( gParam.nOptionExplorer & (1<<ROOT_ID3TAG) )
	{
		TCHAR *szID3v[] = {  _T("No Tag"), _T("ID3v1"), _T("ID3v2"), _T("ID3v1+v2") };

		ZeroMemory(&tvis, sizeof(tvis)); 
		tvis.item.mask           = TVIF_TEXT | TVIF_PARAM | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		tvis.hParent             = m_hRoot[ROOT_ID3TAG];
		
		for (i = 0; i<NUMELEMENTS(ID3Count); i++)
		{
			if (ID3Count[i] >0)
			{	
				tvis.item.pszText = szID3v[i];	// ID3Tag version
				tvis.item.lParam  = i;			// id ID3Tag
				tvis.item.iImage         = 6+i;
				tvis.item.iSelectedImage = 6+i;
				hItem = TreeView_InsertItem(m_hwndTree, &tvis);
			}
		}

	//	pb->StepIt(k++);
	}



// ///////////////////////////
// MODE
// ///////////////////////////
	if ( gParam.nOptionExplorer & (1<<ROOT_MODE) )
	{
		ZeroMemory(&tvis, sizeof(tvis)); 
		tvis.item.mask           = TVIF_TEXT | TVIF_PARAM | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		tvis.hParent             = m_hRoot[ROOT_MODE];
		
		for (i = 0; i<NUMELEMENTS(ModeCount); i++)
		{
			if (ModeCount[i] >0)
			{
				tvis.item.pszText = ModeName[i];	// Title Mode
				tvis.item.lParam  = i;				// id Mode
				tvis.item.iImage         = 10+i;
				tvis.item.iSelectedImage = 10+i;
				hItem = TreeView_InsertItem(m_hwndTree, &tvis);
			}
		}

		//	pb->StepIt(k++);
	}


// ///////////////////////////
// ALBUM
// ///////////////////////////
	if ( gParam.nOptionExplorer & (1<<ROOT_ALBUM) )
	{
		m_StrArray.clear();

		// Scan all playlist  from memory 
		for (i=0; i < m_minfo.GetCount(); i++)
		{
			mp3i = m_minfo.GetInfo(i);
			GetFrameInfo(mp3i, ID3TAG_TALB, str);

			if (str.empty() == FALSE)
				AddVector(str);
		}

		ZeroMemory(&tvis, sizeof(tvis)); 
		tvis.item.mask           = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		tvis.hParent             = m_hRoot[ROOT_ALBUM];
		tvis.item.iImage         = 5;
		tvis.item.iSelectedImage = 5;

		for (i=0; i < m_StrArray.size(); i++)
		{
			tvis.item.pszText        = (LPTSTR)m_StrArray[i].c_str();
			hItem = TreeView_InsertItem(m_hwndTree, &tvis);
			
			if (m_StrArray[i].compare(m_filterAlbum)==0)			
				hItemSelected = hItem;
		}
	}


// ////////////////////////////////
// Positionne sur l'item selectione
// ////////////////////////////////
	if (hItemSelected == NULL)
	{	// No found filter --> select all xxx
		hItemSelected = hRootItem;
		TreeView_Expand(m_hwndTree, hRootItem, TVE_EXPAND);
		TreeView_SelectItem(m_hwndTree, hItemSelected);
		TreeView_EnsureVisible(m_hwndTree, hItemSelected);
	}
	else
	{
		hRootItem = TreeView_GetParent(m_hwndTree, hItemSelected);
		TreeView_Expand(m_hwndTree, hRootItem, TVE_EXPAND);
		TreeView_SelectItem(m_hwndTree, hItemSelected);
		TreeView_EnsureVisible(m_hwndTree, hItemSelected);
	}	


//	delete pb;	// destroy progress bar
//	ST_CreateStatus(0);

	SetCursor(hcurOld);
}

/* ************************************************************** */
void CDlgPlaylist::TV_SetCurrentItem()
{
	HTREEITEM hItem       = TreeView_GetSelection(m_hwndTree);
    HTREEITEM hItemRoot   = TreeView_GetRoot(m_hwndTree);
	HTREEITEM hItemParent = TreeView_GetParent(m_hwndTree, hItem);
	TCHAR szText[32768];
	

	// RAZ all filters
	szText[0] = '\0';
	m_nFilterBy = NO_FILTER;
	m_filterArtist.assign(szText);
	m_filterComment.assign(szText);
	m_filterAlbum.assign(szText);
	m_filterMode = -1;
	m_filterID3Tag = -1;
		
	for (int j=0 ; j<ID3TAG_MAX_GENRE+2; j++)
		gStatGenre[j].bCheck = FALSE;




	//pas de filtre all xxx
	if (!hItem)		return;
	if (hItem == hItemRoot)		return;
	
	for (int i=0; i< NUMELEMENTS(m_hRoot); i++)
	{
		if (hItem ==  m_hRoot[i])	
			return;
	}
	

	TV_ITEM	tvis;
	ZeroMemory(&tvis, sizeof(tvis));
	tvis.mask       = TVIF_TEXT | TVIF_PARAM;
	tvis.hItem      = hItem;
	tvis.pszText    = szText;
	tvis.cchTextMax = sizeof(szText);
	if (TreeView_GetItem(m_hwndTree, &tvis) == FALSE)
		return;


	if (hItemParent == m_hRoot[ROOT_ARTIST])
	{
		m_nFilterBy = FILTER_ARTIST;
		m_filterArtist.assign(szText);		
	}
	else if (hItemParent == m_hRoot[ROOT_COMMENT])
	{
		m_nFilterBy = FILTER_COMMENT;
		m_filterComment.assign(szText);		
	}
	else if (hItemParent == m_hRoot[ROOT_GENRE])
	{
		int nIdGenre = tvis.lParam;
		m_nFilterBy = FILTER_GENRE;
		gStatGenre[nIdGenre].bCheck = TRUE;
	}
	else if (hItemParent == m_hRoot[ROOT_ID3TAG])
	{
		m_nFilterBy = FILTER_ID3TAG;
		m_filterID3Tag = tvis.lParam;
	}
	else if (hItemParent == m_hRoot[ROOT_MODE])
	{
		m_nFilterBy = FILTER_MODE;
		m_filterMode = tvis.lParam;
	}
	else if (hItemParent == m_hRoot[ROOT_ALBUM])
	{
		m_nFilterBy = FILTER_ALBUM;
		m_filterAlbum.assign(szText);		
	}
	    
	
	return;
}




/* ************************************************************** */
int CDlgPlaylist::AddVector(tstring& str)
{
	// add into the array. since this is sorted, find the right place for the insertion
	int pos = FindVector(str);
	
	switch(pos)
	{
		case -2:	// existe deja pas d'ajout
			return pos;
		
		case -1:	// ajoute a la fin
			m_StrArray.push_back(str);
			pos = m_StrArray.size()-1;
			break;
		default:	// ajoute juste apres
			m_StrArray.insert(&m_StrArray.at(pos), str);
			break;
	}

	return pos;
}

/* ************************************************************** */
/**
 * find the string which is greater than the current Array
 * this is used, to maintain a sorted list
 * returns -1 if no bigger info is found
 */
int CDlgPlaylist::FindVector(tstring& str)
{
	for (int i=0; i<m_StrArray.size(); i++)
	{
		int ret = m_StrArray[i].compare(str);
		
		if (ret > 0)
			return(i);			
		else if (ret == 0)
			return -2;			// Existe deja
	}
	return -1;
}



/* *********************************************************************** */
void CDlgPlaylist::Splitter_Init(void)
{
    /* load the cursor for the splitter */
    m_hSplitterCursor = LoadCursor(plugin.hDllInstance, MAKEINTRESOURCE(IDC_CURSOR_HSPLIT));
}

/* *********************************************************************** */
/* Called with hWnd = Parent Window */
void CDlgPlaylist::Splitter_Recalc()
{
    POINT   p = {0,0};
    RECT    leftRect, rightRect;
    int     dragWidth;
	HWND hWnd = m_hDlg;
	HWND hwndSplitter = GetDlgItem(m_hDlg, IDC_SPLITTER); 

    ClientToScreen(hWnd, &p);

    GetWindowRect(hwndSplitter,  &m_SplitDragRect);
    GetWindowRect(m_hwndTree,  &leftRect);
    GetWindowRect(m_hwndList,  &rightRect);

    OffsetRect(&m_SplitDragRect, -p.x, -p.y);
    OffsetRect(&leftRect, -p.x, -p.y);
    OffsetRect(&rightRect, -p.x, -p.y);

    dragWidth = m_SplitDragRect.right - m_SplitDragRect.left;

    m_SplitLimitRect.left    = leftRect.left + 4;
    m_SplitLimitRect.right   = rightRect.right - 100;
    m_SplitLimitRect.top     = m_SplitDragRect.top;
    m_SplitLimitRect.bottom  = m_SplitDragRect.bottom;
    
    if (m_SplitDragRect.left < m_SplitLimitRect.left)
        m_SplitDragRect.left = m_SplitLimitRect.left;

    if (m_SplitDragRect.right > m_SplitLimitRect.right)
        m_SplitDragRect.left = m_SplitLimitRect.right - dragWidth;

    m_SplitDragRect.right = m_SplitDragRect.left + dragWidth;

}


/* *********************************************************************** */
void CDlgPlaylist::Splitter_OnInvertTracker(HWND hWnd /* HWND PARENT*/, const RECT *rect)
{
    HDC     hDC = GetDC(hWnd);
    HBRUSH  hBrush = CreateSolidBrush(RGB(0xFF, 0xFF, 0xFF));
    HBRUSH  hOldBrush = 0;

    if (hBrush != 0)
        hOldBrush = (HBRUSH)SelectObject(hDC, hBrush);
    
    PatBlt(hDC, rect->left, rect->top, rect->right - rect->left, rect->bottom - rect->top, DSTINVERT);
    
    if (hOldBrush != 0)
        SelectObject(hDC, hOldBrush);
    ReleaseDC(hWnd, hDC);
    DeleteObject(hBrush);
}
/* *********************************************************************** */
void CDlgPlaylist::Splitter_StartTracking(HWND hWnd /* HWND PARENT*/, UINT hitArea)
{
    if (!m_bTracking && hitArea == SPLITTER_HITITEM)
    {
        // Ensure we have and updated cursor structure
        Splitter_Recalc();
        // Draw the first splitter shadow
        Splitter_OnInvertTracker(hWnd, &m_SplitDragRect);
        // Capture the mouse
        SetCapture(hWnd);
        // Set tracking to TRUE
        m_bTracking = TRUE;
        SetCursor(m_hSplitterCursor);
    }
}
/* *********************************************************************** */
void CDlgPlaylist::Splitter_StopTracking(HWND hWnd /* HWND PARENT*/)
{
    if (m_bTracking)
    {
        // erase the tracking image
        Splitter_OnInvertTracker(hWnd, &m_SplitDragRect);
        // Release the mouse
        ReleaseCapture();
        // set tracking to false
        m_bTracking = FALSE;
        SetCursor(LoadCursor(0, IDC_ARROW));
        // set the new splitter position
        m_nSplitterOffset = m_SplitDragRect.left;
        
        // Redraw the screen area
        ResizeControls();
    }
}
/* *********************************************************************** */
UINT CDlgPlaylist::Splitter_HitTest(HWND hWnd /* HWND PARENT*/, POINTS p)
{
    RECT  rect;
    POINT pt;
 
    pt.x = p.x;
    pt.y = p.y;

    // Check which area we hit
    ClientToScreen(hWnd, &pt);
    GetWindowRect(GetDlgItem(m_hDlg, IDC_SPLITTER), &rect);
    if (PtInRect(&rect, pt))
    {
        // We hit the splitter
        return SPLITTER_HITITEM;
    }
  
    // We missed the splitter
    return SPLITTER_HITNOTHING;
}
/* *********************************************************************** */
void CDlgPlaylist::Splitter_OnMouseMove(HWND hWnd /* HWND PARENT*/, UINT nFlags, POINTS p)
{
    if (m_bTracking)  // move the tracking image
    {
        int     nWidth;
        RECT    rect;
        POINT   pt;

        pt.x = (int)p.x;
        pt.y = (int)p.y;

        ClientToScreen(hWnd, &pt);
        GetWindowRect(hWnd, &rect);
        if (! PtInRect(&rect, pt))
        {
            if ((short)pt.x < (short)rect.left)
                pt.x = rect.left;
            if ((short)pt.x > (short)rect.right)
                pt.x = rect.right;
            pt.y = rect.top + 1;
        }

        ScreenToClient(hWnd, &pt);

        // Erase the old tracking image
        Splitter_OnInvertTracker(hWnd, &m_SplitDragRect);
        
        // calc the new one based on p.x draw it
        nWidth = m_SplitDragRect.right - m_SplitDragRect.left;
        m_SplitDragRect.right = pt.x + nWidth / 2;
        m_SplitDragRect.left  = pt.x - nWidth / 2;

        if (pt.x - nWidth / 2 > m_SplitLimitRect.right)
        {
            m_SplitDragRect.right = m_SplitLimitRect.right;
            m_SplitDragRect.left  = m_SplitDragRect.right - nWidth;
        }
        if (pt.x + nWidth / 2 < m_SplitLimitRect.left)
        {
            m_SplitDragRect.left  = m_SplitLimitRect.left;
            m_SplitDragRect.right = m_SplitDragRect.left + nWidth;
        }
        Splitter_OnInvertTracker(hWnd, &m_SplitDragRect);
    }
    else
    {
        switch(Splitter_HitTest(hWnd, p))
        {
        case SPLITTER_HITNOTHING:
            SetCursor(LoadCursor(0, IDC_ARROW));
            break;
        case SPLITTER_HITITEM:
            SetCursor(m_hSplitterCursor);
            break;
        }
    }
}
/* *********************************************************************** */
void CDlgPlaylist::Splitter_OnLButtonDown(HWND hWnd /* HWND PARENT*/, UINT nFlags, POINTS p)
{
    if (!m_bTracking) // See if we need to start a splitter drag
    {
        Splitter_StartTracking(hWnd, Splitter_HitTest(hWnd, p));
    }
}
/* *********************************************************************** */
void CDlgPlaylist::Splitter_OnLButtonUp(HWND hWnd /* HWND PARENT*/, UINT nFlags, POINTS p)
{
    if (m_bTracking)
    {
        Splitter_StopTracking(hWnd);
    }
}




// *********************************************************
BOOL CDlgPlaylist::LaunchThreadFilter()
{
	TCHAR szSearch[40];
	
	
	DWORD dwThreadID=0;

	// Termine le thread existant si il est actif
	if (m_hThread != NULL)
	{

		if (m_bThreadActive == TRUE)
		{
			SetEvent(m_hQuitEvent);
			if (WaitForSingleObject(m_hThread, 10) == WAIT_TIMEOUT)         
				TerminateThread(m_hThread, 0);              		
			ResetEvent(m_hQuitEvent);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
		m_bThreadActive = FALSE;			

	}

	GetDlgItemText(m_hDlg, IDC_EDIT_SEARCH, (LPTSTR)szSearch, 40);

	if (_tcslen(szSearch) == 0)
	{
		LV_UpdatePlaylist(); // Pas besion de thread 	
	}	
	else
	{
		m_hThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)ThreadFilterProc, this, 0, &dwThreadID);

		if (m_hThread != NULL)
			m_bThreadActive = TRUE;	
	}


	return(m_hThread != NULL);
}





// *********************************************************
static DWORD WINAPI ThreadFilterProc(CDlgPlaylist *pdlg)
{
	CDlgPlaylist *p= (CDlgPlaylist *)pdlg;
	p->LV_UpdateListWithFilter();
	return TRUE;
}



/* ************************************************************ */

void CDlgPlaylist::LV_UpdateListWithFilter()
{
	LVITEM lvi; 
	int i, iItem;
	BOOL bFind;
    DWORD dwWaitReturn;
	TCHAR szSearch[40];

	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 
		
    // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

	m_nTotalLength = 0;

	int nCount = m_minfo.GetCount();

	CProgressBar *pb;
	pb = new CProgressBar(m_hDlg, m_hwndStat);
	pb->Init(plugin.hDllInstance, nCount);
	pb->Show();

	GetDlgItemText(m_hDlg, IDC_EDIT_SEARCH, (LPTSTR)szSearch, 40);	

	// Match case checked
	if (!(gParam.nOptionSearch & 32))
		_tcslwr(szSearch);

	
	iItem = 0;

	for (i=0; i < nCount; i++)
	{
		pb->StepIt(i);
		
		tstring artist =_T("");
		tstring title =_T(""); 
		tstring album =_T(""); 
		tstring comment =_T(""); 
		tstring filename =_T("");
		tstring genre= _T("");

		dwWaitReturn = WaitForSingleObject(m_hQuitEvent, 0);
		
		if (dwWaitReturn == WAIT_OBJECT_0)
		{
			goto stopFilter;// stop Thread
		}
		else
		{
			CMP3Info *mp3i = m_minfo.GetInfo(i);

			filename = mp3i->GetCFilename();
			GetFrameInfo(mp3i, ID3TAG_TPE1, artist);
			GetFrameInfo(mp3i, ID3TAG_TIT2, title);
			GetFrameInfo(mp3i, ID3TAG_TALB, album);
			GetFrameInfo(mp3i, ID3TAG_COMM, comment);
			GetFrameInfo(mp3i, ID3TAG_TCON, genre);
	
			bFind = FALSE;

			
			tstring artistlwr(artist.c_str());
			tstring titlelwr(title.c_str()); 
			tstring albumlwr(album.c_str()); 
			tstring commentlwr(comment.c_str()); 
			tstring filenamelwr(filename.c_str());
			

			// compare item in lowercase  
			if (!(gParam.nOptionSearch & 32))
			{
				_tcslwr((TCHAR*)artistlwr.c_str());
				_tcslwr((TCHAR*)titlelwr.c_str());
				_tcslwr((TCHAR*)albumlwr.c_str());
				_tcslwr((TCHAR*)commentlwr.c_str());
				_tcslwr((TCHAR*)filenamelwr.c_str());
			}

			if( (gParam.nOptionSearch & 1)  && _tcsstr(artistlwr.c_str(), szSearch)  != 0 ||
				(gParam.nOptionSearch & 2)  && _tcsstr(titlelwr.c_str(), szSearch)   != 0 ||
				(gParam.nOptionSearch & 4)  && _tcsstr(albumlwr.c_str(), szSearch) != 0 ||
				(gParam.nOptionSearch & 8)  && _tcsstr(commentlwr.c_str(), szSearch) != 0 ||
				(gParam.nOptionSearch & 16) && _tcsstr(filenamelwr.c_str(), szSearch) != 0 )
					bFind = TRUE;
			
			if (LV_Filter(mp3i) && bFind)
			{
				int nImage = mp3i->GetTagVersion();

				// Initialize item-specific LVITEM members.         
				// Add the item.         
				lvi.iItem = iItem; 
				lvi.iSubItem = 0;         
				lvi.iImage = nImage; 
				lvi.lParam = (LPARAM)mp3i;    // item data  
				ListView_InsertItem(m_hwndList, &lvi);  
				
				m_nTotalLength += mp3i->GetPlayLength();
				iItem++;
			}
		}
	}



stopFilter:

	delete pb;
	ST_CreateStatus(1);
	
	LV_UpdateStatus();
	LV_Redraw(TRUE);
	
	ExitThread(1);
	
	m_bThreadActive = FALSE;	
	return;

}



// ********************************************************************
void CDlgPlaylist::SetTextMenu()
{
	HMENU menu;
	int i;
	
	// MenuBar
	MenuSetText(GetMenu(m_hDlg), 0, GS(LNG_MENU_FILE));
	MenuSetText(GetMenu(m_hDlg), 1, GS(LNG_MENU_OPTIONS));
	MenuSetText(GetMenu(m_hDlg), 2, GS(LNG_MENU_ID3TAG));

	// File Menu
	menu = MenuGetPopup(GetMenu(m_hDlg), 0);
	MenuSetText( menu, 0, GS(LNG_MENU_NEW));
	MenuSetText( menu, 1, GS(LNG_MENU_OPEN));
	MenuSetText( menu, 3, GS(LNG_MENU_RELOAD_WINAMP));
	MenuSetText( menu, 5, GS(LNG_MENU_SAVE));
	MenuSetText( menu, 6, GS(LNG_MENU_SAVE_AS));
	MenuSetText( menu, 8, GS(LNG_MENU_ADD));
		// Sub File/Add menu
		menu = MenuGetPopup(menu, 8);
		MenuSetText( menu, 0, GS(LNG_MENU_ADD_FILE));
		MenuSetText( menu, 1, GS(LNG_MENU_ADD_DIRECTORY));
	menu = MenuGetPopup(GetMenu(m_hDlg), 0);
	MenuSetText( menu, 9, GS(LNG_MENU_INSERT_EZ3));
	MenuSetText( menu,11, GS(LNG_MENU_IMPORT));
	MenuSetText( menu,12, GS(LNG_MENU_EXPORT));
	MenuSetText( menu,14, GS(LNG_MENU_PROPERTY_PLAYLIST));
	MenuSetText( menu,15, GS(LNG_MENU_PROPERTY_VOLUME));
	MenuSetText( menu,17, GS(LNG_MENU_QUIT));

	// Option Menu
	menu = MenuGetPopup(GetMenu(m_hDlg), 1);
	MenuSetText( menu, 0, GS(LNG_MENU_OPT_COLUMN));
	MenuSetText( menu, 2, GS(LNG_MENU_OPT_OPTION));           
	MenuSetText( menu, 3, GS(LNG_MENU_OPT_CONFIGURE));        
	MenuSetText( menu, 4, GS(LNG_MENU_OPT_HTMLCSS));          
	MenuSetText( menu, 6, GS(LNG_MENU_OPT_RESET_DEFAULT));    
	MenuSetText( menu, 8, GS(LNG_MENU_OPT_FILTER));           
	MenuSetText( menu, 9, GS(LNG_MENU_OPT_MEDIA_WIZARD));     
	MenuSetText( menu,11, GS(LNG_MENU_OPT_VIEW));             
	MenuSetText( menu,13, GS(LNG_MENU_OPT_EXPLORER));         
	MenuSetText( menu,14, GS(LNG_MENU_OPT_EXPLORER_OPTION)); 
	MenuSetText( menu,16, GS(LNG_MENU_OPT_LANGUAGE));  

	//Languages menus
	// Get languages popup
	menu = MenuGetPopup(menu, 16);
	while(RemoveMenu(menu, 0, MF_BYPOSITION ));

    int n_lang = lng->GetNumberLang();

	for(i=0; i<n_lang; i++)
	{
		MenuAddItem(menu, i, i+10000, lng->GetLanguage(i) );
	}


	// ID3Tag Menu
	menu = MenuGetPopup(GetMenu(m_hDlg), 2);
	MenuSetText( menu, 0, GS(LNG_MENU_ID3TAG_EDIT    ));          
	MenuSetText( menu, 1, GS(LNG_MENU_ID3TAG_MASS    ));          
	MenuSetText( menu, 2, GS(LNG_MENU_ID3TAG_RENAME  ));          
	MenuSetText( menu, 3, GS(LNG_MENU_ID3TAG_WRITE   ));          
	MenuSetText( menu, 4, GS(LNG_MENU_ID3TAG_CONVERT ));          
	MenuSetText( menu, 5, GS(LNG_MENU_ID3TAG_ORGANIZE));          

	// About Menu
	menu = MenuGetPopup(GetMenu(m_hDlg), 3);
	MenuSetText( menu, 0, GS(LNG_MENU_ABOUT         ));          


	DrawMenuBar( m_hDlg );
}



// ********************************************************************
void CDlgPlaylist::SetTextMenuPop(HMENU menu, int nType)
{
	HMENU menupop;

	switch(nType)
	{
	case POPMENU_SEARCH_OPTION:// Search option
		MenuSetText(menu, 0, GS(LNG_MENU_SEARCH_ARTIST          ));                 
		MenuSetText(menu, 1, GS(LNG_MENU_SEARCH_TITLE           ));                 
		MenuSetText(menu, 2, GS(LNG_MENU_SEARCH_ALBUM           ));                 
		MenuSetText(menu, 3, GS(LNG_MENU_SEARCH_COMMENT         ));                 
		MenuSetText(menu, 4, GS(LNG_MENU_SEARCH_FILENAME        ));                 
		MenuSetText(menu, 6, GS(LNG_MENU_SEARCH_SELECTALL       ));                 
		MenuSetText(menu, 7, GS(LNG_MENU_SEARCH_SELECTNONE      ));                 
		MenuSetText(menu, 9, GS(LNG_MENU_SEARCH_MATCHCASEMODE   ));                 
		break;
	case POPMENU_FILE:// File popupmenu
		MenuSetText( menu, 0, GS(LNG_MENU_NEW));
		MenuSetText( menu, 1, GS(LNG_MENU_OPEN));
		MenuSetText( menu, 3, GS(LNG_MENU_SAVE));
		MenuSetText( menu, 4, GS(LNG_MENU_SAVE_AS));
		MenuSetText( menu, 6, GS(LNG_MENU_IMPORT));
		MenuSetText( menu, 7, GS(LNG_MENU_EXPORT));
		break;
	case POPMENU_ADDFILE:// Add popupmenu
		MenuSetText( menu, 0, GS(LNG_MENU_ADD_FILE));
		MenuSetText( menu, 1, GS(LNG_MENU_ADD_DIRECTORY));
		break;
	case POPMENU_PLAYLIST:
		MenuSetText(menu, 0, GS(LNG_MENU_PLAYLIST_PLAY                 ));                     
		MenuSetText(menu, 1, GS(LNG_MENU_PLAYLIST_PLAYALL              ));                     
		MenuSetText(menu, 3, GS(LNG_MENU_PLAYLIST_EDITTAG              ));                     
		MenuSetText(menu, 5, GS(LNG_MENU_PLAYLIST_ENQUEUE              ));                     
		MenuSetText(menu, 6, GS(LNG_MENU_PLAYLIST_ENQUEUEALL           ));                     
		MenuSetText(menu, 7, GS(LNG_MENU_PLAYLIST_CLEARCURRENTPLAYLIST ));                     
		MenuSetText(menu, 8, GS(LNG_MENU_PLAYLIST_SHOWHIDEPLAYLIST     ));                     
		MenuSetText(menu,10, GS(LNG_MENU_PLAYLIST_SELECTALL            ));                  
		MenuSetText(menu,11, GS(LNG_MENU_PLAYLIST_SELECTNONE           ));                  
		MenuSetText(menu,12, GS(LNG_MENU_PLAYLIST_SELECTTOGGLE         ));                  
		MenuSetText(menu,13, GS(LNG_MENU_PLAYLIST_SELECT_WITHSAME      ));                  
		menupop = MenuGetPopup(menu, 13);	// Select the Same...
		  MenuSetText(menupop,1, GS(LNG_ARTIST   ));
		  MenuSetText(menupop,2, GS(LNG_TITLE    ));
		  MenuSetText(menupop,6, GS(LNG_GENRE    ));
		  MenuSetText(menupop,4, GS(LNG_YEAR     ));
		  MenuSetText(menupop,3, GS(LNG_ALBUM    ));
		  MenuSetText(menupop,5, GS(LNG_COMMENT  ));
		  MenuSetText(menupop,8, GS(LNG_MODE     ));
		  MenuSetText(menupop,9, GS(LNG_BITRATE  ));
		  MenuSetText(menupop,10,GS(LNG_FREQ     ));
		  MenuSetText(menupop,11,GS(LNG_LAYER    ));
		  MenuSetText(menupop,12,GS(LNG_TIME     ));
		  MenuSetText(menupop,13,GS(LNG_SIZE     ));
		  MenuSetText(menupop,14,GS(LNG_FILENAME ));
		MenuSetText(menu, 15, GS(LNG_MENU_PLAYLIST_SELECTALLDEADFILES ));
		MenuSetText(menu, 16, GS(LNG_MENU_PLAYLIST_SELECTCROP         ));
		MenuSetText(menu, 17, GS(LNG_MENU_PLAYLIST_DEL                ));
		MenuSetText(menu, 18, GS(LNG_MENU_PLAYLIST_REMOVEFILE         ));
		MenuSetText(menu, 20, GS(LNG_MENU_PLAYLIST_SHOWDUPLICATE      ));
		MenuSetText(menu, 21, GS(LNG_MENU_PLAYLIST_PROPERTY           ));
		break;
	}
	


}


// ********************************************************************
void CDlgPlaylist::RefreshAllCtrl_Lang()
{
	// Reactualise les menus
	SetTextMenu();

	// Recherche
	DlgSetText(m_hDlg, R_SEARCH, GS(LNG_SEARCH) );

	// Les 2 boutons File, Add
	InvalidateRect(GetDlgItem(m_hDlg, IDC_BUTTON_FILE), NULL, TRUE);
	InvalidateRect(GetDlgItem(m_hDlg, IDC_BUTTON_ADD), NULL, TRUE);
	
	// Reaffecte les nvx noms aux colonnes					
	setting->DefaultColumnName();	

	// Reaffiche les entetes de colonnes dans le bon ordre+largeure
	LV_ColumnSaveSetting();
	LV_ColumnResetDisplay(FALSE);

	// Reactualise la barre d'etat
	UpdateStatus(); 
}










