#include "priv.h"

#include "resource.h"
#include "definition.h"
#include "Dlg.h"
#include "Util.h"
#include "settings.h"
#include "Language.h"

// external declaration
extern INITSTRUCT gParam;
extern COLUMNSSTRUCT gDefault_column[COLUMN_MAX];
extern TEMPLATESTRUCT gaTemplateDefault[MAX_TEMPLATE];
extern CLanguage    *lng;
extern CSetting     *setting;

/* ************************************************************************ */
BOOL CDlgReset::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		InitLanguage();
		return FALSE;

	case WM_COMMAND:

		switch (LOWORD(wParam))
		{
		case IDOK:
			{
				int ret = ResetSettings();
				EndDialog(m_hDlg, ret);
				return TRUE;
			}

		case IDCANCEL:
			EndDialog(m_hDlg, 0);
			return TRUE;

		
		}
		
	}
	return FALSE;
}


/* ************************************************************** */
int CDlgReset::ResetSettings()
{
	int bSave = 0;
	
	if (IsDlgButtonChecked(m_hDlg, IDC_RESET_CSS) == BST_CHECKED ||
		IsDlgButtonChecked(m_hDlg, IDC_RESET_COLUMNS) == BST_CHECKED ||
		IsDlgButtonChecked(m_hDlg, IDC_RESET_ID3TOOLS) == BST_CHECKED ||
		IsDlgButtonChecked(m_hDlg, IDC_RESET_OPTION) == BST_CHECKED )
		bSave = 1;

	if (bSave)
	{
		if (MessageBox(m_hDlg, GS(LNG_RESETDEFAULT), GS(LNG_QUESTION), MB_OKCANCEL|MB_ICONQUESTION) == IDOK)
			bSave = 1;
		else
			bSave = 0;
	}

	if (bSave)
	{
		if (IsDlgButtonChecked(m_hDlg, IDC_RESET_CSS) == BST_CHECKED)
			setting->ResetHTML();	

		if (IsDlgButtonChecked(m_hDlg, IDC_RESET_COLUMNS) == BST_CHECKED)
			setting->ResetColumn();

		if (IsDlgButtonChecked(m_hDlg, IDC_RESET_ID3TOOLS) == BST_CHECKED)
			setting->ResetID3Tools();	

		if (IsDlgButtonChecked(m_hDlg, 	IDC_RESET_OPTION) == BST_CHECKED)
			setting->ResetOption();
	}

	return bSave;
}


// **********************************************
void CDlgReset::InitLanguage()
{
	LNGRESSTRUCT lrs[8] = 
	{
		{ R_RESET_DESCRIPTION, LNG_RESET_DESCRIPTION },
		{ R_RESET_FRAME_CATEGORIE, LNG_RESET_FRAME_CATEGORIE },
		{ IDC_RESET_CSS, LNG_RESET_HTML },
		{ IDC_RESET_COLUMNS, LNG_RESET_VIEW_COLUMN },
		{ IDC_RESET_ID3TOOLS, LNG_RESET_ID3TOOLS },
		{ IDC_RESET_OPTION, LNG_RESET_OPTION },
		{ IDOK, LNG_BTN_OK },
		{ IDCANCEL, LNG_BTN_CANCEL }
	};
	
	for (int i = 0 ; i<NUMELEMENTS(lrs); i++)
		DlgSetText(m_hDlg, lrs[i].idRes, GS(lrs[i].idLNG) );

	SetWindowText(m_hDlg, GS(LNG_RESET_TITLE) );
}


// //////////////////////////////
// End part for DlgReset Dialog
// //////////////////////////////