/* 
 *  language.cpp 
 *
 *	Copyright (C) Alberto Vigata - January 2000 - ultraflask@yahoo.com
 *
 *  This file is part of FlasKMPEG, a free MPEG to MPEG/AVI converter
 *	
 *  FlasKMPEG is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  FlasKMPEG is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#include <windows.h>
#include <winbase.h>
#include "priv.h"
#include "Util.h"
#include "Language.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
static TCHAR filenames[MAX_LANGUAGES][MAX_PATH];
static TCHAR* strings[MAX_STRINGS];
static int strings_alloc[MAX_STRINGS];


//////////////////////////////////////////////////////////////////////
CLanguage::CLanguage()
{
	m_nSelectedLanguage=0;
}
//////////////////////////////////////////////////////////////////////
CLanguage::~CLanguage()
{
	DeleteLang();
}

//////////////////////////////////////////////////////////////////////
void CLanguage::DeleteLang()
{
	//Deallocate strings
	for(int i=0; i<MAX_STRINGS; i++)
	{			      
		if(strings_alloc[i]==1)
			free(strings[i]);
	}
	
	//RAZ all NULL pointer
	ZeroMemory(strings, MAX_STRINGS * sizeof(TCHAR*));
	ZeroMemory(strings_alloc, MAX_STRINGS * sizeof(int));

}
//////////////////////////////////////////////////////////////////////
int CLanguage::Start(TCHAR* szSetLang)
{
	WIN32_FIND_DATA find_data;
	TCHAR     szDirectory[MAX_PATH];
	TCHAR     szLang[MAX_PATH];
	HANDLE   search_handle;
	int i, nLen;

	GetWinampPathPlugin(szDirectory);
	_tcscat(szDirectory,_T("*.lang.ezp") );

	// Dans la liste il y a au moins pas defaut
	// une langue "English (Default)"
	m_nCountLanguage = 1;
	_tcscpy( filenames[0], _T("English(Default)") );

	i = 1;
	search_handle = FindFirstFile(szDirectory, &find_data);
	if(search_handle!=INVALID_HANDLE_VALUE)
	{
		do
		{
			_tcscpy( szLang, find_data.cFileName );
			nLen = _tcslen(szLang);
			szLang[nLen - 9] = 0;		// remove ".lang.ezp"
			
			_tcscpy( filenames[i], szLang );

			if (_tcscmp(szSetLang, szLang) == 0)
				m_nSelectedLanguage = i;
			
			i++;
		} while( FindNextFile(search_handle, &find_data ));

		FindClose(search_handle);
		m_nCountLanguage = i;
	}
	
	if(m_nCountLanguage==1 && m_nSelectedLanguage == 0)
	{	// no language found, so load default string 'English (default)'
		m_nSelectedLanguage=0;
		LoadDefaultStrings();
	}
	else
	{
		
		//Load strings from files (m_nSelectedLanguage)
		ReadLanguage();
	}
	
	return 0;
}
//////////////////////////////////////////////////////////////////////
int CLanguage::GetNumberLang()
{
	return m_nCountLanguage;
}
//////////////////////////////////////////////////////////////////////
TCHAR * CLanguage::GetLanguage()
{
	int index=0;

	if(m_nSelectedLanguage > MAX_LANGUAGES)
		index = 0;
	else
		index = m_nSelectedLanguage;

	return filenames[index];
}
//////////////////////////////////////////////////////////////////////
TCHAR * CLanguage::GetLanguage(int index)
{
	if(index > MAX_LANGUAGES)
		index = 0;
	
	return filenames[index];
}

//////////////////////////////////////////////////////////////////////
int CLanguage::GetLanguageID()
{
	return m_nSelectedLanguage;
}
//////////////////////////////////////////////////////////////////////
int CLanguage::SetLanguageID(int nNewLang)
{
	m_nSelectedLanguage = nNewLang;
	return 1;
}

//////////////////////////////////////////////////////////////////////
TCHAR* CLanguage::GetString(int nIndex)
{
	if(nIndex > MAX_STRINGS)
		return _T("");

	return strings[nIndex] ? strings[nIndex]: _T("");
}

//////////////////////////////////////////////////////////////////////
int CLanguage::ReadLanguage()
{
	FILE *f=NULL;
	TCHAR token=0;
	int nID, nChars;
	TCHAR full_name[MAX_PATH];

	// Si=0 chargement langue par defaut
	if (m_nSelectedLanguage == 0)
	{
		DeleteLang();
		LoadDefaultStrings();
		return 1;
	}

	GetWinampPathPlugin(full_name);
	_tcscat(full_name,filenames[m_nSelectedLanguage]);
	_tcscat(full_name, _T(".lang.ezp"));

	//open file
	f = _tfopen(full_name, _T("rb"));
	if(!f)
		return 0;

	// Delete strings list
	DeleteLang();
	

	while(!feof(f))
	{
		//Search for token #
		while(token!='#' && !feof(f))
			fread(&token, 1, 1, f);
		//get number after token
		if(!feof(f))
		{
			fscanf(f, "%d", &nID);
			if(nID>MAX_STRINGS)
				continue;
		}

		//Search for token $
		while(token!='#' && !feof(f))
			fread(&token, 1, 1, f);
		while(token!='"' && !feof(f))
			fread(&token, 1, 1, f);

		//Read every single byte of the string
		if(!feof(f))
		{
			fread(&token, 1, 1, f);
			nChars = 0;

			strings[nID]=NULL;

			while(token!='"' && !feof(f))
			{
				nChars++;
				strings[nID]=(TCHAR *)realloc(strings[nID], nChars*sizeof(TCHAR));
				strings[nID][nChars - 1] = token;
				fread(&token, 1, 1, f);
			}
			//NULL terminate the string
			nChars++;
			strings[nID]=(TCHAR *)realloc(strings[nID], nChars*sizeof(TCHAR));
			strings[nID][nChars - 1] = 0;

			strings_alloc[nID]=1;
		}


	}

	
	fclose(f);
	return 1;
}



//////////////////////////////////////////////////////////////////////
void CLanguage::LoadDefaultStrings()
{
	//RAZ all NULL pointer
	ZeroMemory(strings, MAX_STRINGS * sizeof(TCHAR*));
	ZeroMemory(strings_alloc, MAX_STRINGS * sizeof(int));

strings[LNG_LANGUAGE_PACK] =_T("English");
strings[LNG_APPNAME] =_T("EZPlaylist for Winamp");
strings[LNG_TIPS_EXTPLAYLIST] =_T("Extended playlist");
strings[LNG_TIPS_PREVIOUSTRACK] =_T("Previous track");
strings[LNG_TIPS_PLAYPAUSE] =_T("Play / Pause");
strings[LNG_TIPS_STOP] =_T("Stop");
strings[LNG_TIPS_NEXTTRACK] =_T("Next track");
strings[LNG_TIPS_LOADFILE] =_T("Load file");
strings[LNG_TIPS_VOLUME] =_T("Volume");
strings[LNG_TIPS_NOTAG] =_T("No Tag");
strings[LNG_TIPS_EZMENU] =_T("Menu EZ-Playlist for Winamp");
strings[LNG_BTN_OK] =_T("OK");
strings[LNG_BTN_CANCEL] =_T("Cancel");
strings[LNG_BTN_APPLY] =_T("Apply");
strings[LNG_BTN_UPDATE] =_T("&Update");
strings[LNG_BTN_FILE] =_T("File");
strings[LNG_BTN_GO] =_T("GO");
strings[LNG_BTN_TRANSFER] =_T("Transfer");
strings[LNG_BTN_UP] =_T("&Up");
strings[LNG_BTN_DOWN] =_T("&Down");
strings[LNG_QUESTION] =_T("Question");
strings[LNG_ERROR] =_T("Error");
strings[LNG_INFORMATION] =_T("Information");
strings[LNG_FAILED] =_T("Failed");
strings[LNG_SUCCESS] =_T("Success");
strings[LNG_MSG_SELECT_COLUMN] =_T("At least 1 column must be present !!!");
strings[LNG_MSG_SELECT_FILE] =_T("At least 1 file must be selected !!!");
strings[LNG_MSG_CHOOSELOG] =_T("Choose a log file !!!");
strings[LNG_MSG_ERRLOG] =_T("Error log file !!!");
strings[LNG_LOG_FILE] =_T("file%d=(%s): %s");
strings[LNG_STATUS_WORKING] =_T("Working %d/%d: current file: %s");
strings[LNG_TITLE_ID3TOOLS] =_T("ID3Tag Tools");
strings[LNG_LOG_COPYID3V1_ID3V2] =_T("Copy ID3v1 to ID3v2 TAG %d files (Trace LOG)");
strings[LNG_LOG_COPYID3V2_ID3V1] =_T("Copy ID3v2 to ID3v1 TAG %d files (Trace LOG)");
strings[LNG_LOG_REMOVEID3V1] =_T("Remove ID3v1 TAG %d files (Trace LOG)");
strings[LNG_LOG_REMOVEID3V2] =_T("Remove ID3v2 TAG %d files (Trace LOG)");
strings[LNG_LOG_REMOVEID3V1_ID3V2] =_T("Remove ID3v1+ID3v2 TAG %d files (Trace LOG)");
strings[LNG_RESULT_COPYID3V1_ID3V2] =_T("Copy ID3v1 --> ID3v2 on %d file(s)");
strings[LNG_RESULT_COPYID3V2_ID3V1] =_T("Copy ID3v2 --> ID3v1 on %d file(s)");
strings[LNG_RESULT_REMOVEID3V1] =_T("Remove ID3v1 TAG on %d file(s)");
strings[LNG_RESULT_REMOVEID3V2] =_T("Remove ID3v2 TAG on %d file(s)");
strings[LNG_RESULT_REMOVEID3V1_ID3V2] =_T("Remove ID3v1+ID3v2 TAG on %d file(s)");
strings[LNG_MSG_EDITTAG_INFO] =_T("Information: Edit Tag");
strings[LNG_MSG_EDITTAG_ERR] =_T("Error: Edit Tag");
strings[LNG_MSG_EDITTAG_CONFIRM] =_T("Write modifications ?");
strings[LNG_MSG_EDITTAG_SELECTFILE] =_T("Select before a song");
strings[LNG_MSG_EDITTAG_CANTREAD] =_T("Can't read/write ID3 Tag !!!");
strings[LNG_MSG_EDITTAG_CANTWRITE] =_T("Can't write ID3 Tag !!!");
strings[LNG_MSG_EDITTAG_STATUSUPDATING] =_T("Updating file (Writing new tag), please wait...");
strings[LNG_MSG_MASSTAG_INFO] =_T("Information: Mass Tag");
strings[LNG_MSG_MASSTAG_ERR] =_T("Error: Mass Tag");
strings[LNG_MSG_MASSTAG_LOG] =_T("Mass set %s TAG %d files (Trace LOG)");
strings[LNG_MSG_MASSTAG_RESULT] =_T("Mass set ID3TAG on %d file(s)");
strings[LNG_MSG_RENAMEFILE_INFO] =_T("Information: Rename file from ID3 TAG");
strings[LNG_MSG_RENAMEFILE_ERR] =_T("Erreur: Rename file from ID3 TAG");
strings[LNG_MSG_RENAMEFILE_LOG] =_T("Rename %d files from ID3Tag (Trace LOG)");
strings[LNG_MSG_RENAMEFILE_RESULT] =_T("Rename filename from ID3TAG on %d file(s)");
strings[LNG_UNKOWNARTIST] =_T("Unkown artist");
strings[LNG_UNTITLED] =_T("Untitled");
strings[LNG_UNKOWNALBUM] =_T("Unkown album");
strings[LNG_UNKOWNGENRE] =_T("Unkown genre");
strings[LNG_MSG_WRITETAG_INFO] =_T("Information: Write TAG from filename");
strings[LNG_MSG_WRITETAG_ERR] =_T("Erreur: Write TAG from filename");
strings[LNG_MSG_WRITETAG_LOG] =_T("Write %s Tag from %d files (Trace LOG)");
strings[LNG_MSG_WRITETAG_RESULT] =_T("Write ID3TAG from filename on %d file(s)");
strings[LNG_MSG_ORGANIZEFILE_LOG ] =_T("Organize %d files (Trace LOG)");
strings[LNG_MSG_ORGANIZE_RESULT] =_T("Organize %d file(s) target directory %");
strings[LNG_MSG_CHOOSEPATHDEST] =_T("Choose a target directory !!!");
strings[LNG_LOG_ORGANIZE_FILE] =_T("file%d=(%s): %s --> %s");
strings[LNG_MSG_TIME_MUSTBENOTNULL] =_T("Please custom time must be a number not null !!!");
strings[LNG_MSG_CONFIRM_MASS] =_T("Mass set ID3TAG on %d file(s) ?");
strings[LNG_MSG_CONFIRM_RENAME] =_T("Rename file from ID3TAG on %d file(s) ?");
strings[LNG_MSG_CONFIRM_WRITETAGFROMFILENAME] =_T("Write ID3TAG based on %d file(s) ?");
strings[LNG_MSG_CONFIRM_COPYID3V1TOID3V2] =_T("Copy ID3v1 --> ID3v2 on %d file(s) ?");
strings[LNG_MSG_CONFIRM_COPYID3V2TOID3V1] =_T("Copy ID3v2 --> ID3v1 on %d file(s) ?");
strings[LNG_MSG_CONFIRM_REMOVEID3V1] =_T("Remove ID3v1 on %d file(s) ?");
strings[LNG_MSG_CONFIRM_REMOVEID3V2] =_T("Remove ID3v2 on %d file(s) ?");
strings[LNG_MSG_CONFIRM_REMOVEID3V1ID3V2] =_T("Remove ID3v1+ID3v2 on %d file(s) ?");
strings[LNG_MSG_CONFIRM_ORGANIZE] =_T("Organize %d files(s) mp3 to target directories ?");
strings[LNG_MOREFILES] =_T("<More files> ...");
strings[LNG_PROCEEDTHEM] =_T("Would you like to proceed them ?");
strings[LNG_SEPARATORNOTNULL] =_T("Separator must be not null");
strings[LNG_CHECKID3] =_T("Check before ID3v1 / ID3v2");
strings[LNG_COLUMNFILTER_GENRE] =_T("Genre");
strings[LNG_COLUMNFILTER_COUNT] =_T("Count");
strings[LNG_RESETDEFAULT] =_T("Are your sure to reset settings ?");
strings[LNG_ARTIST] =_T("Artist");
strings[LNG_TITLE] =_T("Title");
strings[LNG_ALBUM] =_T("Album");
strings[LNG_YEAR] =_T("Year");
strings[LNG_COMMENT] =_T("Comment");
strings[LNG_GENRE] =_T("Genre");
strings[LNG_TRACK] =_T("Track#");
strings[LNG_MODE] =_T("Mode");
strings[LNG_BITRATE] =_T("Bitrate");
strings[LNG_FREQ] =_T("Freq.");
strings[LNG_LAYER] =_T("Layer");
strings[LNG_TIME] =_T("Time");
strings[LNG_SIZE] =_T("Size");
strings[LNG_FILENAME] =_T("File name");
strings[LNG_ID3TAG] =_T("ID3Tag");
strings[LNG_BPM] =_T("BPM");
strings[LNG_BPM_DESCRIPTION] =_T("BPM (Beat per minute");
strings[LNG_INIT_KEY] =_T("Init Key");
strings[LNG_SUBTITLE] =_T("Subtitle");
strings[LNG_REMIXED] =_T("Remixed");
strings[LNG_COMPOSER] =_T("Composer");
strings[LNG_HTML_DATE] =_T("Date: %s");
strings[LNG_FILTEROPENDIALOG_LOG] =_T("ASCII file (*.txt)|*.TXT|Log file (*.log)|*.log|");
strings[LNG_FILTEROPENDIALOG_IMPORT] =_T("M3U Playlist file (*.m3u)|*.m3u|PLS Playlist file (*.pls)|*.pls|");
strings[LNG_FILTEROPENDIALOG_EXPORT] =_T("HTML File (*.html)|*.html|M3U Playlist file (*.m3u)|*.m3u|PLS Playlist file (*.pls)|*.pls|TEXT file (separator: tabulation) (*.txt)|*.txt|TEXT file (separator: space) (*.txt)|*.txt|CSV file (separator: comma separator) (*.csv)|*.csv|");
strings[LNG_FILTERADDFILE] =_T("MP3 Audio Files (*.mp3)|*.mp3|MPEG Audio Files (*.mp1;*.mp2;*.mp3)|*.mp1;*.mp2;*.mp3|");
strings[LNG_IMPORTPLAYLIST] =_T("Import playlist");
strings[LNG_EXPORTPLAYLIST] =_T("Export data");
strings[LNG_STATUS_TOTALFILE] =_T("Total files: %i");
strings[LNG_STATUS_TOTALSIZE] =_T("Total size: %u MB");
strings[LNG_STATUS_TOTALSIZE_FLOAT] =_T("Total size: %.2f MB");
strings[LNG_STATUS_AVGSIZE_FLOAT] =_T("Avg file size: %.2f MB");
strings[LNG_STATUS_AVGLENGTH] =_T("Avg length: %s");
strings[LNG_STATUS_SELLENGTH] =_T("Sel length: %s");
strings[LNG_STATUS_TOTALLENGTH] =_T("Total length: %s");
strings[LNG_PROPERTY_ID3TAG] =_T("\t%-d:\tNo Tag\r\n\t%-d:\tID3v1\r\n\t%-d:\tID3v2\r\n\t%-d:\tID3v1+v2\r\n");
strings[LNG_SHORTTIME_FRMT] =_T("%2d minutes and %2d seconds");
strings[LNG_FULLTIME_FRMT] =_T("%2d hours %2d minutes and %2d seconds");
strings[LNG_CLIPBORADMEMERROR] =_T("Error Allocating mem for clipboard!");
strings[LNG_NOTHING] =_T("Nothing");
strings[LNG_VOLUME_VALUE] =_T("Volume %d%%");
strings[LNG_ERR_FILEPLAYLIST] =_T("Error file: EZ-Playlist");
strings[LNG_ERR_BADFILE] =_T("Bad file format: EZ-Playlist");
strings[LNG_ERR_CORRUPTFILE] =_T("Corrupt file: EZ-Playlist");
strings[LNG_ERR_BADFILEPLS] =_T("Bad file: PLS playlist");
strings[LNG_ADDFILE_TITLE] =_T("Add files");
strings[LNG_ADDFILE_TOOMANYFILE] =_T("Too many files selected");
strings[LNG_ADDFILE_ERROR] =_T("Add files Error");
strings[LNG_SHBROWSEFOLDER_CHECKBOX] =_T("Recurse &subdirectories");
strings[LNG_TITLEFILTER] =_T("Filter by %s (%s)");
strings[LNG_ALLFILTER] =_T("All %ss");
strings[LNG_TITLE_INFOPLAYLIST] =_T("Properties of the selection");
strings[LNG_ERROR_REMOVEFILE] =_T("Error deleting: %s");
strings[LNG_CONFIRM_REMOVEFILE] =_T("Using this command will remove all selected files PERMANENTLY from disk. Are you sure you want to do this ?");
strings[LNG_MENU_FILE] =_T("&File");
strings[LNG_MENU_NEW] =_T("&New|Ctrl+N");
strings[LNG_MENU_OPEN] =_T("&Open...|Ctrl+O");
strings[LNG_MENU_SAVE] =_T("&Save...|Ctrl+S");
strings[LNG_MENU_SAVE_AS] =_T("S&ave As...");
strings[LNG_MENU_RELOAD_WINAMP] =_T("Reload from WinAmp|F5");
strings[LNG_MENU_ADD] =_T("Add");
strings[LNG_MENU_ADD_FILE] =_T("Add files...|Ctrl+Insert");
strings[LNG_MENU_ADD_DIRECTORY] =_T("Add directory|Ctrl+Shift+Insert");
strings[LNG_MENU_INSERT_EZ3       ] =_T("Insert EZ3 File...");
strings[LNG_MENU_IMPORT] =_T("&Import");
strings[LNG_MENU_EXPORT] =_T("&Export");
strings[LNG_MENU_PROPERTY_PLAYLIST] =_T("Playlist information...|Ctrl+I");
strings[LNG_MENU_PROPERTY_VOLUME] =_T("Volume  information");
strings[LNG_MENU_QUIT] =_T("&Quit|Ctrl+Q");
strings[LNG_MENU_OPTIONS] =_T("&Option");
strings[LNG_MENU_OPT_COLUMN] =_T("&Columns...|Ctrl+L");
strings[LNG_MENU_OPT_OPTION] =_T("&Option...|Ctrl+M");
strings[LNG_MENU_OPT_CONFIGURE] =_T("Configure...|Ctrl+P");
strings[LNG_MENU_OPT_HTMLCSS] =_T("HTML CSS class editor...|Ctrl+H");
strings[LNG_MENU_OPT_RESET_DEFAULT] =_T("Reset to default...");
strings[LNG_MENU_OPT_FILTER] =_T("Filter...|Ctrl+K");
strings[LNG_MENU_OPT_MEDIA_WIZARD] =_T("Media Wizard|Ctrl+Y");
strings[LNG_MENU_OPT_VIEW] =_T("&View");
strings[LNG_MENU_OPT_EXPLORER] =_T("Explorer|F6");
strings[LNG_MENU_OPT_EXPLORER_OPTION] =_T("Option explorer...");
strings[LNG_MENU_OPT_LANGUAGE] =_T("Language");
strings[LNG_MENU_ID3TAG] =_T("&ID3Tag");
strings[LNG_MENU_ID3TAG_EDIT] =_T("Edit ID3...|Ctrl+E");
strings[LNG_MENU_ID3TAG_MASS] =_T("Mass set ID3...");
strings[LNG_MENU_ID3TAG_RENAME] =_T("Rename file from ID3...");
strings[LNG_MENU_ID3TAG_WRITE] =_T("Write ID3 from file...");
strings[LNG_MENU_ID3TAG_CONVERT] =_T("Conversion / Remove ID3...");
strings[LNG_MENU_ID3TAG_ORGANIZE] =_T("Organize MP3 file...");
strings[LNG_MENU_ABOUT] =_T("&About|F1");
strings[LNG_MENU_SEARCH_ARTIST] =_T("Artist|Ctrl+1");
strings[LNG_MENU_SEARCH_TITLE] =_T("Title|Ctrl+2");
strings[LNG_MENU_SEARCH_ALBUM] =_T("Album|Ctrl+3");
strings[LNG_MENU_SEARCH_COMMENT] =_T("Comment|Ctrl+4");
strings[LNG_MENU_SEARCH_FILENAME] =_T("File name|Ctrl+5");
strings[LNG_MENU_SEARCH_SELECTALL] =_T("Search all");
strings[LNG_MENU_SEARCH_SELECTNONE] =_T("Search none");
strings[LNG_MENU_SEARCH_MATCHCASEMODE] =_T("Match case mode");
strings[LNG_MENU_CAPITALIZE_UNCHANGED] =_T("unChaNGeD");
strings[LNG_MENU_CAPITALIZE_ALLLOWERCASE] =_T("all lowercase");
strings[LNG_MENU_CAPITALIZE_ALLUPPERCASE] =_T("ALL UPPERCASE");
strings[LNG_MENU_CAPITALIZE_FIRSTLETTEROFSECTIONUPPERCASE] =_T("First letter of section uppercase");
strings[LNG_MENU_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE] =_T("First Letter Of Each Word Uppercase");
strings[LNG_MENU_CONVERT_NO] =_T("No Convert");
strings[LNG_MENU_CONVERT_UNDERSCORE_TO_SPACE] =_T("Convert Underscore to space");
strings[LNG_MENU_CONVERT_SPACE_TO_UNDERSCORE] =_T("Convert Space to underscore");
strings[LNG_MENU_SWAPID3_TITLE_ARTIST] =_T("Title <--> Artist");
strings[LNG_MENU_SWAPID3_TITLE_ALBUM] =_T("Title <--> Album");
strings[LNG_MENU_SWAPID3_ARTIST_TITLE] =_T("Artist <--> Title");
strings[LNG_MENU_SWAPID3_ARTIST_ALBUM] =_T("Artist <--> Album");
strings[LNG_MENU_SWAPID3_ALBUM_TITLE] =_T("Album <--> Title");
strings[LNG_MENU_SWAPID3_ALBUM_ARTIST] =_T("Album <--> Artist");
strings[LNG_MENU_PLAYLIST_PLAY] =_T("Play|Enter");
strings[LNG_MENU_PLAYLIST_PLAYALL] =_T("Play all");
strings[LNG_MENU_PLAYLIST_EDITTAG] =_T("Edit ID3Tag|Ctrl+E");
strings[LNG_MENU_PLAYLIST_ENQUEUE] =_T("Enqueue to Winamp playlist");
strings[LNG_MENU_PLAYLIST_ENQUEUEALL] =_T("Enqueue all to Winamp playlist");
strings[LNG_MENU_PLAYLIST_CLEARCURRENTPLAYLIST] =_T("Clear current Winamp playlist");
strings[LNG_MENU_PLAYLIST_SHOWHIDEPLAYLIST] =_T("Show/Hide Winamp playlist");
strings[LNG_MENU_PLAYLIST_SELECTALL] =_T("Select All|Ctrl+A");
strings[LNG_MENU_PLAYLIST_SELECTNONE] =_T("Select none");
strings[LNG_MENU_PLAYLIST_SELECTTOGGLE] =_T("Select toggle|Ctrl+T");
strings[LNG_MENU_PLAYLIST_SELECT_WITHSAME] =_T("Select with same");
strings[LNG_MENU_PLAYLIST_SELECTALLDEADFILES] =_T("Select all dead files");
strings[LNG_MENU_PLAYLIST_SELECTCROP] =_T("Crop|Ctrl+Del");
strings[LNG_MENU_PLAYLIST_DEL] =_T("Remove file(s) from playlist|Del");
strings[LNG_MENU_PLAYLIST_REMOVEFILE] =_T("Delete Physically selected file(s)");
strings[LNG_MENU_PLAYLIST_SHOWDUPLICATE] =_T("Find duplicates");
strings[LNG_MENU_PLAYLIST_PROPERTY] =_T("Property|Alt+Enter");
strings[LNG_COL_TITLE] =_T("Columns");
strings[LNG_COL_VIEWINFO_FROM] =_T("&View Information from:");
strings[LNG_COL_SHOW_NUMBER] =_T("&Show number in playlist");
strings[LNG_COL_SHOW_GRID] =_T("Show &Grid");
strings[LNG_COL_FRAME_SELECTFIELDS] =_T("Select fields");
strings[LNG_COL_AVAILABLE_FIELDS] =_T("Available fields:");
strings[LNG_COL_SELECTED_FIELDS] =_T("Selected fields:");
strings[LNG_COL_BTN_ADD] =_T("Add>>");
strings[LNG_COL_BTN_ADD_ALL] =_T("A&dd all >>");
strings[LNG_COL_BTN_REMOVE] =_T("<< Remove");
strings[LNG_COL_BTN_REMOVE_ALL] =_T("<< Remove all");
strings[LNG_MSG_SELECT_FIELD] =_T("Choose at least 1 field must be present !!!");
strings[LNG_MSG_ERR_BAD_NUMBER] =_T("Bad number format !!!");
strings[LNG_MSG_ERR_BAD_RANGE_TIME] =_T("Time value error: Enter number between %d to %d");
strings[LNG_MSG_ERR_BAD_RANGE_FILESIZE] =_T("File size value error: Enter number between %d%% to %d%%");
strings[LNG_OPT_TITLE] =_T("Option");
strings[LNG_OPT_FRAME_EXPORT] =_T("Export");
strings[LNG_OPT_ALL_ITEM] =_T("E&xport all items");
strings[LNG_OPT_SELECTED_ITEM] =_T("Export only &selected items");
strings[LNG_OPT_FRAME_HTML] =_T("HTML options");
strings[LNG_OPT_TABLE_FORMAT] =_T("&Table format");
strings[LNG_OPT_LISTING_FORMAT] =_T("&Listing format");
strings[LNG_OPT_ADD_HYPERLINK] =_T("add &Hyperlink to file");
strings[LNG_OPT_BOTH_ID3V1ID3V2] =_T("Both ID3v1 and ID3v2");
strings[LNG_OPT_FRAME_STARTUP] =_T("Load on startup");
strings[LNG_OPT_START_EZPLAYLIST] =_T("E&ZPlaylist");
strings[LNG_OPT_START_WINAMPPLAYLIST] =_T("&Winamp Playlist");
strings[LNG_OPT_FRAME_MOUSE] =_T("Mouse Double Click");
strings[LNG_OPT_DBLCLIC_PLAY] =_T("&Play");
strings[LNG_OPT_DBLCLIC_ENQUEUE] =_T("&Enqueue");
strings[LNG_OPT_DBLCLIC_EDIT_TAG] =_T("Edit &ID3 Tag");
strings[LNG_DUPLI_TITLE] =_T("Find duplicates");
strings[LNG_DUPLI_FRAME] =_T("Fields");
strings[LNG_DUPLI_DIFF_LENGTH] =_T("Difference between songs &length is less than");
strings[LNG_DUPLI_DIFF_SIZE] =_T("Difference between songs  file&size is less than");
strings[LNG_CFG_TITLE] =_T("EZPlaylist systray control for WinAmp");
strings[LNG_CFG_FRAME_CTRL] =_T("Show Tray controls");
strings[LNG_CFG_GROUP_IN_MENU] =_T("Group in systray menu");
strings[LNG_HTML_TITLE] =_T("HTML CSS class editor");
strings[LNG_HTML_FONT_REGULAR] =_T("Regular");
strings[LNG_HTML_FONT_BOLD] =_T("Bold");
strings[LNG_HTML_FONT_ITALIC] =_T("Italic");
strings[LNG_HTML_FONT_BOLD_ITALIC] =_T("Bold italic");
strings[LNG_HTML_MSG_HEXANOTVALID] =_T("Input is not a valid hexnumber");
strings[LNG_HTML_MSG_ERRHEXA] =_T("Error in hexvalue !!!");
strings[LNG_HTML_FRAME_CSS] =_T("CSS HTML");
strings[LNG_HTML_TEMPLATE] =_T("Template:");
strings[LNG_HTML_COLOR_TEXT] =_T("Color Text");
strings[LNG_HTML_COLOR_BACKGROUND] =_T("Color Background");
strings[LNG_HTML_FONT_NAME] =_T("Font name");
strings[LNG_HTML_FONT_STYLE] =_T("Font style");
strings[LNG_HTML_FONT_SIZE] =_T("Font size");
strings[LNG_HTML_BODY_TEXT] =_T("Body text");
strings[LNG_HTML_TITLE_PAGE] =_T("Title page");
strings[LNG_HTML_HEADER_COL] =_T("Header columns");
strings[LNG_HTML_TEXT_COLUMNS] =_T("Text columns");
strings[LNG_HTML_HYPERLINK_COLOR] =_T("Hyperlink color");
strings[LNG_RESET_TITLE] =_T("Reset Default Options");
strings[LNG_RESET_DESCRIPTION] =_T("Select the options to be restored to the default settings.");
strings[LNG_RESET_FRAME_CATEGORIE] =_T("Categories");
strings[LNG_RESET_HTML] =_T("&HTML Template Styles");
strings[LNG_RESET_VIEW_COLUMN] =_T("&View, Columns...");
strings[LNG_RESET_ID3TOOLS] =_T("&ID3 Tools settings");
strings[LNG_RESET_OPTION] =_T("&Option, configure settings");
strings[LNG_ABOUT_TITLE] =_T("About EZPlaylist for Winamp");
strings[LNG_ABOUT_BTN_HELP] =_T("Help >>");
strings[LNG_ABOUT_BTN_ABOUT] =_T("<< About");
strings[LNG_MEDIA_TITLE] =_T("Media Wizard (Match Time)");
strings[LNG_MEDIA_FRAME] =_T("Length");
strings[LNG_MEDIA_45MN] =_T("45 minutes");
strings[LNG_MEDIA_60MN] =_T("60 minutes");
strings[LNG_MEDIA_74MN] =_T("74 minutes (CD-R)");
strings[LNG_MEDIA_80MN] =_T("80 minutes");
strings[LNG_MEDIA_90MN] =_T("90 minutes");
strings[LNG_MEDIA_120MN] =_T("120 minutes");
strings[LNG_MEDIA_CUSTOM_MN] =_T("Custom");
strings[LNG_DLGID3TAG_TITLE] =_T("ID3Tag Tools");
strings[LNG_PAGE_MASSTAG_TITLE] =_T("Mass Tag");
strings[LNG_PAGE_RENAMEFILE_TITLE] =_T("Rename file from Tag");
strings[LNG_PAGE_WRITETAG_TITLE] =_T("Write Tag from filename");
strings[LNG_PAGE_CONVERSIONREMOVE_TITLE] =_T("Conversion / Remove Tag");
strings[LNG_PAGE_ORGANIZE_TITLE] =_T("Organize MP3 files");
strings[LNG_PAGE_EDITTAG_TITLE] =_T("Edit ID3Tag");
strings[LNG_PAGE_EDITTAG_MP3INFO1] =_T("Size: %d bytes\r\nLength: %lu seconds\r\n%s\r\n%s\r\n%d frames");
strings[LNG_PAGE_EDITTAG_MP3INFO2] =_T("%dHz %s\r\nCRCs: %s\r\nCopyrighted: %s\r\nOriginal: %s\r\nEmphasis: %s");
strings[LNG_ARTIST_SHORTCUT] =_T("&Artist");
strings[LNG_TITLE_SHORTCUT] =_T("&Title");
strings[LNG_ALBUM_SHORTCUT] =_T("Al&bum");
strings[LNG_YEAR_SHORTCUT] =_T("&Year");
strings[LNG_COMMENT_SHORTCUT] =_T("&Comment");
strings[LNG_GENRE_SHORTCUT] =_T("&Genre");
strings[LNG_TRACK_SHORTCUT] =_T("Trac&k#");
strings[LNG_DLGID3TAG_TRACE_LOG] =_T("Trace LO&G");
strings[LNG_DLGID3TAG_ONLY_SELECTED_ITEM] =_T("&Only selected item");
strings[LNG_DLGID3TAG_SEPARATOR] =_T("Separator:");
strings[LNG_DLGID3TAG_FILE_FORMAT] =_T("File format:");
strings[LNG_DLGID3TAG_OPTION] =_T("Option");
strings[LNG_DLGID3TAG_BTN_REMOVEID3V1] =_T("Remove ID3v1");
strings[LNG_DLGID3TAG_BTN_REMOVEID3V2] =_T("Remove ID3v2");
strings[LNG_DLGID3TAG_BTN_REMOVEID3V1_2] =_T("Remove ID3v1+ID3v2");
strings[LNG_DLGID3TAG_DIRECTORY] =_T("Directory name:");
strings[LNG_DLGID3TAG_TARGET] =_T("Target:");
strings[LNG_DLGID3TAG_BTN_MOVEUP] =_T("Move &Up");
strings[LNG_DLGID3TAG_BTN_MOVEDOWN] =_T("Move &Down");
strings[LNG_DLGID3TAG_BTN_SELECTALL] =_T("Select A&ll");
strings[LNG_DLGID3TAG_BTN_NONE] =_T("&None");
strings[LNG_SEARCH] =_T("Search:");
strings[LNG_PROGRESS_COMPLETE] =_T("Progress %d%% complete");
strings[LNG_HEADER_VOLUMENAME] =_T("Volume");
strings[LNG_HEADER_TOTALFILE] =_T("Total files");
strings[LNG_HEADER_TYPE] =_T("Type media");
strings[LNG_DRIVE_UNKNOWN ] =_T("Unknown drive");
strings[LNG_DRIVE_REMOVABLE] =_T("Removable drive");
strings[LNG_DRIVE_FIXED ] =_T("Hard drive");
strings[LNG_DRIVE_CDROM ] =_T("CDROM or DVDROM");
strings[LNG_DRIVE_OTHER ] =_T("Other drive");
strings[LNG_DRIVE_RAMDRIVE] =_T("Ram Drive");
strings[LNG_DRIVE_NETWORK] =_T("Network drive");
strings[LNG_OPTIONEXPLORER_TITLE] =_T("Option explorer");
strings[LNG_OPTIONEXPLORER_FRAME] =_T("Folder item");

}