/*****************************************************************************
    This file is part of MP3ext.
    Copyright (C) 1999  Michael Mutschler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: michael@mutschler.de
******************************************************************************/
// MP3MultiInfo.cpp: implementation of the MP3MultiInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "priv.h"
#include "MP3MultiInfo.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifndef BOOL
typedef int BOOL;
#endif
#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

#pragma intrinsic(strlen, strcat)

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MP3MultiInfo::MP3MultiInfo()
{
	m_CurrentDriveIndex = -1;	// no drive for the moment
	
	ResetAll();
}

MP3MultiInfo::~MP3MultiInfo()
{
	ResetAll();
}

void MP3MultiInfo::ResetAll()
{ 	
	ResetList();	
	ResetListDriveVolume();	
}

void MP3MultiInfo::DeleteInfo(int index)
{
	CMP3Info *mp3i = GetInfo(index);
	DWORD dwSerial = mp3i->GetDriveVolumeSerialNumber();
	int indexDrive = FindInfoDriveVolume(dwSerial);
	
	if (indexDrive != -1)
	{
		DRIVEVOLUMESTRUCT *dvs;
		dvs = GetInfoDriveVolume(indexDrive);

		if (dvs)
		{
			(dvs->nCountFile )--;
			if (dvs->nCountFile <=0)
				DeleteInfoDriveVolume(indexDrive);
		}
	}

	// remove MP3info's
	delete mp3i;

	// remove from array
	MP3InfoArray.erase(MP3InfoArray.begin() + index);
}

void MP3MultiInfo::ResetList()
{
	// remove all the MP3Info's
	for (int i=0; i<MP3InfoArray.size(); i++)
		delete GetInfo(i);

	if (MP3InfoArray.size() > 0)
		MP3InfoArray.erase(MP3InfoArray.begin(), MP3InfoArray.end());
}


int MP3MultiInfo::ReplaceInfo(int index, CMP3Info *mp3i)
{
	// Affecte le numero de serie du volume
	DWORD dwSerial = m_DriveInfoArray[m_CurrentDriveIndex]->dwSerialNumber;
	mp3i->SetDriveVolumeSerialNumber(dwSerial); 

	if (MP3InfoArray[index]) delete MP3InfoArray[index];
	MP3InfoArray[index] = mp3i;
	return(index);
}


int MP3MultiInfo::FindInfo(CMP3Info *mp3i)
{
	for (int i=0; i<MP3InfoArray.size(); i++)
	{
		if (MP3InfoArray[i] == mp3i)
			return(i);
	}
	return -1;
}


int MP3MultiInfo::AddInfo(CMP3Info *mp3i)
{
	// Affecte le numero de serie du volume
	int nMax = m_DriveInfoArray.size();
	
	DWORD dwSerial =  0; 

	if (m_CurrentDriveIndex >-1 && m_CurrentDriveIndex <nMax)
	{
		DRIVEVOLUMESTRUCT *dvs = NULL;
		
		dvs = m_DriveInfoArray[m_CurrentDriveIndex];
		if (dvs)
		{
			dwSerial= dvs->dwSerialNumber ;

			(dvs->nCountFile)++;
		}
	}
	
	// Affecte le numero de serie du volume
	mp3i->SetDriveVolumeSerialNumber(dwSerial); 
	
	int pos = AddInfoQuick(mp3i);
	return(pos);
}


int MP3MultiInfo::AddInfoQuick(CMP3Info *mp3i)
{
	// Appel depuis CDlgPlaylist::LoadEZ3
	// pas besoin de recherche le volume correspondant
	// charger et increment avant
	MP3InfoArray.push_back(mp3i);
	int pos = MP3InfoArray.size()-1;
	return(pos);
}
// //////////////////////////////////////////////////////////////
//           FUNCTIONS TO ACCES DRIVE VOLUME INFO ARRAY
// //////////////////////////////////////////////////////////////

// ************************************************************** 
DRIVEVOLUMESTRUCT* MP3MultiInfo::GetInfoDriveVolume(int index)
{
	DRIVEVOLUMESTRUCT* dvs=NULL;
	
	if(index < m_DriveInfoArray.size() )
		dvs = m_DriveInfoArray[index];

	return dvs;
	
}

// ************************************************************** 
void MP3MultiInfo::DeleteInfoDriveVolume(int index)
{
	// remove DriveVolume info's structure
	free (GetInfoDriveVolume(index) );

	// remove from array
	m_DriveInfoArray.erase(m_DriveInfoArray.begin() + index);
}

// ************************************************************** 
void MP3MultiInfo::ResetListDriveVolume()
{
	// Supprime les strucut info volume
	for (int i=0; i<m_DriveInfoArray.size(); i++)
		free(m_DriveInfoArray[i]);

	// Vide le vector
	m_DriveInfoArray.clear();
}

// **************************************************************
int MP3MultiInfo::ReplaceInfoDriveVolume(int index, DRIVEVOLUMESTRUCT *dvs)
{
	if (m_DriveInfoArray[index]) 
		free( m_DriveInfoArray[index]);
	
	m_DriveInfoArray[index] = dvs;
	return(index);
}

// **************************************************************
int MP3MultiInfo::AddInfoDriveVolume(TCHAR *szVolumeDrive)
{
	DRIVEVOLUMESTRUCT dvs;
	int pos;
	
	ZeroMemory(&dvs, sizeof(DRIVEVOLUMESTRUCT) );

	if ( SetDriveVolumeStruct(szVolumeDrive, &dvs) == FALSE)
		return -1;
	
	
	// add into the array. since this is sorted, find the right place for the insertion
	pos = FindInfoDriveVolume(dvs.dwSerialNumber);


	if(pos == -1)
	{
		DRIVEVOLUMESTRUCT *dvsNew;

		// ajoute les infos dans le vecteur
		dvsNew = (DRIVEVOLUMESTRUCT *)malloc(sizeof(DRIVEVOLUMESTRUCT));
		memcpy(dvsNew, &dvs, sizeof(DRIVEVOLUMESTRUCT));
		dvsNew->nCountFile       = 0;
		dvsNew->szDescription[0] = '\0';
		m_DriveInfoArray.push_back(dvsNew);
	}
		
	pos = m_DriveInfoArray.size()-1;

	// the new current DriveVolume index
	m_CurrentDriveIndex =  pos;

	return pos;
}

// ************************************************************** 
int MP3MultiInfo::FindInfoDriveVolume(DWORD dwSerial)
{
	int size = m_DriveInfoArray.size();

	for (int i=0; i<size; i++)
	{
		if (m_DriveInfoArray[i]->dwSerialNumber == dwSerial)
			return i;
	}
	
	return -1;
}

// ************************************************************** 
BOOL MP3MultiInfo::DeleteInfoDriveVolume(DWORD dwSerial)
{
	int pos = FindInfoDriveVolume(dwSerial);

	if (pos == -1)
		return FALSE;

	free(m_DriveInfoArray[pos]);
	m_DriveInfoArray.erase(m_DriveInfoArray.begin() + pos);

	return TRUE;
}


// ************************************************************** 
TCHAR* MP3MultiInfo::GetInfoDriveName(CMP3Info *mp3i)
{
	DWORD dwSerial = mp3i->GetDriveVolumeSerialNumber();
	int indexDrive = FindInfoDriveVolume(dwSerial);

	if (indexDrive != -1)
	{
		DRIVEVOLUMESTRUCT *dvs;
		dvs = GetInfoDriveVolume(indexDrive);

		if (dvs)
		{
			return dvs->szName; 
		}
	}

	return NULL;
}

// ************************************************************** 
BOOL MP3MultiInfo::SetDriveVolumeStruct(TCHAR *szVolumeDrive, DRIVEVOLUMESTRUCT *dvs)
{
	UINT errmode; 
	TCHAR szRoot[MAX_PATH]; 

	errmode = SetErrorMode(SEM_FAILCRITICALERRORS);  
	
	_tcsncpy(szRoot , szVolumeDrive, 3);
	szRoot[3] = 0; 

	// Si c'est un chemin reseau "\\ComputerName\ShareFolderName\xxxx"
	if (_tcsncmp(szVolumeDrive, _T("\\\\"), 2) ==0)
	{
		TCHAR szNetwork[MAX_PATH];
		TCHAR *pSlash, *pStart;

		_tcscpy(szNetwork, szVolumeDrive);

		pSlash = _tcsstr(szNetwork + 2, _T("\\"));
		pSlash = _tcsstr(pSlash + 2, _T("\\"));		// va au 2eme antislash
	
		pStart = szNetwork;

		_tcsncpy(szRoot, szNetwork,  pSlash - pStart);
		_tcscat(szRoot, _T("\\") );
		
	}
	else
	{	// type "C:\"
		_tcsncpy(szRoot , szVolumeDrive, 3);
		szRoot[3] = 0; 
	}
	
	dvs->nTypeMedia = GetDriveType(szRoot);

	_tcscpy(dvs->szRootDrive, szRoot);
	
	SetErrorMode(errmode); 

	if( dvs->nTypeMedia != DRIVE_FIXED)
	{
		if (CheckRemoveableReady(szRoot) == FALSE)
			return FALSE;
		else
			if( !GetVolumeInformation( szRoot, dvs->szName, (DWORD)255, &dvs->dwSerialNumber,	NULL, NULL, NULL, NULL ) )
				return FALSE; 

	}
	else
	{
		if( !GetVolumeInformation( szRoot, dvs->szName, (DWORD)255, &dvs->dwSerialNumber,	NULL, NULL, NULL, NULL ) )
			return FALSE; 
	}

	return TRUE; 
} 


/****************************************************************************
*
*    FUNCTION: CheckRemoveableReady(LPTSTR)
*
*    PURPOSE: Verifies that a removeable media drive contains a disk
*
*    COMMENTS:
*
*        This function is called each time a drive type is determined to be
*        removeable (DRIVE_REMOVEABLE).  An attempt is made to open a
*        file in the root directory of the drive.  If the attempt succeeds,
*        then media is present in the drive and subsequent calls to the
*        drive can be safely made.  If the call fails, then there is no media
*        present in the drive and no attempts should be made to access this
*        drive.  The Error Mode is temporarily set to 1 to allow failures
*        to immediately return to the calling program.  This eliminates
*        unwanted dialog boxes that prompt for disks to be placed in the
*        drive.
*
*    INPUT: szDriveName - removeable media drive name (ex - "a:")
*
*    OUTPUT: Returns TRUE if media present
*                    FALSE if media is not present
*
****************************************************************************/
BOOL MP3MultiInfo::CheckRemoveableReady( LPTSTR lpszDriveName )
{
    TCHAR szFileName[MAX_PATH];
	DWORD dwHold;
 
    SetErrorMode( SEM_FAILCRITICALERRORS );

    lstrcpy( szFileName, lpszDriveName );
    lstrcat( szFileName, TEXT(".") );

    dwHold = GetFileAttributes( szFileName );

    SetErrorMode( 0 );

    //  If no error, media must be in drive.
    if( dwHold != 0xFFFFFFFF )
        return(TRUE);
    else
        return(FALSE);
}



// ************************************************************** 
DRIVEVOLUMESTRUCT* MP3MultiInfo::GetFileEZ3(int index)
{
	return (GetInfoDriveVolume(index));
}

// ************************************************************** 
void MP3MultiInfo::SetFileEZ3(DRIVEVOLUMESTRUCT *dvs)
{
	DRIVEVOLUMESTRUCT *dvsNew;

	// ajoute les infos dans le vecteur
	dvsNew = (DRIVEVOLUMESTRUCT *)malloc(sizeof(DRIVEVOLUMESTRUCT));
	memcpy(dvsNew, dvs, sizeof(DRIVEVOLUMESTRUCT));
	m_DriveInfoArray.push_back(dvsNew);
}






