// systray.c
#include "priv.h"

#include "gen.h"
#include "resource.h"
#include "definition.h"
#include "Volume.h"
#include "Dlg.h"
#include "systray.h"
#include "Util.h"
#include "Language.h"
#include "settings.h"

#define SYSTRAY_ICON_BASE 1024

// internal declaration function
void SysTray_Init();
void SysTray_Destroy();
BOOL SysTray_Add(HWND hwnd, UINT uID, HICON hIcon, TCHAR* lpszTip);
BOOL SysTray_Del(HWND hwnd, UINT uID);
LRESULT CALLBACK SysTray_WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

void winamp_previous_track(HWND hwnd);
void winamp_play_pause(HWND hwnd);
void winamp_stop(HWND hwnd);
void winamp_next_track(HWND hwnd);
void winamp_stop(HWND hwnd);
void winamp_openfile(HWND hwnd);
void winamp_volume_up(HWND hwnd);
void winamp_volume_down(HWND hwnd);
BOOL ezplaylist_init(HANDLE hInstance);

BOOL CALLBACK ConfigProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);


// Declaration variables MENU in taskbar
#define IDM_SYSTRAY_EXTPLAYLIST     WM_USER + 100
#define IDM_SYSTRAY_PREVIOUSTRACK   WM_USER + 101
#define IDM_SYSTRAY_PLAYPAUSE       WM_USER + 102
#define IDM_SYSTRAY_STOP            WM_USER + 103
#define IDM_SYSTRAY_NEXTTRACK       WM_USER + 104
#define IDM_SYSTRAY_LOADFILE        WM_USER + 105
#define IDM_SYSTRAY_VOLUME          WM_USER + 106




#ifdef STRICT 
 WNDPROC lpWndProcOld;	
#else 
 FARPROC lpWndProcOld;		
#endif 

// Global declaration
HWND ghWndMain = NULL;
CLanguage    *lng = NULL;
CSetting     *setting = NULL;
CDlgPlaylist *dlg = NULL;
HICON Icons[NUM_ICONSYSTRAY];
TCHAR *tips[NUM_ICONSYSTRAY];
INITSTRUCT gParam;


// External declaration main.cpp
extern winampGeneralPurposePlugin plugin;
extern TCHAR szAppName[];
extern int systray_enabled;


// ****************************************************************
void SysTray_Init()
{
	
	// Lit les parametres dans le fichier 'gen-EZPlaylist.ini'
	setting = new CSetting;
	if (setting)
		setting->Init();

	// Initialise la gestion multi-langue	
	lng = new CLanguage;
	if(lng)
		lng->Start(gParam.szLanguage);

	tips[0]= GS(LNG_TIPS_EXTPLAYLIST);
	tips[1]= GS(LNG_TIPS_PREVIOUSTRACK);
	tips[2]= GS(LNG_TIPS_PLAYPAUSE);
	tips[3]= GS(LNG_TIPS_STOP);
	tips[4]= GS(LNG_TIPS_NEXTTRACK);
	tips[5]= GS(LNG_TIPS_LOADFILE);
	tips[6]= GS(LNG_TIPS_VOLUME);
	tips[7]= GS(LNG_TIPS_EZMENU);
	
	for (int i = 0; i < NUM_ICONSYSTRAY; i ++)
		Icons[i] = LoadIcon(plugin.hDllInstance,MAKEINTRESOURCE(IDI_ICON0+i));
	
	SysTray_Icons();


	// /////////////////////////////
	// Hook procedure SysTrayWndProc 
	// /////////////////////////////
#ifdef STRICT 
	lpWndProcOld = (WNDPROC)GetWindowLong(plugin.hwndParent,GWL_WNDPROC);
#else 
	lpWndProcOld = (FARPROC)GetWindowLong(plugin.hwndParent,GWL_WNDPROC);
#endif 
	SetWindowLong(plugin.hwndParent, GWL_WNDPROC, (LONG)SysTray_WndProc);

}

// ************************************************************************
void SysTray_Destroy()
{
	// delete all icons in taskbar
	systray_enabled=0;
	SysTray_Icons();

	// clean up Dialogbox EZ-playlist
	DestroyWindow(ghWndMain); 
	UnregisterClass(szAppName, plugin.hDllInstance);

	// Sauvegarde les parametres dans le fichier 'gen-EZPlaylist.ini'
	if (setting != NULL)
	{
		setting->Save();
		delete setting;
	}
	if (dlg != NULL) 
		delete dlg;

	if (lng != NULL)
		delete lng;
}

// ************************************************************************
void SysTray_Icons()
{
	static int l=0;
	int i;
	if (l == systray_enabled) return;

	for (i = NUM_ICONSYSTRAY-1; i >= 0; i --)
	{
		if (l & (1<<i))
			SysTray_Del(plugin.hwndParent,i);
	}
	
	l = systray_enabled;	
	for (i = 0; i < NUM_ICONSYSTRAY; i ++)
	{
		if (systray_enabled & (1<<i))
			SysTray_Add(plugin.hwndParent,i,Icons[i],tips[i]);
	}
}

// ****************************************************************
BOOL SysTray_Add(HWND hwnd, UINT uID, HICON hIcon, TCHAR* lpszTip)
{
  NOTIFYICONDATA tnid;

  ZeroMemory(&tnid, sizeof(tnid));

  tnid.cbSize = sizeof(NOTIFYICONDATA);
  tnid.hWnd   = hwnd;
  tnid.uID    = uID+SYSTRAY_ICON_BASE;
  tnid.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
  tnid.hIcon  = hIcon;
  tnid.uCallbackMessage = WM_USER + 27;
  _tcsncpy(tnid.szTip,lpszTip,sizeof(tnid.szTip)-1);
//  _tcscpy(tnid.szTip, lpszTip); en mode UNICODE
  
  return (Shell_NotifyIcon(NIM_ADD, &tnid));
}

// ****************************************************************
BOOL SysTray_Del(HWND hwnd, UINT uID) 
{
  NOTIFYICONDATA tnid;
  
  ZeroMemory(&tnid, sizeof(tnid));

  tnid.cbSize = sizeof(NOTIFYICONDATA);
  tnid.hWnd   = hwnd;
  tnid.uID    = uID+SYSTRAY_ICON_BASE;
  return(Shell_NotifyIcon(NIM_DELETE, &tnid));
}

// ******************************************************************
LRESULT CALLBACK SysTray_WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{

#define N_BORDER_SIZE	2
#define N_BITMAP_SIZE	16
	
	const int WMA_SYSTRAY = WM_USER+27;
	static int m_nFontHeight = 0;
	static HFONT m_menuFont = NULL;

	switch(message)
	{
		// WM_MEASUREITEM (OwnerDraw Menu with icon)		
		case WM_MEASUREITEM: 
		{
			 LPMEASUREITEMSTRUCT lpMeasureItemStruct = (LPMEASUREITEMSTRUCT) lParam;

			 if (lpMeasureItemStruct->CtlType != ODT_MENU)
				break;

			TCHAR  str[] = _T("Very Long String");
			HDC hDC = GetWindowDC(NULL);

			if (!(HFONT)m_menuFont) 
			{
				NONCLIENTMETRICS info;
				info.cbSize = sizeof(info);
				SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(info), &info, 0);
				m_menuFont = CreateFontIndirect(&info.lfMenuFont);
			}
			SelectObject (hDC, &m_menuFont);

			SIZE size;
			::GetTextExtentPoint32(hDC, str, _tcslen(str), &size);
			ReleaseDC(NULL, hDC);

			lpMeasureItemStruct->itemHeight = N_BORDER_SIZE + ((size.cy > N_BITMAP_SIZE) ? size.cy : N_BITMAP_SIZE) + N_BORDER_SIZE;    
			lpMeasureItemStruct->itemWidth  = 2*N_BORDER_SIZE + N_BITMAP_SIZE + 2*N_BORDER_SIZE + size.cx + N_BORDER_SIZE;    
			m_nFontHeight = size.cy;
			break;
		}

		// WM_DRAWITEM (OwnerDraw Menu with icon)
		case WM_DRAWITEM:
		{
			LPDRAWITEMSTRUCT lpDrawItemStruct = (LPDRAWITEMSTRUCT) lParam; 
			
			if (lpDrawItemStruct->CtlType != ODT_MENU)
				break;

			HDC hDC = lpDrawItemStruct->hDC;
			LPCTSTR text   = (LPCTSTR) lpDrawItemStruct->itemData;
			BOOL bSelected = (lpDrawItemStruct->itemState) & ODS_SELECTED;
			BOOL bGrayed   = (lpDrawItemStruct->itemState) & ODS_GRAYED;
			BOOL bChecked  = (lpDrawItemStruct->itemState) & ODS_CHECKED;

			COLORREF crBack;
			crBack = bSelected ? ::GetSysColor (COLOR_HIGHLIGHT) : ::GetSysColor (COLOR_MENU);
			SetBkColor (hDC,crBack);
			RECT rect;
			rect = lpDrawItemStruct->rcItem;
			rect.left += N_BORDER_SIZE + 2*N_BORDER_SIZE + N_BITMAP_SIZE;
			rect.right += N_BORDER_SIZE + 2*N_BORDER_SIZE + N_BITMAP_SIZE;
			::SetBkColor(hDC, crBack);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);
			POINT ptText;
			ptText.x = lpDrawItemStruct->rcItem.left + 2*N_BORDER_SIZE + N_BITMAP_SIZE + 2*N_BORDER_SIZE; 
			ptText.y = lpDrawItemStruct->rcItem.top + (lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top - m_nFontHeight)/2 - 1;

			SetBkMode(hDC, TRANSPARENT);
			int nLen = _tcslen(text);

			if (!bGrayed)
			{	if (bSelected)
				{	SetTextColor (hDC, ::GetSysColor (COLOR_HIGHLIGHTTEXT));
					TextOut (hDC, ptText.x, ptText.y, text, nLen);
				}
				else
				{	SetTextColor (hDC, ::GetSysColor (COLOR_MENUTEXT));
					TextOut (hDC, ptText.x, ptText.y, text, nLen);
				}
			}
			else
			{	if (bSelected)
				{	SetTextColor (hDC, GetSysColor(COLOR_MENU));
					TextOut (hDC, ptText.x, ptText.y, text, nLen);
				}
				else
				{	SetTextColor(hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
					TextOut (hDC, ptText.x + 1, ptText.y + 1, text, nLen);

					SetTextColor (hDC, GetSysColor(COLOR_GRAYTEXT));
					TextOut (hDC, ptText.x, ptText.y, text, nLen);
				}
			}

			POINT pt;
			pt.x = lpDrawItemStruct->rcItem.left + 2*N_BORDER_SIZE;
			pt.y = lpDrawItemStruct->rcItem.top + (lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top - N_BITMAP_SIZE)/2;

			UINT uIcon = IDI_EMPTY;

			switch(lpDrawItemStruct->itemID)
			{
				case IDM_SYSTRAY_EXTPLAYLIST:   uIcon = IDI_ICON0 ; break;
				case IDM_SYSTRAY_PREVIOUSTRACK: uIcon = IDI_ICON1 ; break;
				case IDM_SYSTRAY_PLAYPAUSE:     uIcon = IDI_ICON2 ; break;
				case IDM_SYSTRAY_STOP:          uIcon = IDI_ICON3 ; break;
				case IDM_SYSTRAY_NEXTTRACK:     uIcon = IDI_ICON4 ; break;
				case IDM_SYSTRAY_LOADFILE:      uIcon = IDI_ICON5 ; break;
			}
			HICON hIcon = NULL;
			
			hIcon = LoadIcon(plugin.hDllInstance, MAKEINTRESOURCE(uIcon));
			if(hIcon)	
			{	
				DrawIconEx (hDC, pt.x, pt.y, hIcon, N_BITMAP_SIZE, N_BITMAP_SIZE, NULL,	NULL, DI_NORMAL);
				DeleteObject (hIcon);
			}
			

			break;
		}

		case WM_DESTROY:
			if ((HFONT)m_menuFont) 
				DeleteObject(m_menuFont);		
			break;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
			// Via Popup Menu
			case IDM_SYSTRAY_EXTPLAYLIST:	ezplaylist_init(plugin.hDllInstance);	return 0;
			case IDM_SYSTRAY_PREVIOUSTRACK:	winamp_previous_track(hwnd);			return 0;
			case IDM_SYSTRAY_PLAYPAUSE:		winamp_play_pause(hwnd);				return 0;
			case IDM_SYSTRAY_STOP:			winamp_stop(hwnd);						return 0;
			case IDM_SYSTRAY_NEXTTRACK:		winamp_next_track(hwnd);				return 0;
			case IDM_SYSTRAY_LOADFILE:		winamp_openfile(hwnd);					return 0;
			}
			break;

		case WMA_SYSTRAY:
		{
			// Via Taskbar
			int which = LOWORD(wParam) - 1024;

			switch (LOWORD(lParam))
			{
				case WM_LBUTTONDOWN:
					switch (which)
					{
						case 0:	ezplaylist_init(plugin.hDllInstance);	return 0;
						case 1: winamp_previous_track(hwnd);			return 0;
						case 2: winamp_play_pause(hwnd);				return 0;
						case 3: winamp_stop(hwnd);						return 0;
						case 4: winamp_next_track(hwnd);				return 0;
						case 5: winamp_openfile(hwnd);					return 0;
						case 6:	winamp_volume_up(hwnd);					return 0;
						case 7: 
							{
							HMENU hMenu = CreatePopupMenu();
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_EXTPLAYLIST,   GS(LNG_TIPS_EXTPLAYLIST) ); 
							AppendMenu(hMenu, MF_SEPARATOR, NULL, _T(""));
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_PREVIOUSTRACK, GS(LNG_TIPS_PREVIOUSTRACK) );
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_PLAYPAUSE,     GS(LNG_TIPS_PLAYPAUSE) );
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_STOP,          GS(LNG_TIPS_STOP) );
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_NEXTTRACK,     GS(LNG_TIPS_NEXTTRACK) );
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_LOADFILE,      GS(LNG_TIPS_LOADFILE) );
						
							POINT pos;
							GetCursorPos(&pos);
							SetForegroundWindow(hwnd);  

							TrackPopupMenu (hMenu, 0, pos.x, pos.y,0,hwnd, NULL);
					
							// BUGFIX: See "PRB: Menus for Notification Icons Don't Work Correctly"
							PostMessage(hwnd, WM_NULL, 0, 0);
							DestroyMenu(hMenu);
							
							return 0;
							}

					}
					return 0;
			
				case WM_RBUTTONDOWN:
					switch (which)
					{
					case 0:	
					case 1:	
					case 2:	
					case 3:	
					case 4:	
					case 5:	
					case 7:	
						{
							SetForegroundWindow(hwnd);
							CDlgSysTray dlg;
							dlg.DoModal(IDD_DLGSYSTRAY, hwnd);
						}
						return 0;	
					case 6:	
						winamp_volume_down(hwnd);				
						return 0;
					}
					return 0;
			}
		}
	}

	return CallWindowProc(lpWndProcOld,hwnd,message,wParam,lParam);
	
}

/* ************************************************************************ */
BOOL ezplaylist_init(HANDLE hInstance)
{
	static bInit = FALSE;

	if (bInit == FALSE)
	{
		WNDCLASS  wc;
		// Fill in window class structure with parameters 
		// that describe the main window.                     
		memset(&wc,0,sizeof(wc));
		wc.lpfnWndProc = DefDlgProc;			// our window procedure
		wc.hInstance = (HINSTANCE)hInstance;	// hInstance of DLL
		wc.lpszClassName = szAppName;			// our window class name

		if (!RegisterClass(&wc)) 
			return (FALSE); 
		
		bInit= TRUE;
	}

	if (ghWndMain == NULL && dlg !=  NULL)
	{
		delete dlg;
		dlg = NULL;
	}

	if (ghWndMain != NULL)
	{
	    ShowWindow (ghWndMain, SW_SHOWNORMAL);
		UpdateWindow (ghWndMain);
		SetForegroundWindow(ghWndMain);
		return (TRUE);
	}
	
	// Perform initializations that apply to a specific instance 
	// Make the window visible; update its client area; and return "success" 

	dlg = new CDlgPlaylist;		

	if (dlg == NULL)
		return FALSE;

	ghWndMain = dlg->DoModeless(IDD_DLGEZPLAYLIST, plugin.hwndParent);

	// If window could not be created, return "failure"
	if (!ghWndMain)
	{
		delete dlg;
		dlg = NULL;
		return (FALSE);
	}

	// SetWindowPlacement provoque un ShowWindow
	SetWindowPlacement(ghWndMain, &gParam.wp);
	UpdateWindow (ghWndMain);
	SetForegroundWindow(ghWndMain);

	return TRUE;
}





// ****************** WRAPPER WINAMP FUNCTION ************************
// -----------------------------------------------
void winamp_previous_track(HWND hwnd)
{
	int a;

	// not playing, let's hit prev
	if ((a=SendMessage(hwnd,WM_USER,0,104)) == 0) 
		// Previous
		SendMessage(hwnd,WM_COMMAND,40044,0);
	else if (a != 3 && SendMessage(hwnd,WM_USER,0,105) > 2000) // restart
		//Play
		SendMessage(hwnd,WM_COMMAND,40045,0);
	else 
		// previous
		SendMessage(hwnd,WM_COMMAND,40044,0);
}
// -----------------------------------------------
void winamp_play_pause(HWND hwnd)
{
	int a;

	// not playing, let's hit play
	if ((a=SendMessage(hwnd,WM_USER,0,104)) != 1) 
		SendMessage(hwnd,WM_COMMAND,40045,0);
	else  // prev
		SendMessage(hwnd,WM_COMMAND,40046,0);
}	
// -----------------------------------------------
void winamp_stop(HWND hwnd)
{
	if (GetKeyState(VK_SHIFT) & (1<<15))
		SendMessage(hwnd,WM_COMMAND,40147,0);
	else
		SendMessage(hwnd,WM_COMMAND,40047,0);
}
// -----------------------------------------------
void winamp_next_track(HWND hwnd)
{
	SendMessage(hwnd,WM_COMMAND,40048,0);
}
// -----------------------------------------------
void winamp_openfile(HWND hwnd)
{
	SetForegroundWindow(hwnd);
	if (GetKeyState(VK_CONTROL) & (1<<15))
		SendMessage(hwnd,WM_COMMAND,40185,0);	// internet location
	else if (GetKeyState(VK_SHIFT) & (1<<15))
		SendMessage(hwnd,WM_COMMAND,40187,0);	// open directory
	else
		SendMessage(hwnd,WM_COMMAND,40029,0);	// open file
}
// -----------------------------------------------
void winamp_volume_up(HWND hwnd)
{
	SetForegroundWindow(hwnd);
	int k, nMax;
	if (GetKeyState(VK_CONTROL) & (1<<15))
		nMax = 20;
	else
		nMax = 2;

	for (k=0; k<nMax; k++)
		SendMessage(hwnd,WM_COMMAND,40058,0);
}
// -----------------------------------------------
void winamp_volume_down(HWND hwnd)
{
	SetForegroundWindow(hwnd);
	int k, nMax;
	if (GetKeyState(VK_CONTROL) & (1<<15))
		nMax = 20;
	else
		nMax = 2;

	for (k=0; k<nMax; k++)
		SendMessage(hwnd,WM_COMMAND,40059,0);
}

